/*******************************************************************************

    FILE: action.c
    AUTHOR: RPC (based on C++ by SVW)
    CREATION DATE: Thu Nov 28 13:18:07 2002
    CONTENTS: Generic action related code

*******************************************************************************/

#include "cs.h"

/******************************************************************************/

extern Schema *selected_parent(ContentionScheduler *cs, Schema *this);

/******************************************************************************/
/* Effector functions: what is in the hand? ***********************************/

static double holdsNothing(ContentionScheduler *cs, Effector *effector)
{
    return( effector->itemsHeld == NULL ? 1.0 : 0.0 );
}

static double holdsAnything(ContentionScheduler *cs, Effector *effector)
{
    return( effector->itemsHeld == NULL ? 0.0 : 1.0 );
}

static double holdsObjectWithRole(ContentionScheduler *cs, Effector *effector, ObjectRole role)
{
    if (effector->itemsHeld == NULL) {
        return(0.0);
    }
    else {
        ObjectList *tmp = effector->itemsHeld;
        double max = minAct;

        while (tmp != NULL) {
            max = MAX(max, tmp->head->activation[role][cs->cycle]);
            tmp = tmp->tail;
        }
        return(max);
    }
}

static double holdsImplement(ContentionScheduler *cs, Effector *effector)
{
    return(holdsObjectWithRole(cs, effector, IMPLEMENT));
}

static double holdsSource(ContentionScheduler *cs, Effector *effector)
{
    return(holdsObjectWithRole(cs, effector, SOURCE));
}

static double holdsTarget(ContentionScheduler *cs, Effector *effector)
{
    return(holdsObjectWithRole(cs, effector, TARGET));
}

static double holdsTheme(ContentionScheduler *cs, Effector *effector)
{
    return(holdsObjectWithRole(cs, effector, THEME));
}

static double holdsSourceOrTarget(ContentionScheduler *cs, Effector *effector)
{
    return(FZOR(holdsSource(cs, effector), holdsTarget(cs, effector)));
}

/******************************************************************************/

extern void action_log_add_warning(ContentionScheduler *cs, char *type, char *warning);

void errorObjectIncorrect(ContentionScheduler *cs, char *template, char *arg1, char *arg2, char *arg3)
{
    char buffer[256];

    g_snprintf(buffer, 256, template, arg1, arg2, arg3);
    action_log_add_warning(cs, "OBJECT ERROR", buffer);
    fprintf(stderr, "%5d\tOBJECT ERROR: %s\n", cs->cycle, buffer);
}

void errorEffectorIncorrect(ContentionScheduler *cs, char *template, char *e1n, char *e2n)
{
    char buffer[256];

    g_snprintf(buffer, 256, template, e1n, e2n);
    action_log_add_warning(cs, "EFFECTOR ERROR", buffer);
    fprintf(stderr, "%5d\tEFFECTOR ERROR: %s\n", cs->cycle, buffer);
}

void errorObjectState(ContentionScheduler *cs, char *template, char *arg1, char *arg2, char *arg3)
{
    char buffer[256];

    g_snprintf(buffer, 256, template, arg1, arg2, arg3);
    action_log_add_warning(cs, "OBJECT STATE ERROR", buffer);
    fprintf(stderr, "%5d\tSTATE ERROR: %s\n", cs->cycle, buffer);
}

void errorEffectorState(ContentionScheduler *cs, char *template, char *effector)
{
    char buffer[256];

    g_snprintf(buffer, 256, template, effector);
    action_log_add_warning(cs, "EFFECTOR STATE ERROR", buffer);
    fprintf(stderr, "%5d\tEFFECTOR STATE ERROR: %s\n", cs->cycle, buffer);
}

void errorObjectNull(ContentionScheduler *cs, char *role, char *action)
{
    char buffer[256];

    g_snprintf(buffer, 256, "%s of %s", role, action);
    action_log_add_warning(cs, "NULL OBJECT ERROR", buffer);
    fprintf(stderr, "%5d\tNULL OBJECT ERROR: %s\n", cs->cycle, buffer);
}

void errorEffectorNull(ContentionScheduler *cs, char *action)
{
    action_log_add_warning(cs, "NULL EFFECTOR ERROR", action);
    fprintf(stderr, "%5d\tNULL EFFECTOR ERROR: %s\n", cs->cycle, action);
}

void actionMessage(ContentionScheduler *cs, char *action, char *message)
{
    extern void action_log_add_action(ContentionScheduler *cs, char *act, char *message);


    if (messages[RUN] != NULL) {
        fprintf(messages[RUN], "%5d\t%s: %s\n", cs->cycle, action, message);
    }
    action_log_add_action(cs, action, message);
}

/******************************************************************************/
/* A local buffer used for actionMessage:                                     */

static char buffer[256]; 

/******************************************************************************/

void record_intervention(ContentionScheduler *cs, char *action)
{
    FILE *fp;

    if (cs->logfile != NULL) {
        if ((fp = fopen(cs->logfile, "a")) == NULL) {
            fprintf(stderr, "LOG WARNING: Cannot write to log file\n");
        }
        else {
            fprintf(fp, "intervention %s\n", action);
            fclose(fp);
        }
    }
}

void record_action(ContentionScheduler *cs, char *action, char *arg1, char *arg2, char *arg3)
{
    FILE *fp;

    if (cs->logfile != NULL) {
        if ((fp = fopen(cs->logfile, "a")) == NULL) {
            fprintf(stderr, "LOG WARNING: Cannot write to log file\n");
        }
        else {
            fprintf(fp, "%s", action);
            if (arg1 != NULL) { fprintf(fp, " %s", arg1); }
            if (arg2 != NULL) { fprintf(fp, " %s", arg2); }
            if (arg3 != NULL) { fprintf(fp, " %s", arg3); }
            fprintf(fp, "\n");
            fclose(fp);
        }
    }
}

/******************************************************************************/

static Object *mostActiveObjectOnNthCycle(int cycle, Object *o1, Object *o2, ObjectRole r)
{
    if (r == EMPTY) {
        fprintf(stderr, "WARNING: mostActiveObject() called with EMPTY ObjectRole: NULL returned\n");
        return(NULL);
    }
    else if (o1 == NULL) {
        return(o2);
    }
    else {
        return((o1->activation[r][cycle] > o2->activation[r][cycle])?o1:o2);
    }
}

/*----------------------------------------------------------------------------*/

static Object *bestObjectForRole(ContentionScheduler *cs, ObjectRole role)
{
    Object *best = NULL;
    ObjectList *os;

    for (os = cs->objects; os; os = os->tail) {
        best = mostActiveObjectOnNthCycle(cs->cycle, best, os->head, role);
    }

    return(best);
}

static Object *previousBestObjectForRole(ContentionScheduler *cs, ObjectRole role)
{
    Object *best = NULL;
    ObjectList *os;

    for (os = cs->objects; os; os = os->tail) {
        best = mostActiveObjectOnNthCycle(cs->cycle-1, best, os->head, role);
    }

    return(best);
}

Boolean fixatedObjectIsBestForFunction(ContentionScheduler *cs, ObjectRole role)
{
    return(cs->fixated_object == previousBestObjectForRole(cs, role));
}

/******************************************************************************/

static Effector *bestEffector(ContentionScheduler *cs, double effectorType(ContentionScheduler *, Effector *))
{
    Effector *best = NULL;
    EffectorList *es;
    double max = 0.0, this;

    for (es = cs->effectors; es; es = es->tail) {
        if ((this = effectorType(cs, es->head) * es->head->activation[cs->cycle]) > max) {
            max = this;
            best = es->head;
        }
    }
    /* If nothing found, look again without restrictions */
    if (best == NULL) {
        for (es = cs->effectors; es; es = es->tail) {
            if (es->head->activation[cs->cycle] > max) {
                best = es->head;
                max = best->activation[cs->cycle];
            }
        }
    }
    return(best);
}

/******************************************************************************/

static Effector *leastActiveEffector(ContentionScheduler *cs)
{
    EffectorList *el;
    Effector *worst = NULL;

    for (el = cs->effectors; el != NULL; el = el->tail) {
        if ((worst == NULL) || (worst->activation[cs->cycle] > el->head->activation[cs->cycle])) {
            worst = el->head;
        }
    }
    return(worst);
}

/*----------------------------------------------------------------------------*/

static Boolean usesObjectsHeldBy(ContentionScheduler *cs, Schema *this, Effector *effector)
{
    ObjectTriggerFunctionList *tf;
    ObjectList *os;

    for (os = effector->itemsHeld; os != NULL; os = os->tail) {
        for (tf = this->objectTriggers; tf != NULL; tf = tf->next) {
            if (tf->trigger(cs, os->head) > 0) {
                return(TRUE);
            }
        }
    }
    return(FALSE);
}

/*----------------------------------------------------------------------------*/

static Schema *selected_basic_schema(ContentionScheduler *cs)
{
    SchemaList *sl;

    for (sl = cs->schemas; sl != NULL; sl = sl->tail) {
        if (sl->head->selected && (sl->head->corresponding_action != ACTION_NONE)) {
            return(sl->head);
        }
    }
    return(NULL);
}

/*----------------------------------------------------------------------------*/

/* This function attempts to find a full hand holding something not useful    */
/* to the current schema's selected parent.                                   */

/* If the current schema has no selected parent, or both hands are empty,     */
/* or both hands are full and useful, this function returns the less active   */
/* hand.                                                                      */

/* If both hands are full and not useful to the selected parent, we           */
/* recursively search for usefulness to the selected grandparent etc.         */

static Effector *worstEffectorForSchema(ContentionScheduler *cs, Schema *schema)
{
    Effector *leftHand = cs->effectors->head;
    Boolean leftHandUseful = FALSE;
    Boolean leftHandFull = FALSE;
    Effector *rightHand = cs->effectors->tail->head;
    Boolean rightHandUseful = FALSE;
    Boolean rightHandFull = FALSE;
    Schema *parent;

    if ((schema != NULL) && ((parent = selected_parent(cs, schema)) != NULL)) {
        if (isFullEffectorBoolean(leftHand)) {
            leftHandFull = TRUE;

            if (usesObjectsHeldBy(cs, parent, leftHand)) {
                leftHandUseful = TRUE;
            }
        }

        if (isFullEffectorBoolean(rightHand)) {
            rightHandFull = TRUE;

            if (usesObjectsHeldBy(cs, parent, rightHand)) {
                rightHandUseful = TRUE;
            }
        }

        if (leftHandFull && rightHandFull && !leftHandUseful && !rightHandUseful) {
            return(worstEffectorForSchema(cs, parent));
        }
        else if (leftHandFull && !leftHandUseful) {
            return(leftHand);
        }
        else if (rightHandFull && !rightHandUseful) {
            return(rightHand);
        }
    }
    return(leastActiveEffector(cs));
}

Effector *effectorGetWorst(ContentionScheduler *cs)
{
    return(worstEffectorForSchema(cs, selected_basic_schema(cs)));
}

/******************************************************************************/

/* This function is called during action execution to bind an object to a     */
/* functional role.                                                           */

void objectBindToRole(ContentionScheduler *cs, Object *object, ObjectRole role)
{
    char buffer[128];

    if (role == EMPTY) {
        g_snprintf(buffer, 128, "BIND ERROR: Attempt to bind %s to invalid role (%s)", object->name, roleName[role]);
        message(BIND, cs, buffer);
    }
    else if (cs->current_object[role] == NULL) {
        object->inhibit[role] = FALSE; /* Just in case ... */
        cs->current_object[role] = object;
        g_snprintf(buffer, 128, "Binding %s as %s", object->name, roleName[role]);
        message(BIND, cs, buffer);
    }
    else if (cs->current_object[role] != object) {
        g_snprintf(buffer, 128, "BIND ERROR: Cannot bind %s as %s because %s is already bound to that role", object->name, roleName[role], cs->current_object[role]->name);
        message(BIND, cs, buffer);
    }
}

/*************** effectorDiscardContents(HandContainingObject) ****************/

/* When an object is discarded the hand no longer holds the object and the    */
/* object's position changes to the position where the object was placed.     */
/* effectorDiscardContents only takes place if the selected hand is non-NULL  */
/* and full, ie no error is registered otherwise.                             */

void effectorDiscardContents(ContentionScheduler *cs, Effector *effector)
{
    Object *theme = NULL;

    if ((effector != NULL) && (effectorGetContents(effector) != NULL)) {
        theme = effectorGetContents(effector);
        g_snprintf(buffer, 256, "Discard %s with %s", theme->name, effector->name);
        actionMessage(cs, "Discard", buffer);
        record_action(cs, "Discard", theme->name, effector->name, NULL);

        traceEffectorPreAction(cs, effector);
        effector->itemsHeld = objectListDeleteElement(effector->itemsHeld, theme);
        traceEffectorPostAction(cs, effector);

        traceObjectPreAction(cs, theme);
        objectSetPosition(theme, effectorDiscardCoordinate(effector), 1);
        traceObjectPostAction(cs, theme);
    }
}

/******************************************************************************/
/******************** pickUp(ObjectWithinReach, EmptyHand) ********************/

/* Pick up should fail if the object is out of reach.                         */

void pickUp(ContentionScheduler *cs, Object *theme, Effector *effector)
{
    Object *tmp;

    if (((tmp = objectGetAttachment(theme)) != NULL) && isHeldBoolean(tmp)) {
        /* If part of theme is already held, then we need to discard that */
        /* part first...                                                  */
        effectorDiscardContents(cs, tmp->holder);
    }

    if ((effector == NULL) || isFullEffectorBoolean(effector)) {
        effector = effectorGetWorst(cs);
        effectorDiscardContents(cs, effector);
    }

    if (theme == NULL) {
        errorObjectNull(cs, "Theme", "PickUp");
    }
    else if (isTable(theme) > 0.9) {
        errorObjectState(cs, "%s: desired object (%s) cannot be picked up", "PickUp", theme->name, NULL);
    }
    else if (isHeldBoolean(theme)) {
        errorObjectState(cs, "%s: desired object (%s) is already held by %s", "PickUp", theme->name, theme->holder->name);
    }
    else if (isContainedBoolean(theme) && isClosedBoolean(theme->container)) {
        errorObjectState(cs, "%s: desired object (%s) is inside %s which is closed", "PickUp", theme->name, theme->container->name);
    }
    else {
        g_snprintf(buffer, 256, "Picking up %s with %s", theme->name, effector->name);
        actionMessage(cs, "PickUp", buffer);
        record_action(cs, "PickUp", theme->name, effector->name, NULL);
        
        traceEffectorPreAction(cs, effector);
        effectorAddItemToContents(effector, theme);
        traceEffectorPostAction(cs, effector);

        traceObjectPreAction(cs, theme);
        objectSetPosition(theme, 0.0, 0.0);
        theme->holder = effector;
        traceObjectPostAction(cs, theme);

        if (isContainedBoolean(theme)) {
            Object *tmp = theme->container;
            traceObjectPreAction(cs, tmp);
            objectDeleteContained(tmp, theme);
            traceObjectPostAction(cs, tmp);
        }

        if (isCarriedBoolean(theme)) {
            Object *tmp = theme->carrier;
            traceObjectPreAction(cs, tmp);
            objectDeleteCarried(tmp, theme);
            traceObjectPostAction(cs, tmp);
        }

        if ((theme->base != NULL) && isContainedBoolean(theme->base)) {
            /* Picking up something by its attachment ... */
            Object *tmp = theme->base->container;
            traceObjectPreAction(cs, tmp);
            objectDeleteContained(tmp, theme->base);
            traceObjectPostAction(cs, tmp);
        }

        if ((theme->base != NULL) && isCarriedBoolean(theme->base)) {
            /* Picking up something by its attachment ... */
            Object *tmp = theme->base->carrier;
            traceObjectPreAction(cs, tmp);
            objectDeleteCarried(tmp, theme->base);
            traceObjectPostAction(cs, tmp);
        }

        if (theme->itemsAttached != NULL) {
            ObjectList *tmp;
            for (tmp = theme->itemsAttached; tmp != NULL; tmp = tmp->tail) {
                if (isContainedBoolean(tmp->head)) {
                    Object *tmp2 = tmp->head->container;
                    traceObjectPreAction(cs, tmp2);
                    objectDeleteContained(tmp2, tmp->head);
                    traceObjectPostAction(cs, tmp2);
                }
                if (isCarriedBoolean(tmp->head)) {
                    Object *tmp2 = tmp->head->carrier;
                    traceObjectPreAction(cs, tmp2);
                    objectDeleteCarried(tmp2, tmp->head);
                    traceObjectPostAction(cs, tmp2);
                }
            }
        }
    }
}

/******************************************************************************/
/******************** fixate(Object, Role) ********************/

/* Fixate reorients attention to the specified object.           */

void fixate(ContentionScheduler *cs, Object *theme, ObjectRole role)
{
    if (theme == NULL) {
        errorObjectNull(cs, "Theme", "Fixate");
    }
    else {
        g_snprintf(buffer, 256, "Fixating %s", theme->name);
        actionMessage(cs, "Fixate", buffer);
        record_action(cs, "Fixate", theme->name, NULL, NULL);
        
        traceObjectPreAction(cs, theme);
        cs->fixated_object = theme;
        traceObjectPostAction(cs, theme);

        objectBindToRole(cs, theme, role);
    }
}

/******************************************************************************/
/************************ putDown(HandContainingObject) ***********************/

/* When an object is put down the hand no longer holds the object and the     */
/* object's position changes to the position where the object was placed.     */
/* This version of putDown relinquishes all objects in the given effector.    */

void putDown(ContentionScheduler *cs, Effector *effector)
{
    Object *theme = NULL;

    if (effector == NULL) {
        errorEffectorNull(cs, "PutDown");
    }
    else if (!isFullEffectorBoolean(effector)) {
        errorEffectorState(cs, "PutDown: effector (%s) is empty", effector->name);
    }
    else {
        while ((theme = effectorGetContents(effector)) != NULL) {
            g_snprintf(buffer, 256, "Putting down %s with %s", theme->name, effector->name);
            actionMessage(cs, "PutDown", buffer);
            record_action(cs, "PutDown", theme->name, effector->name, NULL);

            traceEffectorPreAction(cs, effector);
            effector->itemsHeld = objectListDeleteElement(effector->itemsHeld, theme);
            traceEffectorPostAction(cs, effector);

            traceObjectPreAction(cs, theme);
            objectSetPosition(theme, effectorDiscardCoordinate(effector), 1);
            traceObjectPostAction(cs, theme);
        }
    }
}

/******************************************************************************/

extern void screwOpen(ContentionScheduler *cs, Effector *jarHand, Effector *lidHand);
extern void screwClosed(ContentionScheduler *cs, Effector *jarHand, Effector *lidHand);
extern void seal(ContentionScheduler *cs, Effector *bagHand, Effector *emptyHand);
extern void unseal(ContentionScheduler *cs, Effector *bagHand, Effector *emptyHand);
extern void cap_thermos(ContentionScheduler *cs, Object *target, Effector *cupHand);
extern void takeFrom(ContentionScheduler *cs, Object *source, Effector *effector);
extern void takeAnotherFrom(ContentionScheduler *cs, Object *source, Effector *effector);
extern void pour(ContentionScheduler *cs, Object *target, Effector *effector);
extern void dip(ContentionScheduler *cs, Object *source, Effector *effector);
extern void spread(ContentionScheduler *cs, Object *target, Effector *effector);
extern void tear(ContentionScheduler *cs, Effector *holdHand, Effector *tearHand);
extern void wrap(ContentionScheduler *cs, Object *target, Effector *effector);
extern void pack(ContentionScheduler *cs, Object *target, Effector *effector);
extern void freeHands(ContentionScheduler *cs);
extern void chooseSlice(ContentionScheduler *cs, Object *slice);
extern void swap(ContentionScheduler *cs, Effector *sourceHand, Effector *targetHand);
extern void openLid(ContentionScheduler *cs, Object *target, Effector *effector);
extern void closeLid(ContentionScheduler *cs, Object *target, Effector *effector);
extern void flip(ContentionScheduler *cs, Object *theme, Effector *effector);
extern void poke(ContentionScheduler *cs, Object *theme, Effector *effector);
extern void eat(ContentionScheduler *cs, Effector *effector);
extern void fold(ContentionScheduler *cs, Object *theme, Effector *effector);

extern void place_stamp(ContentionScheduler *cs, Effector *effector);
extern void write_address(ContentionScheduler *cs, Effector *effector);
extern void fold_letter(ContentionScheduler *cs, Effector *effector);
extern void pack_envelope(ContentionScheduler *cs, Effector *effector1, Effector *effector2);
extern void seal_envelope(ContentionScheduler *cs, Effector *effector);
extern void moisten_by_licking(ContentionScheduler *cs, Effector *effector);
extern void tuck_flap(ContentionScheduler *cs, Effector *effector);

extern void fill_scoop(ContentionScheduler *cs, Effector *effector);
extern void empty_scoop(ContentionScheduler *cs, Effector *effector);
extern void pour_some(ContentionScheduler *cs, Effector *effector);
extern void screwPotOpen(ContentionScheduler *cs, Effector *jarHand, Effector *lidHand);
extern void screwPotClosed(ContentionScheduler *cs, Effector *jarHand, Effector *lidHand);

extern void cut_by_sawing(ContentionScheduler *cs, Effector *effector);
extern void cut_by_pushing(ContentionScheduler *cs, Effector *effector);
extern void squeeze_by_twisting(ContentionScheduler *cs, Effector *effector);
extern void squeeze_by_pushing(ContentionScheduler *cs, Effector *effector);
extern void squeeze_by_hand(ContentionScheduler *cs, Effector *effector);
extern void pour_all(ContentionScheduler *cs, Effector *effector);
extern void cover(ContentionScheduler *cs, Effector *effector);
extern void bounce(ContentionScheduler *cs, Effector *effector);

extern void candle_into_holder(ContentionScheduler *cs, Effector *effector);
extern void slide_open(ContentionScheduler *cs, Effector *boxHand);
extern void slide_closed(ContentionScheduler *cs, Effector *boxHand);
extern void strike_match(ContentionScheduler *cs, Effector *matchHand);
extern void match_to_candle(ContentionScheduler *cs, Effector *effector);
extern void blow_out_match(ContentionScheduler *cs);
extern void tear_open(ContentionScheduler *cs, Effector *boxHand);
extern void peel_open(ContentionScheduler *cs, Effector *boxHand);

extern void drink(ContentionScheduler *cs, Effector *effector);
extern void uncork_bottle(ContentionScheduler *cs, Effector *effector);
extern void recork_bottle(ContentionScheduler *cs, Effector *effector);

/*----------------------------------------------------------------------------*/

void execute_action(ContentionScheduler *cs, Schema *schema)
{
    Object *obj1 = NULL;

    switch(schema->corresponding_action) {

    case ACTION_NONE:
        fprintf(stderr, "WARNING: Attempting to execute non-primitive action!\n");
        break;
    case ACTION_FIXATE_SOURCE:
        fixate(cs, obj1 = bestObjectForRole(cs, SOURCE), SOURCE);
        break;
    case ACTION_FIXATE_TARGET:
        fixate(cs, obj1 = bestObjectForRole(cs, TARGET), TARGET);
        break;
    case ACTION_FIXATE_THEME:
        fixate(cs, obj1 = bestObjectForRole(cs, THEME), THEME);
        break;
    case ACTION_FIXATE_IMPLEMENT:
        fixate(cs, obj1 = bestObjectForRole(cs, IMPLEMENT), IMPLEMENT);
        break;
    case ACTION_PICK_UP:
        pickUp(cs, obj1 = cs->fixated_object, bestEffector(cs, holdsNothing));
        break;
    case ACTION_PUT_DOWN:
        putDown(cs, bestEffector(cs, holdsAnything));
        break;
#ifdef LUNCH_TASK
    case ACTION_SCREW_OPEN:
        screwOpen(cs, bestEffector(cs, holdsSourceOrTarget), bestEffector(cs, holdsNothing));
        break;
    case ACTION_SCREW_CLOSED:
        screwClosed(cs, bestEffector(cs, holdsSourceOrTarget), bestEffector(cs, holdsTheme));
        break;
    case ACTION_UNSEAL:
        unseal(cs, bestEffector(cs, holdsAnything), bestEffector(cs, holdsNothing));
        break;
    case ACTION_SEAL:
        seal(cs, bestEffector(cs, holdsAnything), bestEffector(cs, holdsNothing));
        break;
    case ACTION_CAP:
        cap_thermos(cs, obj1 = bestObjectForRole(cs, TARGET), bestEffector(cs, holdsTheme));
        break;
    case ACTION_TAKE_FROM:
        takeFrom(cs, obj1 = bestObjectForRole(cs, SOURCE), bestEffector(cs, holdsNothing));
        break;
    case ACTION_TAKE2_FROM:
        takeFrom(cs, obj1 = bestObjectForRole(cs, SOURCE), bestEffector(cs, holdsNothing));
        takeAnotherFrom(cs, obj1, effector1);
        break;
    case ACTION_POUR_SOME:
        pour(cs, obj1 = bestObjectForRole(cs, TARGET), bestEffector(cs, holdsSource));
        break;
    case ACTION_DIP:
        dip(cs, obj1 = bestObjectForRole(cs, SOURCE), bestEffector(cs, holdsImplement));
        break;
    case ACTION_SPREAD:
        spread(cs, obj1 = bestObjectForRole(cs, TARGET), bestEffector(cs, holdsImplement));
        break;
    case ACTION_TEAR:
        tear(cs, bestEffector(cs, holdsAnything), bestEffector(cs, holdsNothing));
        break;
    case ACTION_WRAP:
        wrap(cs, obj1 = bestObjectForRole(cs, TARGET), bestEffector(cs, holdsNothing));
        break;
    case ACTION_PACK:
        pack(cs, obj1 = bestObjectForRole(cs, TARGET), bestEffector(cs, holdsAnything));
        break;
    case ACTION_CHOOSE_SLICE:
        chooseSlice(cs, obj1 = bestObjectForRole(cs, TARGET));
        break;
    case ACTION_FREE_HANDS:
        freeHands(cs);
        break;
    case ACTION_SWAP:
        swap(cs, bestEffector(cs, holdsAnything), bestEffector(cs, holdsNothing));
        break;
    case ACTION_OPEN_LID:
        openLid(cs, obj1 = bestObjectForRole(cs, TARGET), bestEffector(cs, holdsNothing));
        break;
    case ACTION_CLOSE_LID:
        closeLid(cs, obj1 = bestObjectForRole(cs, TARGET), bestEffector(cs, holdsNothing));
        break;
    case ACTION_FLIP:
        flip(cs, obj1 = bestObjectForRole(cs, THEME), bestEffector(cs, holdsImplement));
        break;
    case ACTION_POKE:
        poke(cs, obj1 = bestObjectForRole(cs, THEME), bestEffector(cs, holdsImplement));
        break;
    case ACTION_EAT:
        eat(cs, bestEffector(cs, holdsAnything));
        break;
    case ACTION_FOLD:
        fold(cs, obj1 = bestObjectForRole(cs, THEME), bestEffector(cs, holdsNothing));
        break;
#endif
    case ACTION_PLACE_STAMP:
        place_stamp(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_WRITE_ADDRESS:
        write_address(cs, bestEffector(cs, holdsImplement));
        break;
    case ACTION_FOLD_LETTER:
        fold_letter(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_PACK_ENVELOPE:
        pack_envelope(cs, bestEffector(cs, holdsTarget), bestEffector(cs, holdsTheme));
        break;
    case ACTION_SEAL_ENVELOPE:
        seal_envelope(cs, bestEffector(cs, holdsTarget));
        break;
    case ACTION_MOISTEN_WITH_TOUNGE:
        moisten_by_licking(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_TUCK:
        tuck_flap(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_CUT_BY_SAWING:
        cut_by_sawing(cs, bestEffector(cs, holdsImplement));
        break;
    case ACTION_CUT_BY_PUSHING:
        cut_by_pushing(cs, bestEffector(cs, holdsImplement));
        break;
    case ACTION_SQUEEZE_BY_TWISTING:
        squeeze_by_twisting(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_SQUEEZE_BY_PUSHING:
        squeeze_by_pushing(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_SQUEEZE_BY_HAND:
        squeeze_by_hand(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_POUR_ALL:
        pour_all(cs, bestEffector(cs, holdsSource));
        break;
    case ACTION_POUR_SOME:
        pour_some(cs, bestEffector(cs, holdsSource));
        break;
    case ACTION_COVER:
        cover(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_SCOOP:
        fill_scoop(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_EMPTY_SCOOP:
        empty_scoop(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_SCREW_POT_OPEN:
        screwPotOpen(cs, bestEffector(cs, holdsSourceOrTarget), bestEffector(cs, holdsNothing));
        break;
    case ACTION_SCREW_POT_CLOSED:
        screwPotClosed(cs, bestEffector(cs, holdsSourceOrTarget), bestEffector(cs, holdsTheme));
        break;
    case ACTION_INSERT_CANDLE:
        candle_into_holder(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_SLIDE_OPEN:
        slide_open(cs, bestEffector(cs, holdsSource));
        break;
    case ACTION_SLIDE_CLOSED:
        slide_closed(cs, bestEffector(cs, holdsSource));
        break;
    case ACTION_TEAR_OPEN:
        tear_open(cs, bestEffector(cs, holdsSource));
        break;
    case ACTION_PEEL_OPEN:
        peel_open(cs, bestEffector(cs, holdsSource));
        break;
    case ACTION_STRIKE_MATCH:
        strike_match(cs, bestEffector(cs, holdsImplement));
        break;
    case ACTION_MATCH_TO_CANDLE:
        match_to_candle(cs, bestEffector(cs, holdsImplement));
        break;
    case ACTION_BLOW_OUT_MATCH:
        blow_out_match(cs);
        break;
    case ACTION_DRINK:
        drink(cs, bestEffector(cs, holdsSource));
        break;
    case ACTION_UNCORK:
        uncork_bottle(cs, bestEffector(cs, holdsImplement));
        break;
    case ACTION_RECORK:
        recork_bottle(cs, bestEffector(cs, holdsTheme));
        break;
    case ACTION_BOUNCE_BALL:
        bounce(cs, bestEffector(cs, holdsTheme));
        break;
    default:
        fprintf(stderr, "WARNING: Unknown action: %d\n", schema->corresponding_action);
        break;
    }
}

/******************************************************************************/
