#ifndef _analyse_h

#define _analyse_h

typedef enum {
    ERROR_ACTION_ADDITION, ERROR_ACTION_ANTICIPATION, ERROR_STEP_OMISSION, ERROR_PERSEVERATION,
    ERROR_TOOL_OMISSION, ERROR_MISLOCATION_1, ERROR_MISLOCATION_2,
    ERROR_MISUSE_1, ERROR_MISUSE_2, ERROR_OBJECT_SUBSTITUTION,
    ERROR_TOYING, ERROR_PANTOMIME, ERROR_PERPLEXITY
} ErrorType;

typedef struct line_list {
    char *line;
    struct line_list *next;
} LineList;

typedef struct data_analysis {
    int                 numActionAdditions;
    int                 numActionAnticipations;
    int                 numStepOmissions;
    int                 numPerseverations;
    int                 numToolOmissions;
    int                 numMisUse1;
    int                 numMisUse2;
    int                 numObjectSubstitution;
    int                 numMisLocation1;
    int                 numMisLocation2;
    int                 numToying;
    int                 numPantomime;
    int                 numPerplexity;
    int                 numActions;
    int                 numErrors;
    int                 numAccomplished;
    int                 numCycles;
    int                 numInterventions;
    double              propCommission;
    Boolean             complete;
    struct action_list *actionSequence;
    LineList           *log;
} DataAnalysis;

typedef struct action_list {
    char *name;
    char *argument1;
    char *argument2;
    char *argument3;
    Boolean checked;
    int stepNumber;
    struct action_list *next;    
} ActionList;

extern void analyseErrorLogAppend(DataAnalysis *la, ErrorType et, char *action, char *arg1, char *arg2);

#ifdef STANDALONE

// extern Boolean analyse_results(char *filename, DataAnalysis *la);
extern void analyse_lunch_task(DataAnalysis *la);

#else

extern Boolean analyse_results(char *filename, TaskType *task, DataAnalysis *la);

#endif

extern void report_unrecognised_object(FILE *fp, char *name);
extern void report_unrecognised_action(FILE *fp, ActionList *action);
extern void ignore_consecutive_actions(ActionList *this);
extern void score_consecutive_actions_as_perseverations(DataAnalysis *la, ActionList *action);

#endif
