/*******************************************************************************

    FILE: bottle/goal.c
    AUTHOR: R. Cooper
    CREATION DATE: Wed Aug  4 12:33:49 2004
    CONTENTS: Goal functions for the bottle task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

GoalType goalBottleClosed(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isClosedBottle(os->head));
    }
    return(found);
}

GoalType goalUsedBottleClosed(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isClosedBottle(os->head), isUsed(os->head)));
    }
    return(found);
}

GoalType goalBottleOpen(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isOpenBottle(os->head));
    }
    return(found);
}

GoalType goalGlassFull(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isFullGlass(os->head));
    }
    return(found);
}

GoalType goalGlassEmpty(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isEmptyGlass(os->head));
    }
    return(found);
}

/*----------------------------------------------------------------------------*/

GoalType goalBottleOnTable(ContentionScheduler *cs)
{
    return(goalOnTable(cs, isBottle));
}

/*----------------------------------------------------------------------------*/

GoalType goalCorkHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isCork));
}

GoalType goalBottleHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isBottle));
}

GoalType goalClosedBottleHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isClosedBottle));
}

GoalType goalOpenBottleHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isOpenBottle));
}

GoalType goalOpenSourceHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isOpenSource));
}

GoalType goalGlassHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isGlass));
}

GoalType goalFullGlassHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isFullGlass));
}

GoalType goalBottleOpenerHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isBottleOpener));
}

GoalType goalDrinkConsumed(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isGlass(os->head), isUsed(os->head)));
    }
    return(found);
}

/*----------------------------------------------------------------------------*/

GoalType goalBottleFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isBottle));
}

GoalType goalGlassFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isGlass));
}

GoalType goalBottleOpenerFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isBottleOpener));
}

/*----------------------------------------------------------------------------*/

GoalType goalOpenBottleHeldAndGlassFixated(ContentionScheduler *cs)
{
    return(FZAND(goalOpenBottleHeld(cs), goalGlassFixated(cs)));
}

GoalType goalDrinkConsumedAndGlassOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalDrinkConsumed(cs), goalOnTable(cs, isGlass)));
}

GoalType goalBottleOpenAndOpenerOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalBottleOpen(cs), goalOnTable(cs, isBottleOpener)));
}

/******************************************************************************/
