/*******************************************************************************

    FILE: bottle/object.c
    AUTHOR: R. Cooper
    CREATION DATE: Wed Aug  4 12:33:53 2004
    CONTENTS: Object-related routines for the bottle task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

extern double feature_overlap(int af, int bf);

/******************************************************************************/

double isBottle(Object *o)
{
    return(feature_overlap(o->features, IS_CONTAINER | IS_GLASS | IS_CYLINDRICAL | IS_LARGE));
}

double isBottleOpener(Object *o)
{
    return(feature_overlap(o->features, IS_TOOL | IS_SHARP | IS_EXTENDED));
}

double isCork(Object *o)
{
    return(feature_overlap(o->features, IS_CYLINDRICAL | IS_SMALL));
}

double isGlass(Object *o)
{
    return(feature_overlap(o->features, IS_CONTAINER | IS_GLASS | IS_CYLINDRICAL | IS_MEDIUM));
}

double isEmptyGlass(Object *o)
{
    return(FZAND(isGlass(o), isEmptyObject(o)));
}

double isFullGlass(Object *o)
{
    return(FZAND(isGlass(o), isFullObject(o)));
}

double isEmptyBottle(Object *o)
{
    return(FZAND(isBottle(o), isEmptyObject(o)));
}

double isFullBottle(Object *o)
{
    return(FZAND(isBottle(o), isFullObject(o)));
}

double isOpenSource(Object *o)
{
    return(FZAND(isFullObject(o), FZAND(isContainer(o), isOpen(o))));
}

double isOpenBottle(Object *o)
{
    return(FZAND(isBottle(o), isOpen(o)));
}

double isClosedBottle(Object *o)
{
    return(FZAND(isBottle(o), isClosed(o)));
}

/******************************************************************************/

void write_bottle_object_attributes(FILE *fp, Object *o)
{
    if (o->state & STATE_OPEN) { 
        fprintf(fp, "open, ");
    }
    else if (o->state & STATE_CLOSED) { 
        fprintf(fp, "closed, ");
    }

    if (o->itemsAttached) {
        fprintf(fp, "has attachments, ");
    }
}

/******************************************************************************/

void install_bottle_basic_objects(ContentionScheduler *cs)
{
    Object *object;

    object = objectInstall(cs, "glass", IS_CONTAINER | IS_GLASS | IS_CYLINDRICAL | IS_MEDIUM, 0);

    object = objectInstall(cs, "bottle", IS_CONTAINER | IS_GLASS | IS_CYLINDRICAL | IS_LARGE, STATE_CLOSED);
    objectSetPosition(object, -3, 3);
    objectInstallAttachment(object, objectInstall(cs, "cork", IS_CYLINDRICAL | IS_SMALL, 0));
    objectInstallContents(object, objectInstallItem(cs, "water", IS_FLUID));

    object = objectInstall(cs, "bottle_opener", IS_TOOL | IS_SHARP | IS_EXTENDED, 0);

    object = objectInstall(cs, "table", 0, 0);
}

/******************************************************************************/
