/*******************************************************************************

    FILE: bottle/schema.c
    AUTHOR: R. Cooper
    CREATION DATE: Wed Aug  4 12:33:55 2004
    CONTENTS: Schemas for the bottle task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

void install_bottle_schemas(ContentionScheduler *cs)
{
    Schema *s;

    s = install_schema(cs, "schemaBottleTask", "goalBottleTask", ACTION_NONE, HIGH);
    install_schema_object_trigger(s, SOURCE, schemaBottleTaskTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaBottleTaskTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaBottleTaskTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaBottleTaskTriggersImplement);
    install_schema_effector_trigger(s, schemaBottleTaskTriggersEffector);
    install_schema_subgoal(s, "goalOpenBottle", goalTrue, goalBottleOpen);
    install_schema_subgoal(s, "goalFillGlass", goalBottleOpen, goalGlassFull);
    install_schema_subgoal(s, "goalDrinkFromGlass", goalGlassFull, goalDrinkConsumedAndGlassOnTable);
    install_schema_subgoal(s, "goalSealBottle", goalGlassFull, goalUsedBottleClosed);

    cs->intended_schema = s;

    /* Intermediate level schemas: */

    s = install_schema(cs, "schemaOpenBottle", "goalOpenBottle", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaOpenBottleTriggersSource);
    install_schema_object_trigger(s, IMPLEMENT, schemaOpenBottleTriggersImplement);
    install_schema_effector_trigger(s, schemaOpenBottleTriggersEffector);
    install_schema_subgoal(s, "goalPickUpImplement", goalTrue, goalBottleOpenerHeld);
    install_schema_subgoal(s, "goalUncorkBottle", goalBottleOpenerHeld, goalBottleOpenAndOpenerOnTable); 
    install_schema_scope(s, SOURCE);
    install_schema_scope(s, IMPLEMENT);

    s = install_schema(cs, "schemaFillGlass", "goalFillGlass", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaFillGlassTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaFillGlassTriggersTarget);
    install_schema_effector_trigger(s, schemaFillGlassTriggersEffector);
    install_schema_subgoal(s, "goalPickUpSource", goalTrue, goalOpenSourceHeld);
    install_schema_subgoal(s, "goalPourIntoTarget", goalOpenSourceHeld, goalGlassFull);
    install_schema_scope(s, SOURCE);
    install_schema_scope(s, TARGET);

    s = install_schema(cs, "schemaDrinkFromGlass", "goalDrinkFromGlass", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaDrinkFromGlassTriggersSource);
    install_schema_effector_trigger(s, schemaDrinkFromGlassTriggersEffector);
    install_schema_subgoal(s, "goalPickUpSource", goalTrue, goalCurrentSourceHeld);
    install_schema_subgoal(s, "goalDrink", goalCurrentSourceHeld, goalDrinkConsumed);
    install_schema_subgoal(s, "goalPutDown", goalDrinkConsumed, goalDrinkConsumedAndGlassOnTable);
    install_schema_scope(s, SOURCE);

    s = install_schema(cs, "schemaSealBottle", "goalSealBottle", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaSealBottleTriggersTheme);
    install_schema_object_trigger(s, TARGET, schemaSealBottleTriggersTarget);
    install_schema_effector_trigger(s, schemaSealBottleTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalCorkHeld);
    install_schema_subgoal(s, "goalCorkBottle", goalCorkHeld, goalBottleClosed);
    install_schema_scope(s, TARGET);

    /* 2nd level intermediate schemas: */

    s = install_schema(cs, "schemaPickUpSource", "goalPickUpSource", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaPickUpSourceTriggersSource);
    install_schema_effector_trigger(s, schemaPickUpSourceTriggersEffector);
    install_schema_subgoal(s, "goalFixateSource", goalTrue, goalCurrentSourceFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentSourceFixated, goalCurrentSourceHeld);

    s = install_schema(cs, "schemaPickUpImplement", "goalPickUpImplement", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpImplementTriggersEffector);
    install_schema_subgoal(s, "goalFixateImplement", goalTrue, goalCurrentImplementFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentImplementFixated, goalCurrentImplementHeld);

    s = install_schema(cs, "schemaPickUpTheme", "goalPickUpTheme", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaPickUpThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaPickUpThemeTriggersEffector);
    install_schema_subgoal(s, "goalFixateTheme", goalTrue, goalCurrentThemeFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentThemeFixated, goalCurrentThemeHeld);

    s = install_schema(cs, "schemaUncorkBottle", "goalUncorkBottle", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaUncorkBottleTriggersSource);
    install_schema_effector_trigger(s, schemaUncorkBottleTriggersEffector);
    install_schema_subgoal(s, "goalFixateSource", goalTrue, goalCurrentSourceFixated);
    install_schema_subgoal(s, "goalUncork", goalCurrentSourceFixated, goalBottleOpen);
    install_schema_subgoal(s, "goalPutDown", goalBottleOpen, goalBottleOpenAndOpenerOnTable); 

    s = install_schema(cs, "schemaCorkBottle", "goalCorkBottle", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaCorkBottleTriggersTarget);
    install_schema_effector_trigger(s, schemaCorkBottleTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalRecork", goalCurrentTargetFixated, goalBottleClosed);

    s = install_schema(cs, "schemaPourIntoTarget", "goalPourIntoTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaPourIntoTargetTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPourIntoTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaPourIntoTargetTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalPourSome", goalCurrentTargetFixated, goalCurrentTargetFull);

    /* Low level schemas: */

    s = install_schema(cs, "schemaFixateSource", "goalFixateSource", ACTION_FIXATE_SOURCE, LOW);
    install_schema_object_trigger(s, SOURCE, schemaFixateSourceTriggersSource);
    install_schema_effector_trigger(s, schemaFixateSourceTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTarget", "goalFixateTarget", ACTION_FIXATE_TARGET, LOW);
    install_schema_object_trigger(s, TARGET, schemaFixateTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaFixateTargetTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateImplement", "goalFixateImplement", ACTION_FIXATE_IMPLEMENT, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaFixateImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaFixateImplementTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTheme", "goalFixateTheme", ACTION_FIXATE_THEME, LOW);
    install_schema_object_trigger(s, THEME, schemaFixateThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaFixateThemeTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPickUp", "goalPickUp", ACTION_PICK_UP, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPickUpTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPickUpTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPickUpTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPutDown", "goalPutDown", ACTION_PUT_DOWN, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPutDownTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPutDownTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPutDownTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPutDownTriggersImplement);
    install_schema_effector_trigger(s, schemaPutDownTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPourSome", "goalPourSome", ACTION_POUR_SOME, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPourSomeTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPourSomeTriggersTarget);
    install_schema_effector_trigger(s, schemaPourSomeTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaDrink", "goalDrink", ACTION_DRINK, LOW);
    install_schema_object_trigger(s, SOURCE, schemaDrinkTriggersSource);
    install_schema_effector_trigger(s, schemaDrinkTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaUncork", "goalUncork", ACTION_UNCORK, LOW);
    install_schema_object_trigger(s, SOURCE, schemaUncorkTriggersSource);
    install_schema_object_trigger(s, IMPLEMENT, schemaUncorkTriggersImplement);
    install_schema_effector_trigger(s, schemaUncorkTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaRecork", "goalRecork", ACTION_RECORK, LOW);
    install_schema_object_trigger(s, SOURCE, schemaRecorkTriggersSource);
    install_schema_object_trigger(s, THEME, schemaRecorkTriggersTheme);
    install_schema_effector_trigger(s, schemaRecorkTriggersEffector);
    install_schema_resource(s, HAND);
}

/******************************************************************************/
