/*******************************************************************************

    FILE: bottle/triggers.c
    AUTHOR: R. Cooper
    CREATION DATE: Wed Aug  4 12:33:58 2004
    CONTENTS: Schema triggering functions for the bottle task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

extern double holds(Effector *e, double f(Object *object));

/******************************************************************************/

double schemaBottleTaskTriggersSource(ContentionScheduler *cs, Object *o)
{
    return(FZOR(isFullBottle(o), isFullGlass(o)));
}

double schemaBottleTaskTriggersTarget(ContentionScheduler *cs, Object *o)
{
    return(FZOR(isEmptyGlass(o), isOpenBottle(o)));
}

double schemaBottleTaskTriggersTheme(ContentionScheduler *cs, Object *o)
{
    return(isCork(o));
}

double schemaBottleTaskTriggersImplement(ContentionScheduler *cs, Object *o)
{
    return(isBottleOpener(o));
}

double schemaBottleTaskTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

/******************************************************************************/

double schemaOpenBottleTriggersSource(ContentionScheduler *cs, Object *o)
{
    return(isClosedBottle(o));
}

double schemaOpenBottleTriggersImplement(ContentionScheduler *cs, Object *o)
{
    return(isBottleOpener(o));
}

double schemaOpenBottleTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

/******************************************************************************/

double schemaSealBottleTriggersTheme(ContentionScheduler *cs, Object *o)
{
    return(isCork(o));
}

double schemaSealBottleTriggersTarget(ContentionScheduler *cs, Object *o)
{
    return(isBottle(o));
}

double schemaSealBottleTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

/******************************************************************************/

double schemaPourIntoTargetTriggersSource(ContentionScheduler *cs, Object *object)
{
    return(FZAND(isFullContainer(object), isHeld(object)));
}

double schemaPourIntoTargetTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(isGlass(object));
}

double schemaPourIntoTargetTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(FZOR(holds(e, isFullContainer), holds(e, isOpen)));
}

/******************************************************************************/

double schemaDrinkFromGlassTriggersSource(ContentionScheduler *cs, Object *o)
{
    return(isFullGlass(o));
}

double schemaDrinkFromGlassTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

/******************************************************************************/

double schemaUncorkBottleTriggersSource(ContentionScheduler *cs, Object *o)
{
    return(isClosedBottle(o));
}

double schemaUncorkBottleTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

/******************************************************************************/

double schemaCorkBottleTriggersTarget(ContentionScheduler *cs, Object *o)
{
    return(isOpenBottle(o));
}

double schemaCorkBottleTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

/******************************************************************************/

double schemaPourSomeTriggersSource(ContentionScheduler *cs, Object *object)
{
    return(isFullContainer(object));
}

double schemaPourSomeTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(isGlass(object));
}

double schemaPourSomeTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(FZOR(holds(e, isFullContainer), holds(e, isGlass)));
}

/******************************************************************************/

double schemaDrinkTriggersSource(ContentionScheduler *cs, Object *object)
{
    return(FZAND(isFullGlass(object), isHeld(object)));
}

double schemaDrinkTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isFullGlass));
}

/******************************************************************************/

double schemaUncorkTriggersSource(ContentionScheduler *cs, Object *o)
{
    return(isClosedBottle(o));
}

double schemaUncorkTriggersImplement(ContentionScheduler *cs, Object *o)
{
    return(isCork(o));
}

double schemaUncorkTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

/******************************************************************************/

double schemaRecorkTriggersSource(ContentionScheduler *cs, Object *o)
{
    return(isOpenBottle(o));
}

double schemaRecorkTriggersTheme(ContentionScheduler *cs, Object *o)
{
    return(isCork(o));
}

double schemaRecorkTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

/******************************************************************************/
