/*******************************************************************************

    FILE: candle/analyse.c
    AUTHOR: R. Cooper
    CREATION DATE: Mon Aug 16 17:20:28 2004
    CONTENTS: Analysis routine for the candle task

*******************************************************************************/

#include "../cs.h"
#include "../analyse.h"

/******************************************************************************/

typedef enum candle_object {
    OBJECT_NONE, OBJECT_CANDLE_BASE, OBJECT_CANDLE_TOP, OBJECT_CANDLE_STICK,
    OBJECT_MATCHBOX_DRAWER, OBJECT_MATCHBOX, OBJECT_TABLE,
    OBJECT_MATCH_STICK_0,  OBJECT_MATCH_HEAD_0,
    OBJECT_MATCH_STICK_1,  OBJECT_MATCH_HEAD_1,
    OBJECT_MATCH_STICK_2,  OBJECT_MATCH_HEAD_2,
    OBJECT_MATCH_STICK_3,  OBJECT_MATCH_HEAD_3,
    OBJECT_MATCH_STICK_4,  OBJECT_MATCH_HEAD_4,
    OBJECT_MATCH_STICK_5,  OBJECT_MATCH_HEAD_5,
    OBJECT_MATCH_STICK_6,  OBJECT_MATCH_HEAD_6,
    OBJECT_MATCH_STICK_7,  OBJECT_MATCH_HEAD_7,
    OBJECT_MATCH_STICK_8,  OBJECT_MATCH_HEAD_8,
    OBJECT_MATCH_STICK_9,  OBJECT_MATCH_HEAD_9,
    OBJECT_MATCH_STICK_10, OBJECT_MATCH_HEAD_10,
    OBJECT_MATCH_STICK_11, OBJECT_MATCH_HEAD_11,
    OBJECT_MATCH_STICK_12, OBJECT_MATCH_HEAD_12,
    OBJECT_MATCH_STICK_13, OBJECT_MATCH_HEAD_13,
    OBJECT_MATCH_STICK_14, OBJECT_MATCH_HEAD_14,
    OBJECT_MATCH_STICK_15, OBJECT_MATCH_HEAD_15,
    OBJECT_MATCH_STICK_16, OBJECT_MATCH_HEAD_16,
    OBJECT_MATCH_STICK_17, OBJECT_MATCH_HEAD_17,
    OBJECT_MATCH_STICK_18, OBJECT_MATCH_HEAD_18,
    OBJECT_MATCH_STICK_19, OBJECT_MATCH_HEAD_19,
    OBJECT_UNKNOWN
} CandleObject;

typedef enum candle_location {
    LOC_NONE, LOC_HELD, LOC_ON_TABLE, LOC_IN_MATCHBOX
} CandleLocation;

typedef struct candle_analysis_object_list {
    CandleObject                        id;        /* Enumerated id */
    CandleLocation                      location;
    int                                 state;
    struct candle_analysis_object_list *next;     /* Next object in the list  */
} CandleAnalysisObjectList;

typedef struct candle_state {
    // Information on all possible objects in the world
    CandleAnalysisObjectList   *items; 

    // Fixated item:
    CandleObject                fixated;

    // Other relevant state information:
    Boolean                     candle_positioned;
    Boolean                     match_lit;
    Boolean                     candle_lit;
    Boolean                     match_extinguished;
    Boolean                     box_closed;

} CandleState;

#define CANDLE_STATE_LIT              1
#define CANDLE_STATE_EXTINGUISHED     2
#define CANDLE_STATE_OPEN             4
#define CANDLE_STATE_CLOSED           8

/******************************************************************************/

static CandleObject object_id_from_string(char *arg)
{
    if (arg == NULL) {
        return(OBJECT_NONE);
    }
    else if (strcmp(arg, "nothing") == 0) {
        return(OBJECT_NONE);
    }
    else if (strcmp(arg, "candle_base") == 0) {
        return(OBJECT_CANDLE_BASE);
    }
    else if (strcmp(arg, "candle_top") == 0) {
        return(OBJECT_CANDLE_TOP);
    }
    else if (strcmp(arg, "candle_stick") == 0) {
        return(OBJECT_CANDLE_STICK);
    }
    else if (strcmp(arg, "matchbox") == 0) {
        return(OBJECT_MATCHBOX);
    }
    else if (strcmp(arg, "matchbox_drawer") == 0) {
        return(OBJECT_MATCHBOX_DRAWER);
    }
    else if (strcmp(arg, "match_stick_0") == 0) {
        return(OBJECT_MATCH_STICK_0);
    }
    else if (strcmp(arg, "match_stick_head_0") == 0) {
        return(OBJECT_MATCH_HEAD_0);
    }
    else if (strcmp(arg, "match_stick_1") == 0) {
        return(OBJECT_MATCH_STICK_1);
    }
    else if (strcmp(arg, "match_stick_head_1") == 0) {
        return(OBJECT_MATCH_HEAD_1);
    }
    else if (strcmp(arg, "match_stick_2") == 0) {
        return(OBJECT_MATCH_STICK_2);
    }
    else if (strcmp(arg, "match_stick_head_2") == 0) {
        return(OBJECT_MATCH_HEAD_2);
    }
    else if (strcmp(arg, "match_stick_3") == 0) {
        return(OBJECT_MATCH_STICK_3);
    }
    else if (strcmp(arg, "match_stick_head_3") == 0) {
        return(OBJECT_MATCH_HEAD_3);
    }
    else if (strcmp(arg, "match_stick_4") == 0) {
        return(OBJECT_MATCH_STICK_4);
    }
    else if (strcmp(arg, "match_stick_head_4") == 0) {
        return(OBJECT_MATCH_HEAD_4);
    }
    else if (strcmp(arg, "match_stick_5") == 0) {
        return(OBJECT_MATCH_STICK_5);
    }
    else if (strcmp(arg, "match_stick_head_5") == 0) {
        return(OBJECT_MATCH_HEAD_5);
    }
    else if (strcmp(arg, "match_stick_6") == 0) {
        return(OBJECT_MATCH_STICK_6);
    }
    else if (strcmp(arg, "match_stick_head_6") == 0) {
        return(OBJECT_MATCH_HEAD_6);
    }
    else if (strcmp(arg, "match_stick_7") == 0) {
        return(OBJECT_MATCH_STICK_7);
    }
    else if (strcmp(arg, "match_stick_head_7") == 0) {
        return(OBJECT_MATCH_HEAD_7);
    }
    else if (strcmp(arg, "match_stick_8") == 0) {
        return(OBJECT_MATCH_STICK_8);
    }
    else if (strcmp(arg, "match_stick_head_8") == 0) {
        return(OBJECT_MATCH_HEAD_8);
    }
    else if (strcmp(arg, "match_stick_9") == 0) {
        return(OBJECT_MATCH_STICK_9);
    }
    else if (strcmp(arg, "match_stick_head_9") == 0) {
        return(OBJECT_MATCH_HEAD_9);
    }
    else if (strcmp(arg, "match_stick_10") == 0) {
        return(OBJECT_MATCH_STICK_10);
    }
    else if (strcmp(arg, "match_stick_head_10") == 0) {
        return(OBJECT_MATCH_HEAD_10);
    }
    else if (strcmp(arg, "match_stick_11") == 0) {
        return(OBJECT_MATCH_STICK_11);
    }
    else if (strcmp(arg, "match_stick_head_11") == 0) {
        return(OBJECT_MATCH_HEAD_11);
    }
    else if (strcmp(arg, "match_stick_12") == 0) {
        return(OBJECT_MATCH_STICK_12);
    }
    else if (strcmp(arg, "match_stick_head_12") == 0) {
        return(OBJECT_MATCH_HEAD_12);
    }
    else if (strcmp(arg, "match_stick_13") == 0) {
        return(OBJECT_MATCH_STICK_13);
    }
    else if (strcmp(arg, "match_stick_head_13") == 0) {
        return(OBJECT_MATCH_HEAD_13);
    }
    else if (strcmp(arg, "match_stick_14") == 0) {
        return(OBJECT_MATCH_STICK_14);
    }
    else if (strcmp(arg, "match_stick_head_14") == 0) {
        return(OBJECT_MATCH_HEAD_14);
    }
    else if (strcmp(arg, "match_stick_15") == 0) {
        return(OBJECT_MATCH_STICK_15);
    }
    else if (strcmp(arg, "match_stick_head_15") == 0) {
        return(OBJECT_MATCH_HEAD_15);
    }
    else if (strcmp(arg, "match_stick_16") == 0) {
        return(OBJECT_MATCH_STICK_16);
    }
    else if (strcmp(arg, "match_stick_head_16") == 0) {
        return(OBJECT_MATCH_HEAD_16);
    }
    else if (strcmp(arg, "match_stick_17") == 0) {
        return(OBJECT_MATCH_STICK_17);
    }
    else if (strcmp(arg, "match_stick_head_17") == 0) {
        return(OBJECT_MATCH_HEAD_17);
    }
    else if (strcmp(arg, "match_stick_18") == 0) {
        return(OBJECT_MATCH_STICK_18);
    }
    else if (strcmp(arg, "match_stick_head_18") == 0) {
        return(OBJECT_MATCH_HEAD_18);
    }
    else if (strcmp(arg, "match_stick_19") == 0) {
        return(OBJECT_MATCH_STICK_19);
    }
    else if (strcmp(arg, "match_stick_head_19") == 0) {
        return(OBJECT_MATCH_HEAD_19);
    }
    else if (strcmp(arg, "table") == 0) {
        return(OBJECT_TABLE);
    }
    else {    /* Final catch-all just in case: */
        report_unrecognised_object(stdout, arg);
        return(OBJECT_UNKNOWN);
    }
}

static CandleAnalysisObjectList *object_ptr_from_id(CandleState *state, CandleObject id)
{
    CandleAnalysisObjectList *tmp = state->items;

    while ((tmp != NULL) && (tmp->id != id)) {
        tmp = tmp->next;
    }
    if ((tmp == NULL) || (tmp->id != id)) {
        return(NULL);
    }
    else {
        return(tmp);
    }
}

static CandleAnalysisObjectList *object_ptr_from_string(CandleState *state, char *arg)
{
    return(object_ptr_from_id(state, object_id_from_string(arg)));
}

static Boolean action_compare(ActionList *this, char *action, CandleObject arg1)
{
    if (this == NULL) {
        return(FALSE);
    }
    else if (strcmp(this->name, action) != 0) {
        return(FALSE);
    }
    else if (object_id_from_string(this->argument1) != arg1) {
        return(FALSE);
    }
    else {
        return(TRUE);
    }
}

static Boolean isCandle(CandleAnalysisObjectList *item)
{
    switch(item->id) {
        case OBJECT_CANDLE_BASE:
        case OBJECT_CANDLE_TOP:
            return(TRUE);
        default:
            return(FALSE);
    }
}

static Boolean isMatch(CandleAnalysisObjectList *item)
{
    switch(item->id) {
        case OBJECT_MATCH_STICK_0:  
        case OBJECT_MATCH_HEAD_0:
        case OBJECT_MATCH_STICK_1:  
        case OBJECT_MATCH_HEAD_1:
        case OBJECT_MATCH_STICK_2:  
        case OBJECT_MATCH_HEAD_2:
        case OBJECT_MATCH_STICK_3:  
        case OBJECT_MATCH_HEAD_3:
        case OBJECT_MATCH_STICK_4:  
        case OBJECT_MATCH_HEAD_4:
        case OBJECT_MATCH_STICK_5:  
        case OBJECT_MATCH_HEAD_5:
        case OBJECT_MATCH_STICK_6:  
        case OBJECT_MATCH_HEAD_6:
        case OBJECT_MATCH_STICK_7:  
        case OBJECT_MATCH_HEAD_7:
        case OBJECT_MATCH_STICK_8:  
        case OBJECT_MATCH_HEAD_8:
        case OBJECT_MATCH_STICK_9:  
        case OBJECT_MATCH_HEAD_9:
        case OBJECT_MATCH_STICK_10: 
        case OBJECT_MATCH_HEAD_10:
        case OBJECT_MATCH_STICK_11: 
        case OBJECT_MATCH_HEAD_11:
        case OBJECT_MATCH_STICK_12: 
        case OBJECT_MATCH_HEAD_12:
        case OBJECT_MATCH_STICK_13: 
        case OBJECT_MATCH_HEAD_13:
        case OBJECT_MATCH_STICK_14: 
        case OBJECT_MATCH_HEAD_14:
        case OBJECT_MATCH_STICK_15: 
        case OBJECT_MATCH_HEAD_15:
        case OBJECT_MATCH_STICK_16: 
        case OBJECT_MATCH_HEAD_16:
        case OBJECT_MATCH_STICK_17: 
        case OBJECT_MATCH_HEAD_17:
        case OBJECT_MATCH_STICK_18: 
        case OBJECT_MATCH_HEAD_18:
        case OBJECT_MATCH_STICK_19: 
        case OBJECT_MATCH_HEAD_19:
            return(TRUE);
        default:
            return(FALSE);
    }
}

static Boolean isMatchStick(CandleAnalysisObjectList *item)
{
    switch(item->id) {
        case OBJECT_MATCH_STICK_0:  
        case OBJECT_MATCH_STICK_1:  
        case OBJECT_MATCH_STICK_2:  
        case OBJECT_MATCH_STICK_3:  
        case OBJECT_MATCH_STICK_4:  
        case OBJECT_MATCH_STICK_5:  
        case OBJECT_MATCH_STICK_6:  
        case OBJECT_MATCH_STICK_7:  
        case OBJECT_MATCH_STICK_8:  
        case OBJECT_MATCH_STICK_9:  
        case OBJECT_MATCH_STICK_10: 
        case OBJECT_MATCH_STICK_11: 
        case OBJECT_MATCH_STICK_12: 
        case OBJECT_MATCH_STICK_13: 
        case OBJECT_MATCH_STICK_14: 
        case OBJECT_MATCH_STICK_15: 
        case OBJECT_MATCH_STICK_16: 
        case OBJECT_MATCH_STICK_17: 
        case OBJECT_MATCH_STICK_18: 
        case OBJECT_MATCH_STICK_19: 
            return(TRUE);
        default:
            return(FALSE);
    }
}

static CandleAnalysisObjectList *getMatchStickHead(CandleState *state, CandleAnalysisObjectList *item)
{
    switch(item->id) {
        case OBJECT_MATCH_STICK_0:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_0));
        case OBJECT_MATCH_STICK_1:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_1));
        case OBJECT_MATCH_STICK_2:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_2));
        case OBJECT_MATCH_STICK_3:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_3));
        case OBJECT_MATCH_STICK_4:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_4));
        case OBJECT_MATCH_STICK_5:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_5));
        case OBJECT_MATCH_STICK_6:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_6));
        case OBJECT_MATCH_STICK_7:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_7));
        case OBJECT_MATCH_STICK_8:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_8));
        case OBJECT_MATCH_STICK_9:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_9));
        case OBJECT_MATCH_STICK_10:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_10));
        case OBJECT_MATCH_STICK_11:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_11));
        case OBJECT_MATCH_STICK_12:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_12));
        case OBJECT_MATCH_STICK_13:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_13));
        case OBJECT_MATCH_STICK_14:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_14));
        case OBJECT_MATCH_STICK_15:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_15));
        case OBJECT_MATCH_STICK_16:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_16));
        case OBJECT_MATCH_STICK_17:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_17));
        case OBJECT_MATCH_STICK_18:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_18));
        case OBJECT_MATCH_STICK_19:
            return(object_ptr_from_id(state, OBJECT_MATCH_HEAD_19));
        default:
            return(NULL);
    }
}

static Boolean match_head_is_alight(CandleState *state, CandleAnalysisObjectList *item)
{
    CandleAnalysisObjectList *head = getMatchStickHead(state, item);

    if (head == NULL) {
        return(FALSE);
    }
    else {
        return(head->state & CANDLE_STATE_LIT);
    }
}

/******************************************************************************/

static void state_add_item(CandleState *state, CandleObject id, CandleLocation loc, int item_state)
{
    CandleAnalysisObjectList *new = (CandleAnalysisObjectList *)malloc(sizeof(CandleAnalysisObjectList));

    if (new != NULL) {
        new->id = id;
        new->location = loc;
        new->state = item_state;
        new->next = state->items;
        state->items = new;
    }
}

static void state_initialise(CandleState *state)
{
    state->items = NULL;

    /* All objects must be initialised to match candle/object.c */

    state_add_item(state, OBJECT_CANDLE_STICK, LOC_ON_TABLE, 0);
    state_add_item(state, OBJECT_CANDLE_BASE, LOC_ON_TABLE, 0);
    state_add_item(state, OBJECT_CANDLE_TOP, LOC_ON_TABLE, 0);
    state_add_item(state, OBJECT_MATCHBOX, LOC_ON_TABLE, CANDLE_STATE_CLOSED);
    state_add_item(state, OBJECT_MATCHBOX_DRAWER, LOC_IN_MATCHBOX, CANDLE_STATE_CLOSED);
    state_add_item(state, OBJECT_MATCH_STICK_0, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_0, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_1, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_1, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_2, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_2, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_3, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_3, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_4, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_4, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_5, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_5, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_6, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_6, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_7, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_7, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_8, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_8, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_9, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_9, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_10, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_10, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_11, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_11, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_12, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_12, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_13, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_13, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_14, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_14, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_15, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_15, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_16, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_16, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_17, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_17, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_18, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_18, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_STICK_19, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_MATCH_HEAD_19, LOC_IN_MATCHBOX, 0);
    state_add_item(state, OBJECT_TABLE, LOC_NONE, 0);

    state->fixated = OBJECT_NONE;

    state->candle_positioned = FALSE;
    state->match_lit = FALSE;
    state->candle_lit = FALSE;
    state->match_extinguished = FALSE;
    state->box_closed = FALSE;
}

static void state_destroy(CandleState *state)
{
    while (state->items != NULL) {
        CandleAnalysisObjectList *tmp = state->items;
        state->items = tmp->next;
        free(tmp);
    }
}

/******************************************************************************/

static void check_consecutive_toying_actions(ActionList *this, CandleObject arg1)
{
    if (action_compare(this->next, "PickUp", arg1)) {
        if (action_compare(this->next->next, "PutDown", arg1)) {
            this->next->checked = TRUE;
            this->next->next->checked = TRUE;
            check_consecutive_toying_actions(this->next->next, arg1);
        }
        else if (action_compare(this->next->next, "Discard", arg1)) {
            this->next->checked = TRUE;
            this->next->next->checked = TRUE;
            check_consecutive_toying_actions(this->next->next, arg1);
        }
    }
}

/******************************************************************************/

static void score_action_fixate(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1)
{
    CandleAnalysisObjectList *item1;

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "Fixate", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else {
        state->fixated = item1->id;
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_pickup(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1)
{
    CandleAnalysisObjectList *item1;

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "PickUp", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if (item1->location == LOC_HELD) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "PickUp", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else {
        item1->location = LOC_HELD;
        if (action_compare(action->next, "PutDown", item1->id)) {
            // Score TOYING error and ignore any consecutive toying
            analyseErrorLogAppend(la, ERROR_TOYING, "PickUp/PutDown", arg1, NULL);
            check_consecutive_toying_actions(action->next, item1->id);
        }
        else if (action_compare(action->next, "Discard", item1->id)) {
            // Score TOYING error and ignore any consecutive toying
            analyseErrorLogAppend(la, ERROR_TOYING, "PickUp/Discard", arg1, NULL);
            check_consecutive_toying_actions(action->next, item1->id);
        }
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_putdown(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1)
{
    CandleAnalysisObjectList *item1;

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "PutDown", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if (item1->location != LOC_HELD) {
        // Score PERPLEXITY: Object is not held
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "PutDown", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else {
        item1->location = LOC_ON_TABLE;
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_discard(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1)
{
    CandleAnalysisObjectList *item1;

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "Discard", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if (item1->location != LOC_HELD) {
        // Score PERPLEXITY: Object is not held
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "Discard", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else {
        item1->location = LOC_ON_TABLE;
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_candle_into_holder(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1, char *arg2)
{
    CandleAnalysisObjectList *item1;        // Candle base
    CandleAnalysisObjectList *item2;        // Candle stick

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "CandleIntoHolder", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if (object_id_from_string(arg2) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "CandleIntoHolder", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if ((item2 = object_ptr_from_string(state, arg2)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if (item1 == item2) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "CandleIntoHolder", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if ((item1->id != OBJECT_CANDLE_BASE) && (item1->id != OBJECT_CANDLE_TOP)) {
        analyseErrorLogAppend(la, ERROR_MISUSE_1, "CandleIntoHolder", arg1, arg2);
    }
    else if (item1->id == OBJECT_CANDLE_TOP) {
        analyseErrorLogAppend(la, ERROR_MISLOCATION_2, "CandleIntoHolder", arg1, arg2);
        if ((item2->id == OBJECT_CANDLE_STICK) || (item2->id == OBJECT_TABLE)) {
            state->candle_positioned = TRUE;
        }
    }
    else if (item2->id == OBJECT_TABLE) {
        analyseErrorLogAppend(la, ERROR_MISLOCATION_1, "CandleIntoHolder", arg1, arg2);
        state->candle_positioned = TRUE;
    }
    else if (item2->id != OBJECT_CANDLE_STICK) {
        analyseErrorLogAppend(la, ERROR_MISLOCATION_1, "CandleIntoHolder", arg1, arg2);
    }
    else {
        state->candle_positioned = TRUE;
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_slide_open(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1)
{
    CandleAnalysisObjectList *item1;        // The matchbox

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "SlideOpen", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if ((item1->id != OBJECT_MATCHBOX) && (item1->id != OBJECT_MATCHBOX_DRAWER)) {
        // Score MISUSE1: Attempting to opening some odd/inappropriate object
        analyseErrorLogAppend(la, ERROR_MISUSE_1, "SlideOpen", arg1, NULL);
    }
    else if (item1->state & CANDLE_STATE_OPEN) {
        // Score PERSEVERATION: Object is already open
        analyseErrorLogAppend(la, ERROR_PERSEVERATION, "SlideOpen", arg1, NULL);
    }
    else if (item1->id == OBJECT_MATCHBOX_DRAWER) {
        // Score MISLOCATION 1: Opening the matchbox drawer, instead of the matchbox
        // Let's assume that this works, despite being an unconventional method

        CandleAnalysisObjectList *matchbox = object_ptr_from_id(state, OBJECT_MATCHBOX);

        analyseErrorLogAppend(la, ERROR_MISLOCATION_1, "SlideOpen", arg1, NULL);
        matchbox->state = (matchbox->state | CANDLE_STATE_OPEN) & ~CANDLE_STATE_CLOSED;
    }
    else {
        // Perfect!
        item1->state = (item1->state | CANDLE_STATE_OPEN) & ~CANDLE_STATE_CLOSED;
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_slide_closed(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1)
{
    CandleAnalysisObjectList *item1;        // The matchbox

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "SlideClosed", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if ((item1->id != OBJECT_MATCHBOX) && (item1->id != OBJECT_MATCHBOX_DRAWER)) {
        // Score MISUSE1: Attempting to opening close odd/inappropriate object
        analyseErrorLogAppend(la, ERROR_MISUSE_1, "SlideClosed", arg1, NULL);
    }
    else if (item1->state & CANDLE_STATE_CLOSED) {
        // Object is already closed ...
        if (state->box_closed) {
            // Score PERSEVERATION: Object has been closed before
            analyseErrorLogAppend(la, ERROR_PERSEVERATION, "SlideClosed", arg1, NULL);
        }
        else {
            // Score ANTICIPATION: Object has not been opened yet!
            analyseErrorLogAppend(la, ERROR_ACTION_ANTICIPATION, "SlideClosed", arg1, NULL);
        }
    }
    else if (item1->id == OBJECT_MATCHBOX_DRAWER) {
        // Score MISLOCATION 1: Closing the matchbox drawer, instead of the matchbox
        // Let's assume that this works, despite being an unconventional method

        CandleAnalysisObjectList *matchbox = object_ptr_from_id(state, OBJECT_MATCHBOX);

        analyseErrorLogAppend(la, ERROR_MISLOCATION_1, "SlideClosed", arg1, NULL);
        matchbox->state = (matchbox->state | CANDLE_STATE_CLOSED) & ~CANDLE_STATE_OPEN;
    }
    else {
        // Perfect!
        item1->state = (item1->state | CANDLE_STATE_CLOSED) & ~CANDLE_STATE_OPEN;
        state->box_closed = TRUE;
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_strike_match(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1, char *arg2)
{
    CandleAnalysisObjectList *item1;        // The (unlit) match
    CandleAnalysisObjectList *item2;        // The matchbox
    CandleAnalysisObjectList *head;         // The head of the unlit match

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PANTOMIME, "StrikeMatch", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if (object_id_from_string(arg2) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PANTOMIME, "StrikeMatch", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if ((item2 = object_ptr_from_string(state, arg2)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if (item1 == item2) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "StrikeMatch", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if (item1 == getMatchStickHead(state, item2)) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "StrikeMatch", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if (item2 == getMatchStickHead(state, item1)) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "StrikeMatch", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if (!isMatch(item1)) {
        // Score MISUSE1: Not a match!
        analyseErrorLogAppend(la, ERROR_MISUSE_1, "StrikeMatch", arg1, arg2);
    }
    else if ((head = getMatchStickHead(state, item1)) == NULL) {
        // Score MISLOCATION1: Wrong end of match!
        analyseErrorLogAppend(la, ERROR_MISLOCATION_1, "StrikeMatch", arg1, arg2);
    }
    else if (head->state & CANDLE_STATE_LIT) {
        // Score PERSEVERATION: Match is already alight
        analyseErrorLogAppend(la, ERROR_PERSEVERATION, "StrikeMatch", arg1, arg2);
    }
    else if (head->state & CANDLE_STATE_EXTINGUISHED) {
        // Score PERSEVERATION: Match has previously been used
        analyseErrorLogAppend(la, ERROR_PERSEVERATION, "StrikeMatch", arg1, arg2);
    }
    else if ((item2->id != OBJECT_MATCHBOX) && (item2->id != OBJECT_MATCHBOX_DRAWER)) {
        // Score MISLOCATION1: Completely wrong target!
        analyseErrorLogAppend(la, ERROR_MISLOCATION_1, "StrikeMatch", arg1, arg2);
    }
    else if (item2->id == OBJECT_MATCHBOX_DRAWER) {
        // Score MISLOCATION2: Wrong part of target!
        analyseErrorLogAppend(la, ERROR_MISLOCATION_2, "StrikeMatch", arg1, arg2);
    }
    else {
        // Everything seems to be sensible...
        head->state = head->state | CANDLE_STATE_LIT;
        state->match_lit = TRUE;
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_match_to_candle(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1, char *arg2)
{
    CandleAnalysisObjectList *item1;        // The lit match
    CandleAnalysisObjectList *item2;        // The candle top

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PANTOMIME, "MatchToCandle", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if (object_id_from_string(arg2) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PANTOMIME, "MatchToCandle", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if ((item2 = object_ptr_from_string(state, arg2)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if (item1 == item2) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "MatchToCandle", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if (item1 == getMatchStickHead(state, item2)) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "MatchToCandle", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if (item2 == getMatchStickHead(state, item1)) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "MatchToCandle", arg1, arg2);
        ignore_consecutive_actions(action);
    }
    else if (!isMatch(item1)) {
        // Score MISUSE1: Not a match!
        analyseErrorLogAppend(la, ERROR_MISUSE_1, "MatchToCandle", arg1, arg2);
    }
    else if (!isMatchStick(item1)) {
        // Score MISLOCATION2: Wrong end of match
        analyseErrorLogAppend(la, ERROR_MISLOCATION_2, "MatchToCandle", arg1, arg2);
    }
    else if (!match_head_is_alight(state, item1)) {
        // Score ANTICIPATION: Lighting candle before the match has been lit!
        analyseErrorLogAppend(la, ERROR_ACTION_ANTICIPATION, "MatchToCandle", arg1, arg2);
    }
    else if (!isCandle(item2)) {
        // Score MISLOCATION1: Not a candle!
        analyseErrorLogAppend(la, ERROR_MISLOCATION_1, "MatchToCandle", arg1, arg2);
    }
    else if (item2->id == OBJECT_CANDLE_BASE) {
        // Score MISLOCATION2: Wrong end of candle
        analyseErrorLogAppend(la, ERROR_MISLOCATION_2, "MatchToCandle", arg1, arg2);
    }
    else if (item2->state & CANDLE_STATE_LIT) {
        // Score PERSEVERATION: Candle is already alight
        analyseErrorLogAppend(la, ERROR_PERSEVERATION, "MatchToCandle", arg1, arg2);
    }
    else {
        item2->state = item2->state | CANDLE_STATE_LIT;
        state->candle_lit = TRUE;
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_blow_out_flame(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1)
{
    CandleAnalysisObjectList *item1;        // The lit match

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PANTOMIME, "BlowOut", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if (!isCandle(item1) && !isMatch(item1)) {
        // Score MISUSE1: Attempting to blow out some odd/inappropriate object
        analyseErrorLogAppend(la, ERROR_MISUSE_1, "BlowOut", arg1, NULL);
    }
    else if (!(item1->state & CANDLE_STATE_LIT)) {
        // Object is not alight. Several possiblities:
        if (item1->id == OBJECT_CANDLE_BASE) {
            // Score MISLOCATION2: Attempting to blow out the wrong end of the candle
            analyseErrorLogAppend(la, ERROR_MISLOCATION_2, "BlowOut", arg1, NULL);
        }
        else if (isMatchStick(item1)) {
            // Score MISLOCATION2: Attempting to blow out the wrong end of the match
            analyseErrorLogAppend(la, ERROR_MISLOCATION_2, "BlowOut", arg1, NULL);
        }
        else if (item1->state & CANDLE_STATE_EXTINGUISHED) {
            // Score PERSEVERATION: Object has already been extinguished
            analyseErrorLogAppend(la, ERROR_PERSEVERATION, "BlowOut", arg1, NULL);
        }
        else {
            // Score ANTICIPATION: The thing hasn't been lit yet!
            analyseErrorLogAppend(la, ERROR_ACTION_ANTICIPATION, "BlowOut", arg1, NULL);
	}
    }
    else {
        if (!state->candle_lit) {
            // Score ANTICIPATION: Blowing out the match before the candle has been lit!
            analyseErrorLogAppend(la, ERROR_ACTION_ANTICIPATION, "BlowOut", arg1, NULL);
            state->candle_lit = TRUE; // To prevent later scoring of the omission error
        }
        state->match_extinguished = TRUE;
        item1->state = (item1->state | CANDLE_STATE_EXTINGUISHED) & ~CANDLE_STATE_LIT;
        if (item1->id == OBJECT_CANDLE_TOP) {
            // Score MISLOCATION1: Blowing out the candle instead of the match
            analyseErrorLogAppend(la, ERROR_MISLOCATION_1, "BlowOut", arg1, NULL);
	}
        else {
            // We must be dealing with a lit match. Everything is OK!
        }
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_tear_open(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1)
{
    CandleAnalysisObjectList *item1;        // The container

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "TearOpen", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if ((item1->id != OBJECT_MATCHBOX) && (item1->id != OBJECT_MATCHBOX_DRAWER)) {
        // Score MISUSE1: Attempting to opening some odd/inappropriate object
        analyseErrorLogAppend(la, ERROR_MISUSE_1, "TearOpen", arg1, NULL);
    }
    else if (item1->state & CANDLE_STATE_OPEN) {
        // Score PERSEVERATION: Object is already open
        analyseErrorLogAppend(la, ERROR_PERSEVERATION, "TearOpen", arg1, NULL);
    }
    else {
        // Score MISUSE 2: Opening the matchbox/drawer with wrong action
        analyseErrorLogAppend(la, ERROR_MISUSE_2, "TearOpen", arg1, NULL);
    }
}

/*----------------------------------------------------------------------------*/

static void score_action_peel_open(CandleState *state, DataAnalysis *la, ActionList *action, char *arg1)
{
    CandleAnalysisObjectList *item1;        // The container

    if (object_id_from_string(arg1) == OBJECT_NONE) {
        analyseErrorLogAppend(la, ERROR_PERPLEXITY, "PeelOpen", arg1, NULL);
        ignore_consecutive_actions(action);
    }
    else if ((item1 = object_ptr_from_string(state, arg1)) == NULL) {
        report_unrecognised_action(stdout, action);
    }
    else if ((item1->id != OBJECT_MATCHBOX) && (item1->id != OBJECT_MATCHBOX_DRAWER)) {
        // Score MISUSE1: Attempting to opening some odd/inappropriate object
        analyseErrorLogAppend(la, ERROR_MISUSE_1, "PeelOpen", arg1, NULL);
    }
    else if (item1->state & CANDLE_STATE_OPEN) {
        // Score PERSEVERATION: Object is already open
        analyseErrorLogAppend(la, ERROR_PERSEVERATION, "PeelOpen", arg1, NULL);
    }
    else {
        // Score MISUSE 2: Opening the matchbox/drawer with wrong action
        analyseErrorLogAppend(la, ERROR_MISUSE_2, "PeelOpen", arg1, NULL);
    }
}

/******************************************************************************/

static void score_action(CandleState *state, DataAnalysis *la, ActionList *action)
{
    if (action == NULL) {
        fprintf(stderr, "INTERNAL ERROR: NULL action in score_action/3 (file is candle/analyse.c)\n");
    }
    else if (strcmp(action->name, "Fixate") == 0) {
        score_action_fixate(state, la, action, action->argument1);
    }
    else if (strcmp(action->name, "PickUp") == 0) {
        score_action_pickup(state, la, action, action->argument1);
    }
    else if (strcmp(action->name, "PutDown") == 0) {
        score_action_putdown(state, la, action, action->argument1);
    }
    else if (strcmp(action->name, "Discard") == 0) {
        score_action_discard(state, la, action, action->argument1);
    }
    else if (strcmp(action->name, "CandleIntoHolder") == 0) {
        score_action_candle_into_holder(state, la, action, action->argument1, action->argument2);
    }
    else if (strcmp(action->name, "SlideOpen") == 0) {
        score_action_slide_open(state, la, action, action->argument1);
    }
    else if (strcmp(action->name, "SlideClosed") == 0) {
        score_action_slide_closed(state, la, action, action->argument1);
    }
    else if (strcmp(action->name, "StrikeMatch") == 0) {
        score_action_strike_match(state, la, action, action->argument1, action->argument2);
    }
    else if (strcmp(action->name, "MatchToCandle") == 0) {
        score_action_match_to_candle(state, la, action, action->argument1, action->argument2);
    }
    else if (strcmp(action->name, "BlowOutFlame") == 0) {
        score_action_blow_out_flame(state, la, action, action->argument1);
    }
    else if (strcmp(action->name, "TearOpen") == 0) {
        score_action_tear_open(state, la, action, action->argument1);
    }
    else if (strcmp(action->name, "PeelOpen") == 0) {
        score_action_peel_open(state, la, action, action->argument1);
    }
    else {
        report_unrecognised_action(stdout, action);
    }
}

/******************************************************************************/

static void score_omissions(CandleState *state, DataAnalysis *la)
{
    if (!state->candle_positioned) {
        analyseErrorLogAppend(la, ERROR_STEP_OMISSION, "PositionCandle", "candle", "candlestick");
    }
    if (!state->match_lit) {
        analyseErrorLogAppend(la, ERROR_STEP_OMISSION, "LightMatch", "matchbox", "match");
    }
    if (!state->candle_lit) {
        analyseErrorLogAppend(la, ERROR_STEP_OMISSION, "LightCandle", "candle", "match");
    }
    if ((state->match_lit) && (!state->match_extinguished)) {
        analyseErrorLogAppend(la, ERROR_STEP_OMISSION, "ExtinguishMatch", "match", NULL);
    }
    if (!state->box_closed) {
        analyseErrorLogAppend(la, ERROR_STEP_OMISSION, "CloseBox", "matchbox", NULL);
    }
}

/******************************************************************************/

void analyse_candle_task(DataAnalysis *la)
{
    ActionList *as;
    CandleState state;

    state_initialise(&state);

    for (as = la->actionSequence; as; as = as->next) {
        as->checked = FALSE;
    }

    for (as = la->actionSequence; as; as = as->next) {
        if (!as->checked) {
            score_action(&state, la, as);
            as->checked = TRUE;
        }
    }

    score_omissions(&state, la);

    state_destroy(&state);
}

/******************************************************************************/
