/*******************************************************************************

    FILE: candle/goal.c
    AUTHOR: R. Cooper
    CREATION DATE: Tue Aug  3 13:47:01 2004
    CONTENTS: Goal functions for the candle task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

GoalType goalCandleFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isCandle));
}

GoalType goalCandleHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isCandle));
}

GoalType goalCandleLit(ContentionScheduler *cs)
{
    GoalType found = GOAL_FALSE;
    ObjectList *os;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isLitCandle(os->head));
    }
    return(found);
}

GoalType goalMatchFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isMatch));
}

GoalType goalMatchHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isMatch));
}

GoalType goalMatchLit(ContentionScheduler *cs)
{
    GoalType found = GOAL_FALSE;
    ObjectList *os;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isLitMatch(os->head));
    }
    return(found);
}

GoalType goalMatchExtinguished(ContentionScheduler *cs)
{
    GoalType found = GOAL_FALSE;
    ObjectList *os;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isUsedMatch(os->head));
    }
    return(found);
}

GoalType goalExtinguishedMatchOnTable(ContentionScheduler *cs)
{
    return(goalOnTable(cs, isUsedMatch));
}

GoalType goalLitMatchFixated(ContentionScheduler *cs)
{
    return((cs->fixated_object == NULL) ? GOAL_FALSE : FZAND(isLit(cs->fixated_object), isMatchHead(cs->fixated_object)));
}

GoalType goalLitObjectFixated(ContentionScheduler *cs)
{
    return((cs->fixated_object == NULL) ? GOAL_FALSE : isLit(cs->fixated_object));
}

GoalType goalFixatedObjectExtinguished(ContentionScheduler *cs)
{
    return((cs->fixated_object == NULL) ? GOAL_FALSE : isUsed(cs->fixated_object));
}

GoalType goalExtinguishedObjectOnTable(ContentionScheduler *cs)
{
    return(goalOnTable(cs, isUsed));
}

GoalType goalCandleLitAndExtinguishedMatchOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalCandleLit(cs), goalExtinguishedMatchOnTable(cs)));
}

GoalType goalMatchBoxFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isMatchBox));
}

GoalType goalMatchBoxHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isMatchBox));
}

GoalType goalMatchBoxOpen(ContentionScheduler *cs)
{
    GoalType found = GOAL_FALSE;
    ObjectList *os;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isOpenMatchBox(os->head));
    }
    return(found);
}

GoalType goalMatchBoxClosed(ContentionScheduler *cs)
{
    GoalType found = GOAL_FALSE;
    ObjectList *os;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isClosedMatchBox(os->head));
    }
    return(found);
}

GoalType goalCandleInstalled(ContentionScheduler *cs)
{
    GoalType found = GOAL_FALSE;
    ObjectList *os;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isCandleStick(os->head), hasAttachment(os->head)));
    }
    return(found);
}

GoalType goalCandleStickFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isCandleStick));
}

GoalType goalMatchLitAndMatchBoxOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalMatchLit(cs), goalOnTable(cs, isMatchBox)));
}

GoalType goalMatchHeldAndMatchBoxClosed(ContentionScheduler *cs)
{
    return(FZAND(goalMatchHeld(cs), goalMatchBoxClosed(cs)));
}

GoalType goalCandleHeldAndStickFixated(ContentionScheduler *cs)
{
    return(FZAND(goalCandleHeld(cs), goalCandleStickFixated(cs)));
}

/******************************************************************************/
