/*******************************************************************************

    File: candle/schema.c
    AUTHOR: R. Cooper
    CREATION DATE: Tue Aug  3 13:46:26 2004
    CONTENTS: Schemas for the candle task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

void install_candle_schemas(ContentionScheduler *cs)
{
    Schema *s;

    s = install_schema(cs, "schemaCandleTask", "goalLightCandleTask", ACTION_NONE, HIGH);
    install_schema_object_trigger(s, SOURCE, schemaCandleTaskTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaCandleTaskTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaCandleTaskTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaCandleTaskTriggersImplement);
    install_schema_effector_trigger(s, schemaCandleTaskTriggersEffector);
    install_schema_subgoal(s, "goalInstallCandle", goalTrue, goalCandleInstalled);
    install_schema_subgoal(s, "goalLightMatch", goalCandleInstalled, goalMatchLitAndMatchBoxOnTable);
    install_schema_subgoal(s, "goalLightCandle", goalMatchLit, goalCandleLit);
    install_schema_subgoal(s, "goalExtinguishMatch", goalCandleLit, goalCandleLitAndExtinguishedMatchOnTable);
    install_schema_scope(s, IMPLEMENT);

    cs->intended_schema = s;

    /* Intermediate level schemas: */

    s = install_schema(cs, "schemaInstallCandle", "goalInstallCandle", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaInstallCandleTriggersTheme);
    install_schema_object_trigger(s, TARGET, schemaInstallCandleTriggersTarget);
    install_schema_effector_trigger(s, schemaInstallCandleTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalCandleHeld);
    install_schema_subgoal(s, "goalInsertCandle", goalCandleHeld, goalCandleInstalled);
    install_schema_scope(s, TARGET);
    install_schema_scope(s, THEME);

    s = install_schema(cs, "schemaLightMatch", "goalLightMatch", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaLightMatchTriggersSource);
    install_schema_object_trigger(s, IMPLEMENT, schemaLightMatchTriggersImplement);
    install_schema_effector_trigger(s, schemaLightMatchTriggersEffector);
    install_schema_subgoal(s, "goalOpenSourceBox", goalTrue, goalMatchBoxOpen);
    install_schema_subgoal(s, "goalPickUpImplement", goalMatchBoxOpen, goalMatchHeld);
    install_schema_subgoal(s, "goalCloseSourceBox", goalMatchHeld, goalMatchHeldAndMatchBoxClosed);
    install_schema_subgoal(s, "goalStrikeMatch", goalMatchHeldAndMatchBoxClosed, goalMatchLit);
    install_schema_subgoal(s, "goalPutDownSource", goalMatchLit, goalMatchLitAndMatchBoxOnTable);
    install_schema_scope(s, SOURCE);
    install_schema_scope(s, TARGET);

    s = install_schema(cs, "schemaLightCandle", "goalLightCandle", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaLightCandleTriggersTarget);
    install_schema_object_trigger(s, IMPLEMENT, schemaLightCandleTriggersImplement);
    install_schema_effector_trigger(s, schemaLightCandleTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalMatchToCandle", goalCurrentTargetFixated, goalCandleLit);
    install_schema_scope(s, TARGET);

    s = install_schema(cs, "schemaExtinguishMatch", "goalExtinguishMatch", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, IMPLEMENT, schemaExtinguishMatchTriggersImplement);
    install_schema_effector_trigger(s, schemaExtinguishMatchTriggersEffector);
    install_schema_subgoal(s, "goalFixateImplement", goalTrue, goalLitMatchFixated);
    install_schema_subgoal(s, "goalBlowOutFlame", goalLitMatchFixated, goalFixatedObjectExtinguished);
    install_schema_subgoal(s, "goalPutDown", goalFixatedObjectExtinguished, goalExtinguishedObjectOnTable);
    install_schema_scope(s, IMPLEMENT);

    /* 2nd level intermediate schemas: */

    s = install_schema(cs, "schemaOpenSourceBox", "goalOpenSourceBox", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaOpenSourceBoxTriggersSource);
    install_schema_effector_trigger(s, schemaOpenSourceBoxTriggersEffector);
    install_schema_subgoal(s, "goalPickUpSource", goalTrue, goalMatchBoxHeld);
    install_schema_subgoal(s, "goalOpenContainer", goalMatchBoxHeld, goalMatchBoxOpen);

    s = install_schema(cs, "schemaCloseSourceBox", "goalCloseSourceBox", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaCloseSourceBoxTriggersSource);
    install_schema_effector_trigger(s, schemaCloseSourceBoxTriggersEffector);
    install_schema_subgoal(s, "goalPickUpSource", goalTrue, goalMatchBoxHeld);
    install_schema_subgoal(s, "goalCloseContainer", goalMatchBoxHeld, goalMatchBoxClosed);

    s = install_schema(cs, "schemaPickUpSource", "goalPickUpSource", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaPickUpSourceTriggersSource);
    install_schema_effector_trigger(s, schemaPickUpSourceTriggersEffector);
    install_schema_subgoal(s, "goalFixateSource", goalTrue, goalCurrentSourceFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentSourceFixated, goalCurrentSourceHeld);

    s = install_schema(cs, "schemaPickUpTheme", "goalPickUpTheme", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaPickUpThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaPickUpThemeTriggersEffector);
    install_schema_subgoal(s, "goalFixateTheme", goalTrue, goalCurrentThemeFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentThemeFixated, goalCurrentThemeHeld);

    s = install_schema(cs, "schemaPutDownSource", "goalPutDownSource", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaPutDownSourceTriggersSource);
    install_schema_effector_trigger(s, schemaPutDownSourceTriggersEffector);
    install_schema_subgoal(s, "goalPutDown", goalTrue, goalCurrentSourceOnTable);

    s = install_schema(cs, "schemaInsertCandle", "goalInsertCandle", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaInsertCandleTriggersTarget);
    install_schema_effector_trigger(s, schemaInsertCandleTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalCandleHeld, goalCandleHeldAndStickFixated);
    install_schema_subgoal(s, "goalPlaceCandleInStick", goalCandleHeldAndStickFixated, goalCandleInstalled);

    s = install_schema(cs, "schemaPickUpImplement", "goalPickUpImplement", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpImplementTriggersEffector);
    install_schema_subgoal(s, "goalFixateImplement", goalTrue, goalCurrentImplementFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentImplementFixated, goalCurrentImplementHeld);

    s = install_schema(cs, "schemaStrikeMatch", "goalStrikeMatch", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaStrikeMatchTriggersTarget);
    install_schema_object_trigger(s, IMPLEMENT, schemaStrikeMatchTriggersImplement);
    install_schema_effector_trigger(s, schemaStrikeMatchTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalStrikeTarget", goalCurrentTargetFixated, goalMatchLit);

    /* Low level schemas: */

    s = install_schema(cs, "schemaFixateSource", "goalFixateSource", ACTION_FIXATE_SOURCE, LOW);
    install_schema_object_trigger(s, SOURCE, schemaFixateSourceTriggersSource);
    install_schema_effector_trigger(s, schemaFixateSourceTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTarget", "goalFixateTarget", ACTION_FIXATE_TARGET, LOW);
    install_schema_object_trigger(s, TARGET, schemaFixateTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaFixateTargetTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateImplement", "goalFixateImplement", ACTION_FIXATE_IMPLEMENT, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaFixateImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaFixateImplementTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTheme", "goalFixateTheme", ACTION_FIXATE_THEME, LOW);
    install_schema_object_trigger(s, THEME, schemaFixateThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaFixateThemeTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPickUp", "goalPickUp", ACTION_PICK_UP, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPickUpTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPickUpTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPickUpTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPutDown", "goalPutDown", ACTION_PUT_DOWN, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPutDownTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPutDownTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPutDownTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPutDownTriggersImplement);
    install_schema_effector_trigger(s, schemaPutDownTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPlaceCandleInStick", "goalPlaceCandleInStick", ACTION_INSERT_CANDLE, LOW);
    install_schema_object_trigger(s, TARGET, schemaPlaceCandleInStickTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPlaceCandleInStickTriggersTheme);
    install_schema_effector_trigger(s, schemaPlaceCandleInStickTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaSlideOpenBox", "goalOpenContainer", ACTION_SLIDE_OPEN, LOW);
    install_schema_object_trigger(s, SOURCE, schemaOpenContainerBySlidingTriggersSource);
    install_schema_effector_trigger(s, schemaOpenContainerBySlidingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaSlideCloseBox", "goalCloseContainer", ACTION_SLIDE_CLOSED, LOW);
    install_schema_object_trigger(s, SOURCE, schemaCloseContainerBySlidingTriggersSource);
    install_schema_effector_trigger(s, schemaCloseContainerBySlidingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPeelOpen", "goalOpenContainer", ACTION_PEEL_OPEN, LOW);
    install_schema_object_trigger(s, SOURCE, schemaOpenContainerByPeelingTriggersSource);
    install_schema_effector_trigger(s, schemaOpenContainerByPeelingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaTearOpen", "goalOpenContainer", ACTION_TEAR_OPEN, LOW);
    install_schema_object_trigger(s, SOURCE, schemaOpenContainerByTearingTriggersSource);
    install_schema_effector_trigger(s, schemaOpenContainerByTearingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaStrikeTarget", "goalStrikeTarget", ACTION_STRIKE_MATCH, LOW);
    install_schema_object_trigger(s, TARGET, schemaStrikeTargetTriggersTarget);
    install_schema_object_trigger(s, IMPLEMENT, schemaStrikeTargetTriggersImplement);
    install_schema_effector_trigger(s, schemaStrikeTargetTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaMatchToCandle", "goalMatchToCandle", ACTION_MATCH_TO_CANDLE, LOW);
    install_schema_object_trigger(s, TARGET, schemaMatchToCandleTriggersTarget);
    install_schema_object_trigger(s, IMPLEMENT, schemaMatchToCandleTriggersImplement);
    install_schema_effector_trigger(s, schemaMatchToCandleTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaBlowOutFlame", "goalBlowOutFlame", ACTION_BLOW_OUT_MATCH, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaBlowOutFlameTriggersImplement);
    install_schema_effector_trigger(s, schemaBlowOutFlameTriggersEffector);
    install_schema_resource(s, HAND);
}

/******************************************************************************/
