/*******************************************************************************

    File: cs.c
    Contents: A text-based front end to the CS model
    This file (c) R.Cooper 2001

*******************************************************************************/

#include "cs.h"
#include "analyse.h"
#include "file_misc.h"

#include <stdio.h>

/******************************************************************************/
/* Global variables: **********************************************************/

#include "globals.h"

#define WRITE_PAR_VALUES
#define INDEX_FILE "index0.txt"

/*----------------------------------------------------------------------------*/

void action_log_add_warning(ContentionScheduler *cs, char *type, char *warning) { }
void action_log_add_action(ContentionScheduler *cs, char *act, char *message) { }
void action_log_add_selection(ContentionScheduler *cs, char *schema_name, Boolean on) { }
void record_schema_activity(ContentionScheduler *cs, char *name, Boolean selected,  double before, double s, double l, double p, double e, double in, double after, char *note) { }
void record_object_activity(ContentionScheduler *cs, char *name, int r, Boolean selected,  double before, double s, double l, double p, double e, double in, double after, char *note) { }
void record_effector_activity(ContentionScheduler *cs, char *name, Boolean selected, double before, double s, double l, double p, double e, double in, double after, char *note) { }
void post_cycle_trace(ContentionScheduler *cs) { }
void attach_data(ContentionScheduler *cs) { }
void process_gtk_events() {}

/*----------------------------------------------------------------------------*/

static void write_index_headings(char *tmpdir)
{
    char indexfile[32];
    FILE *fp;

    g_snprintf(indexfile, 32, "%s/%s", tmpdir, INDEX_FILE);

    if ((fp = fopen(indexfile, "w")) != NULL) {

        fprintf(fp, "        File");
#ifdef WRITE_PAR_VALUES
        fprintf(fp, "\tSelfS");
        fprintf(fp, "\tLatS");
        fprintf(fp, "\tParS");
        fprintf(fp, "\tEnvS");
        fprintf(fp, "\tNoiseS");
        fprintf(fp, "\tSelfO");
        fprintf(fp, "\tLatO");
        fprintf(fp, "\tParO");
        fprintf(fp, "\tEnvO");
        fprintf(fp, "\tNoiseO");
        fprintf(fp, "\tSelfE");
        fprintf(fp, "\tLatE");
        fprintf(fp, "\tParE");
        fprintf(fp, "\tEnvE");
        fprintf(fp, "\tNoiseE");

        fprintf(fp, "\tPers");
        fprintf(fp, "\tSelTh");
        fprintf(fp, "\tRest");
#endif
        fprintf(fp, "\tActions");
        fprintf(fp, "\tErrors");
        fprintf(fp, "\tInterVs");

        fprintf(fp, "\tOmis");
        fprintf(fp, "\tAnts");
        fprintf(fp, "\tPersevs");
        fprintf(fp, "\tAdds");

        fprintf(fp, "\tMisUse1");
        fprintf(fp, "\tMisUse2");
        fprintf(fp, "\tMisLoc1");
        fprintf(fp, "\tMisLoc2");
        fprintf(fp, "\tToolOm");
        fprintf(fp, "\tObjSub");
        fprintf(fp, "\tPanto");
        fprintf(fp, "\tPerplx");
        fprintf(fp, "\tToying");

        fprintf(fp, "\n");

        fclose(fp);
    }
}

static void write_index_line(char *tmpdir, int uid, DataAnalysis *la)
{
    char indexfile[32];
    FILE *fp;

    g_snprintf(indexfile, 32, "%s/%s", tmpdir, INDEX_FILE);

    if ((fp = fopen(indexfile, "a")) != NULL) {

        fprintf(fp, "tmp.%d.?:", uid);
#ifdef WRITE_PAR_VALUES
        fprintf(fp, "\t%6.4f", self[SCHEMA]);
        fprintf(fp, "\t%6.4f", lateral[SCHEMA]);
        fprintf(fp, "\t%6.4f", parental[SCHEMA]);
        fprintf(fp, "\t%6.4f", environmental[SCHEMA]);
        fprintf(fp, "\t%6.4f", noiseSD[SCHEMA]);
        fprintf(fp, "\t%6.4f", self[OBJECT]);
        fprintf(fp, "\t%6.4f", lateral[OBJECT]);
        fprintf(fp, "\t%6.4f", parental[OBJECT]);
        fprintf(fp, "\t%6.4f", environmental[OBJECT]);
        fprintf(fp, "\t%6.4f", noiseSD[OBJECT]);
        fprintf(fp, "\t%6.4f", self[EFFECTOR]);
        fprintf(fp, "\t%6.4f", lateral[EFFECTOR]);
        fprintf(fp, "\t%6.4f", parental[EFFECTOR]);
        fprintf(fp, "\t%6.4f", environmental[EFFECTOR]);
        fprintf(fp, "\t%6.4f", noiseSD[EFFECTOR]);

        fprintf(fp, "\t%6.4f", persistence);
        fprintf(fp, "\t%6.4f", selectionThreshold);
        fprintf(fp, "\t%6.4f", restAct);
#endif
        fprintf(fp, "\t%d", la->numActions);
        fprintf(fp, "\t%d", la->numErrors);
        fprintf(fp, "\t%d", la->numInterventions);

        fprintf(fp, "\t%d", la->numStepOmissions);
        fprintf(fp, "\t%d", la->numActionAnticipations);
        fprintf(fp, "\t%d", la->numPerseverations);
        fprintf(fp, "\t%d", la->numActionAdditions);

        fprintf(fp, "\t%d", la->numMisUse1);
        fprintf(fp, "\t%d", la->numMisUse2);
        fprintf(fp, "\t%d", la->numMisLocation1);
        fprintf(fp, "\t%d", la->numMisLocation2);
        fprintf(fp, "\t%d", la->numToolOmissions);
        fprintf(fp, "\t%d", la->numObjectSubstitution);
        fprintf(fp, "\t%d", la->numPantomime);
        fprintf(fp, "\t%d", la->numPerplexity);
        fprintf(fp, "\t%d", la->numToying);

        fprintf(fp, "\n");

        fclose(fp);
    }
}

/*----------------------------------------------------------------------------*/

static void initialise_error_counts(DataAnalysis *results)
{
    results->numActions = 0;
    results->numErrors = 0;
    results->numInterventions = 0;
    results->numAccomplished = 0;
    results->numActionAdditions = 0;
    results->numActionAnticipations = 0;
    results->numStepOmissions = 0;
    results->numPerseverations = 0;
    results->numToolOmissions = 0;
    results->numMisUse1 = 0;
    results->numMisUse2 = 0;
    results->numMisLocation1 = 0;
    results->numMisLocation2 = 0;
    results->numObjectSubstitution = 0;
    results->numToying = 0;
    results->numPantomime = 0;
    results->numPerplexity = 0;
    results->numCycles = 0;
}

static void accumulate_results(DataAnalysis *results, DataAnalysis *la)
{
    results->numActions += la->numActions;
    results->numErrors += la->numErrors;
    results->numInterventions += la->numInterventions;
    results->numAccomplished += la->numAccomplished;
    results->numActionAdditions += la->numActionAdditions;
    results->numActionAnticipations += la->numActionAnticipations;
    results->numStepOmissions += la->numStepOmissions;
    results->numPerseverations += la->numPerseverations;
    results->numToolOmissions += la->numToolOmissions;
    results->numMisUse1 += la->numMisUse1;
    results->numMisUse2 += la->numMisUse2;
    results->numMisLocation1 += la->numMisLocation1;
    results->numMisLocation2 += la->numMisLocation2;
    results->numObjectSubstitution += la->numObjectSubstitution;
    results->numToying += la->numToying;
    results->numPantomime += la->numPantomime;
    results->numPerplexity += la->numPerplexity;
    results->numCycles += la->numCycles;
}

static void print_line_list(FILE *fp, DataAnalysis *la)
{
    while (la->log != NULL) {
        LineList *tmp = la->log->next;
        fprintf(fp, "%s\n", la->log->line);
        la->log = tmp;
    }
}

#ifdef STANDALONE
static void analyse_file(ContentionScheduler *cs, char *file, TaskTypeId task, DataAnalysis *results)
{
    char tmpfile[64];
    DataAnalysis la;

    la.actionSequence = NULL;
    la.log = NULL;

    initialise_task(task, &(cs->task));
    analyse_results(file, &(cs->task), &la);
    print_line_list(stdout, &la);
    accumulate_results(results, &la);
}
#endif

static void run_and_analyse_task(ContentionScheduler *cs, char *tmpdir, int uid, int n, TaskTypeId task, DataAnalysis *results)
{
    char tmpfile[64];
    DataAnalysis la;

    la.actionSequence = NULL;
    la.log = NULL;

    initialise_task(task, &(cs->task));
    g_snprintf(tmpfile, 64, "%s/tmp.%d.%d.seed", tmpdir, uid, n);
    random_seed_save(tmpfile, cs);
    g_snprintf(tmpfile, 64, "%s/tmp.%d.%d", tmpdir, uid, n);
    reinitialise_contention_scheduler(cs, tmpfile);
    run(cs, FALSE);
    analyse_results(tmpfile, &(cs->task), &la);
    print_line_list(stdout, &la);
    accumulate_results(results, &la);
}

int main(int argc, char **argv)
{
    extern int getpid();

    ContentionScheduler *cs = create_contention_scheduler();
    DataAnalysis results;
    char tmpdir[16];
    int uid;
    int k;

    results.actionSequence = NULL;
    results.log = NULL;

    initialise_task(NULL_TASK, &(cs->task));
    initialise_contention_scheduler(cs);

#ifdef STANDALONE
    analyse_file(cs, "3478/tmp.8.2", ESPRESSO_BASIC, &results);
#else
    messages[RUN] = NULL; // stdout;

    noiseSD[SCHEMA] = 0.005;
    noiseSD[OBJECT] = 0.005;
    noiseSD[EFFECTOR] = 0.005;

    g_snprintf(tmpdir, 16, "%d", getpid());
    make_directory(tmpdir);
    write_index_headings(tmpdir);

    for (k = 0; k < 10; k++) {
        noiseSD[SCHEMA] = 0.000;
        noiseSD[OBJECT] = 0.000;
        for (uid = 0; uid < 10; uid++) {
    
            noiseSD[SCHEMA] += 0.05;
            noiseSD[OBJECT] += 0.05;
            initialise_error_counts(&results);
            run_and_analyse_task(cs, tmpdir, uid, 0, LETTER_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 1, JUICE_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 2, ESPRESSO_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 3, CANDLE_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 4, BOTTLE_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 5, LETTER_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 6, JUICE_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 7, ESPRESSO_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 8, CANDLE_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 9, BOTTLE_BASIC, &results);
            write_index_line(tmpdir, uid, &results);
	}
    }
#endif
    destroy_contention_scheduler(cs);
    exit(0);
}

/******************************************************************************/
