
#ifndef _cs_h

#define _cs_h

/******************************************************************************/
/* Include files: ------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef MALLOC_CHECK
#include "zmalloc.h"
#endif

/******************************************************************************/

typedef enum {
    NULL_TASK, LETTER_BASIC, JUICE_BASIC, ESPRESSO_BASIC, CANDLE_BASIC,
    BOTTLE_BASIC, LUNCH_BASIC, LUNCH_DISTRACTOR, TT_MAX
} TaskTypeId;

typedef enum actionType {
    ACTION_NONE, ACTION_SCREW_OPEN, ACTION_SCREW_CLOSED, ACTION_UNSEAL, ACTION_SEAL, 
    ACTION_FIXATE_SOURCE, ACTION_FIXATE_TARGET, ACTION_FIXATE_THEME, ACTION_FIXATE_IMPLEMENT,
    ACTION_PICK_UP, ACTION_PUT_DOWN, ACTION_CAP, ACTION_SWAP,
    ACTION_TAKE_FROM, ACTION_TAKE2_FROM, ACTION_POUR_SOME, ACTION_DIP, ACTION_SPREAD,
    ACTION_TEAR, ACTION_COVER, ACTION_WRAP, ACTION_PACK, ACTION_UNIFY,
    ACTION_FREE_HANDS, ACTION_OPEN_LID, ACTION_CLOSE_LID, ACTION_FLIP,
    ACTION_EAT, ACTION_DRINK, ACTION_POKE, ACTION_FOLD, ACTION_CHOOSE_SLICE, 
    ACTION_PLACE_STAMP, ACTION_WRITE_ADDRESS, ACTION_FOLD_LETTER, 
    ACTION_PACK_ENVELOPE, ACTION_SEAL_ENVELOPE, ACTION_MOISTEN_WITH_TOUNGE, ACTION_TUCK,
    ACTION_SCOOP, ACTION_EMPTY_SCOOP, ACTION_SCREW_POT_CLOSED, ACTION_SCREW_POT_OPEN,
    ACTION_POUR_ALL, ACTION_CUT_BY_SAWING, ACTION_CUT_BY_PUSHING,
    ACTION_SQUEEZE_BY_TWISTING, ACTION_SQUEEZE_BY_PUSHING, ACTION_SQUEEZE_BY_HAND,
    ACTION_INSERT_CANDLE, ACTION_STRIKE_MATCH, ACTION_MATCH_TO_CANDLE, ACTION_BLOW_OUT_MATCH,
    ACTION_SLIDE_OPEN, ACTION_SLIDE_CLOSED, ACTION_TEAR_OPEN, ACTION_PEEL_OPEN,
    ACTION_UNCORK, ACTION_RECORK,
    ACTION_BOUNCE_BALL
} ActionType;

typedef enum {HAND, RESOURCES_MAX} Resource;

#define MIN(A,B)       ((A)<(B)?(A):(B))
#define MAX(A,B)       ((A)>(B)?(A):(B))

#define FZNOT(A)   	(1.0 - (A))
#define FZAND(A,B) 	MIN(A,B)
#define FZOR(A,B)  	MAX(A,B)

#include "cs_base.h"

/******************************************************************************/
/* Global Constants: ---------------------------------------------------------*/

#define MaxCycles       2500
#define MaxRoles           5
#define MaxComponents      3
#define MaxMessageTypes    7

/******************************************************************************/
/* Global Type Definitions: --------------------------------------------------*/

typedef enum {FALSE, TRUE} Boolean;
typedef enum {SCHEMA, OBJECT, EFFECTOR} ComponentType;
typedef enum {SIGMOID, TANH, HALFX, M_R, HOUGHTON, NORMALISED} MathsType;
typedef enum {RUN, SELECTION, BIND, WARNING, WORLD, MONITOR} MessageType;
typedef enum {UNKNOWN_LEVEL, HIGH, MEDIUM, LOW} SchemaLevel;
typedef enum {CURRENT, SUM, SUMSQ, AVERAGE, STDDEV} CountType;
typedef enum {SOURCE, TARGET, THEME, IMPLEMENT, EMPTY} ObjectRole;

/******************************************************************************/
/* Global Functions: **********************************************************/

extern Boolean allocCheck(void *p, char *d);
extern Boolean nullCheck(void *p, char *d);
extern void exitMessage(char *d);
extern Boolean stoi(char *string, int *intptr);
extern void printSpace(int i, FILE *fp);
extern FILE *update_trial_number(int *i);

/* Object features: ***********************************************************/

#define IS_TOOL                   1
#define IS_CONTAINER              2
#define IS_LID                    4
#define IS_HANDLE                 8

#define IS_SMALL                 16
#define IS_MEDIUM                32
#define IS_LARGE                 64

#define IS_FLUID                128
#define IS_SHARP                256
#define IS_BLUNT                512

#define IS_FLAT                1024
#define IS_DISK                2048
#define IS_SQUARE              4096
#define IS_EXTENDED            8192
#define IS_SPHERE             16384
#define IS_CYLINDRICAL        32768
#define IS_HEMISPHERE         65536
#define IS_ORANGE_COLOURED   131072

#define IS_METAL             262144
#define IS_GLASS             524288
#define IS_PLASTIC          1048576

#define HOLESIZE_1          2097152
#define HOLESIZE_2          4194304
#define HAS_SCREW_THREAD    8388608
#define HAS_DRAWER         16777216
#define IS_DRAWER          33554432
#define IS_FRONT           67108864
#define IS_BACK           134217728
#define HAS_WICK          268435456

/**
536870912
1073741824
**/

/* State info: ****************************************************************/

#define STATE_OPEN                1
#define STATE_CLOSED              2
#define STATE_MOIST               4
#define STATE_ADDRESSED           8
#define STATE_FOLDED             16
#define STATE_STAMPED            32
#define STATE_CUT                64
#define STATE_LIT               128
#define STATE_USED              256
#define STATE_BOUNCED           512

/******************************************************************************/
/* Class Definitions: *********************************************************/

typedef struct task_type {
    TaskTypeId          id;
    void (*install_schemas)();
    void (*install_objects)();
    void (*install_effectors)();
    void (*write_object_attributes)();
    void (*write_effector_attributes)();
    void (*analyse)();
} TaskType;

typedef struct contention_scheduler {
    TaskType              task;
    int                   cycle;
    Boolean               data_logging;
    Boolean               utilised[MaxRoles];
    char                 *logfile;
    struct schema_list   *schemas;
    struct object_list   *objects;
    struct effector_list *effectors;
    struct schema        *intended_schema;
    struct object        *current_object[MaxRoles];
    struct object        *fixated_object;
    struct schema_list   *update_goal_queue;
    struct schema_list   *selection_queue;
    struct schema_list   *deselection_queue;
    struct schema_list   *execution_queue;
    struct actions       *actions;
    void                 *cl_actions;
    void                 *cl_analysis;
    void                 *cl_schemas;
    void                 *cl_objects[MaxRoles];
    void                 *cl_effectors;
} ContentionScheduler;

typedef struct goal {
    char *name;
    Boolean explicitly_achieved;
    GoalType (*precondition)();
    GoalType (*postcondition)();
} Goal;

typedef struct goal_list {
    Goal *head;
    struct goal_list *tail;
} GoalList;

typedef struct object {
    char *name;
    double activation[MaxRoles][MaxCycles];
    double (*trigger)(ObjectRole r);
    Boolean inhibit[MaxRoles];
    /* Physical aspects of the object: */
    unsigned int features;
    unsigned int state;
    double position[2];
    struct effector *holder;
    struct object_list *itemsContained,  *itemsCarried, *itemsAttached;
    struct object *container, *carrier, *base;
    /* For purposes of the graph: */
    int time0;
    Boolean highlighted;
} Object;

typedef struct object_list {
    Object *head;
    struct object_list *tail;
} ObjectList;

typedef struct effector {
    char *name;
    double activation[MaxCycles];
    ObjectList *itemsHeld;
    Boolean highlighted;
} Effector;

typedef struct effector_list {
    Effector *head;
    struct effector_list *tail;
} EffectorList;

typedef struct object_trigger_function_list {
    ObjectRole role;
    double (*trigger)(ContentionScheduler *cs, Object *o);
    struct object_trigger_function_list *next;
} ObjectTriggerFunctionList;

typedef struct schema {
    char *name;
    char *goal;
    ObjectTriggerFunctionList *objectTriggers;
    double (*effectorTrigger)(ContentionScheduler *cs, Effector *e);
    double activation[MaxCycles];
    Boolean selected;
    Boolean executed;
    Boolean scope[MaxRoles-1];
    Boolean resource[RESOURCES_MAX];
    ActionType corresponding_action;
    GoalList *subgoals;
    struct schema_list *competitors;
    SchemaLevel level;
    Boolean highlighted;
} Schema;

typedef struct schema_list {
    Schema *head;
    struct schema_list *tail;
} SchemaList;

typedef struct actions {
    ActionType          action;
    Object             *obj1;
    Effector           *effector1;
    Effector           *effector2;
    struct actions     *next;
} Actions;

/******************************************************************************/
/* Function declarations: ----------------------------------------------------*/

extern double situationTriggersObject(Object *o, ObjectRole r);

/* In component.c: */

extern Object  *findObject(ContentionScheduler *cs, char *n);
extern Effector *findEffector(ContentionScheduler *cs, char *n);
extern Effector *bestEmptyHand(ContentionScheduler *cs);
extern Effector *bestNonemptyHand(ContentionScheduler *cs);

extern int random_index(int listLength);

/* In effector.c */

extern Effector *initialise_effector(char n[], Boolean s);
extern void destroy_effector(Effector *e);

/* In scheduler.c */

extern ContentionScheduler *create_contention_scheduler();
extern void initialise_contention_scheduler(ContentionScheduler *cs);
extern void reinitialise_contention_scheduler(ContentionScheduler *cs, char *logfile);
extern void destroy_contention_scheduler(ContentionScheduler *cs);

extern void run(ContentionScheduler *cs, Boolean evolve);
extern void step(ContentionScheduler *cs, int n, Boolean evolve);

/* In xframe.c */

extern int usertime();
extern int systime();
extern void attach_data(ContentionScheduler *cs);
extern void update_cycle_display(ContentionScheduler *cs);

extern double initial_activation(ComponentType role);
extern void normalise_schema_activations(ContentionScheduler *cs);
extern void normalise_object_activations(ContentionScheduler *cs);
extern void normalise_effector_activations(ContentionScheduler *cs);

extern void schema_calculate_activation(ContentionScheduler *cs, Schema *this);
extern void object_calculate_activation(ContentionScheduler *cs, Object *this);
extern void effector_calculate_activation(ContentionScheduler *cs, Effector *this);

extern Boolean object_can_serve_role(Object *this, ObjectRole role);
extern Boolean schema_goal_achieved(ContentionScheduler *cs, Schema *this);

extern void message(MessageType c, ContentionScheduler *cs, char *b);

/******************************************************************************/

#define minAct 0.0
#define maxAct 1.0

extern double restAct;
extern double selectionThreshold;
extern double persistence;
extern double goalCutoff;
extern double goalPrecision;
extern MathsType maths;
extern Boolean interventions;
extern double parental[MaxComponents];
extern double environmental[MaxComponents];
extern double self[MaxComponents];
extern double lateral[MaxComponents];
extern double noiseSD[MaxComponents];

extern ContentionScheduler *scheduler;
extern FILE *messages[MaxMessageTypes];
extern char *roleName[MaxRoles];

/******************************************************************************/
/* Schema installation functions: *********************************************/

extern void install_schema_subgoal(Schema *s, char *goal_name, GoalType postcondition(), GoalType precondition());
extern void install_schema_scope(Schema *s, ObjectRole role);
extern void install_schema_resource(Schema *s, Resource r);
extern Schema *install_schema(ContentionScheduler *cs, char *schema_name, char *goal_name, ActionType act, SchemaLevel level);
extern void install_schema_object_trigger(Schema *s, ObjectRole role, double object_trigger());
extern void install_schema_effector_trigger(Schema *s, double effector_trigger());

/* Object installation functions: *********************************************/

extern Object *objectCreate(ContentionScheduler *cs, char *object_name, unsigned int features, int state);
extern void objectDestroy(ContentionScheduler *cs, Object *o);

extern Object *objectInstall(ContentionScheduler *cs, char *object_name, unsigned int features, int state);
extern Object *objectInstallItem(ContentionScheduler *cs, char *name, unsigned int features);
extern void objectInstallAttachment(Object *object, Object *attached);
extern void objectInstallContents(Object *object, Object *content);
extern void objectInstallCover(Object *object, Object *cover);

extern void objectRemoveCover(Object *this, Object *cover);
extern void objectDeleteAttached(Object *base, Object *attached);
extern void objectDeleteCarried(Object *carrier, Object *carried);
extern void objectDeleteContained(Object *container, Object *contained);

extern Object *objectGetAttachment(Object *this);
extern Object *objectGetCover(Object *this);
extern Object *objectGetContent(Object *this);
extern Object *objectGetMostActive(ContentionScheduler *cs, ObjectRole r);

extern void objectSetPosition(Object *object, double x, double y);

extern ObjectList *objectListDeleteElement(ObjectList *ol, Object *theme);
extern ObjectList *objectListAppendElement(ObjectList *ol, Object *theme);

extern void objectBindToRole(ContentionScheduler *cs, Object *object, ObjectRole role);

extern Boolean isInSubordinateContactWith(Object *this, Object *theme);

/* Effector installation functions: *******************************************/

extern Effector *effectorInstall(ContentionScheduler *cs, char *effector_name);

extern void effectorDeleteHeld(Effector *effector, Object *theme);
extern void effectorAddItemToContents(Effector *hand, Object *item);
extern void effectorDiscardContents(ContentionScheduler *cs, Effector *effector);
extern int effectorDiscardCoordinate(Effector *hand);

extern Object *effectorGetContents(Effector *hand);
extern Effector *effectorGetWorst(ContentionScheduler *cs);
extern Effector *effectorGetOtherHand(ContentionScheduler *cs, Effector *hand);

/******************************************************************************/

extern void initialise_schemas(ContentionScheduler *cs);
extern void initialise_objects(ContentionScheduler *cs);
extern void initialise_effectors(ContentionScheduler *cs);
extern void reinitialise_schemas(ContentionScheduler *cs);
extern void reinitialise_objects(ContentionScheduler *cs);
extern void reinitialise_effectors(ContentionScheduler *cs);
extern void random_seed_load(char *filename, ContentionScheduler *cs);
extern void random_seed_save(char *filename, ContentionScheduler *cs);

extern void initialise_task(TaskTypeId id, TaskType *task);

extern char *task_names[TT_MAX];

/******************************************************************************/

extern GoalType isUsed(Object *o);
extern GoalType wasBounced(Object *o);
extern GoalType isHeld(Object *this);
extern GoalType isAttached(Object *this);
extern GoalType isCarried(Object *this);
extern GoalType isContained(Object *this);
extern GoalType isCovered(Object *this);
extern GoalType isEmptyObject(Object *this);
extern GoalType isFullObject(Object *this);
extern GoalType isClosed(Object *this);
extern GoalType isOpen(Object *this);
extern GoalType hasAttachment(Object *this);
extern GoalType isContainer(Object *o);
extern GoalType isTable(Object *o);
extern GoalType isTool(Object *o);
extern GoalType isSharp(Object *o);
extern GoalType isSphere(Object *o);
extern GoalType isEmptyContainer(Object *o);

/******************************************************************************/

extern Boolean isEmptyEffectorBoolean(Effector *this);
extern Boolean isFullEffectorBoolean(Effector *this);
extern Boolean isEmptyObjectBoolean(Object *this);
extern Boolean isFullObjectBoolean(Object *this);
extern Boolean isOpenBoolean(Object *this);
extern Boolean isClosedBoolean(Object *this);
extern Boolean isAttachedBoolean(Object *this);
extern Boolean isContainedBoolean(Object *this);
extern Boolean isHeldBoolean(Object *this);
extern Boolean isCarriedBoolean(Object *this);
extern Boolean isCoveredBoolean(Object *this);
extern Boolean isContainerBoolean(Object *this);
extern Boolean isFlatBoolean(Object *this);
extern Boolean isToolBoolean(Object *this);
extern Boolean isFluidBoolean(Object *this);
extern Boolean isObject(Object *this, char *objectName);
extern Boolean isOnTable(Object *this);

/******************************************************************************/

extern double isEmptyEffectorFuzzy(Effector *this);
extern double isFullEffectorFuzzy(Effector *this);

/******************************************************************************/

extern char *string_copy(char *buffer);

/******************************************************************************/

extern GoalType goalCurrentSourceFixated(ContentionScheduler *cs);
extern GoalType goalCurrentTargetFixated(ContentionScheduler *cs);
extern GoalType goalCurrentThemeFixated(ContentionScheduler *cs);
extern GoalType goalCurrentImplementFixated(ContentionScheduler *cs);

extern GoalType goalCurrentSourceHeld(ContentionScheduler *cs);
extern GoalType goalCurrentTargetHeld(ContentionScheduler *cs);
extern GoalType goalCurrentThemeHeld(ContentionScheduler *cs);
extern GoalType goalCurrentImplementHeld(ContentionScheduler *cs);

extern GoalType goalCurrentSourceOnTable(ContentionScheduler *cs);
extern GoalType goalCurrentTargetOnTable(ContentionScheduler *cs);
extern GoalType goalCurrentThemeOnTable(ContentionScheduler *cs);
extern GoalType goalCurrentImplementOnTable(ContentionScheduler *cs);

/******************************************************************************/

/* Error functions: */

extern void errorObjectNull(ContentionScheduler *cs, char *role, char *action);
extern void errorObjectIncorrect(ContentionScheduler *cs, char *template, char *arg1, char *arg2, char *arg3);
extern void errorObjectState(ContentionScheduler *cs, char *template, char *arg1, char *arg2, char *arg3);

extern void errorEffectorNull(ContentionScheduler *cs, char *action);
extern void errorEffectorIncorrect(ContentionScheduler *cs, char *template, char *e1n, char *e2n);
extern void errorEffectorState(ContentionScheduler *cs, char *template, char *effector);

/* Action logging: */

extern void actionMessage(ContentionScheduler *cs, char *action, char *message);
extern void record_intervention(ContentionScheduler *cs, char *action);
extern void record_action(ContentionScheduler *cs, char *action, char *arg1, char *arg2, char *arg3);

/******************************************************************************/

extern void traceEffectorPreAction(ContentionScheduler *cs, Effector *this);
extern void traceEffectorPostAction(ContentionScheduler *cs, Effector *this);

extern void traceObjectPreAction(ContentionScheduler *cs, Object *this);
extern void traceObjectPostAction(ContentionScheduler *cs, Object *this);
extern void traceObjectPostCreate(ContentionScheduler *cs, Object *this);
extern void traceObjectPostNameChange(ContentionScheduler *cs, Object *this);

/******************************************************************************/

#include <glib.h>

#endif
