/*******************************************************************************

    File: cs_client.c
    Contents: A CS client that interfaces with ExPress
    This file (c) R.Cooper P.Yule 2001

*******************************************************************************/

#include "cs.h"
#include "analyse.h"
#include "file_misc.h"

#include <stdio.h>
#include <libexpress.h>

/******************************************************************************/
/* Global variables: **********************************************************/

#include "globals.h"

/******************************************************************************/
/* Dummy function: ************************************************************/

void attach_data(ContentionScheduler *cs) { }

#ifdef SPOOFING
int spoof_server_next_hit(int uid)
{
    return(0);
}

int spoof_server_next_participant()
{
    static int j = 0;

    if (++j > 1) {
        j = 0;
    }
    return(j);
}
#endif

/******************************************************************************/
/* Main routine: **************************************************************/

#if DEBUG

extern void do_test_actions(ContentionScheduler *cs);

int main(int argc, char **argv)
{
    ContentionScheduler *cs = create_contention_scheduler();
    LunchAnalyser la;
    char tmpfile[32] = "tmp.test";

    la.actionSequence = NULL;
    la.log = NULL;

    initialise_contention_scheduler(cs);
    reinitialise_contention_scheduler(cs, tmpfile);
    do_test_actions(cs);
    analyse_results(tmpfile, &la);
    remove(tmpfile);
    destroy_contention_scheduler(cs);
    exit(0);
}

#else

void action_log_add_action(ContentionScheduler *cs, char *act, char *message) { }
void action_log_add_warning(ContentionScheduler *cs, char *type, char *warning) { }
void action_log_add_selection(ContentionScheduler *cs, char *schema_name, Boolean on) { }
void record_schema_activity(ContentionScheduler *cs, char *name, Boolean selected,  double before, double s, double l, double p, double e, double in, double after, char *note) { }
void record_object_activity(ContentionScheduler *cs, char *name, int r, Boolean selected,  double before, double s, double l, double p, double e, double in, double after, char *note) { }
void record_effector_activity(ContentionScheduler *cs, char *name, Boolean selected,  double before, double s, double l, double p, double e, double in, double after, char *note) { }
void post_cycle_trace(ContentionScheduler *cs) { }
void process_gtk_events() {}

/*----------------------------------------------------------------------------*/

static void write_index_headings(char *tmpdir)
{
    char indexfile[32];
    FILE *fp;

    g_snprintf(indexfile, 32, "%s/index.txt", tmpdir);

    if ((fp = fopen(indexfile, "w")) != NULL) {

        fprintf(fp, "        File");

        fprintf(fp, "\tSelfS");
        fprintf(fp, "\tLatS");
        fprintf(fp, "\tParS");
        fprintf(fp, "\tEnvS");
        fprintf(fp, "\tSelfO");
        fprintf(fp, "\tLatO");
        fprintf(fp, "\tParO");
        fprintf(fp, "\tEnvO");
        fprintf(fp, "\tSelfE");
        fprintf(fp, "\tLatE");
        fprintf(fp, "\tParE");
        fprintf(fp, "\tEnvE");
        fprintf(fp, "\tPers");
        fprintf(fp, "\tSelTh");

        fprintf(fp, "\tNoiseS");
        fprintf(fp, "\tNoiseO");
        fprintf(fp, "\tNoiseE");
        fprintf(fp, "\tRest");
        fprintf(fp, "\tDist?");

        fprintf(fp, "\tActions");
        fprintf(fp, "\tErrors");
        fprintf(fp, "\tInterVs");
        fprintf(fp, "\tAccomps");
        fprintf(fp, "\tOmis");
        fprintf(fp, "\tSubOms");
        fprintf(fp, "\tOrphOms");
        fprintf(fp, "\tAnts");
        fprintf(fp, "\tAntOms");
        fprintf(fp, "\tPersevs");
        fprintf(fp, "\tRevs");
        fprintf(fp, "\tArgSubs");
        fprintf(fp, "\tAdds");
        fprintf(fp, "\tCycles");
        fprintf(fp, "\tComProp");
        fprintf(fp, "\tComplete");

        fprintf(fp, "\n");

        fclose(fp);
    }
}

static void write_index_line(char *tmpdir, int uid, DataAnalysis *la)
{
    char indexfile[32];
    FILE *fp;

    g_snprintf(indexfile, 32, "%s/index.txt", tmpdir);

    if ((fp = fopen(indexfile, "a")) != NULL) {

        fprintf(fp, "tmp.%d:", uid);

        fprintf(fp, "\t%6.4f", self[SCHEMA]);
        fprintf(fp, "\t%6.4f", lateral[SCHEMA]);
        fprintf(fp, "\t%6.4f", parental[SCHEMA]);
        fprintf(fp, "\t%6.4f", environmental[SCHEMA]);
        fprintf(fp, "\t%6.4f", self[OBJECT]);
        fprintf(fp, "\t%6.4f", lateral[OBJECT]);
        fprintf(fp, "\t%6.4f", parental[OBJECT]);
        fprintf(fp, "\t%6.4f", environmental[OBJECT]);
        fprintf(fp, "\t%6.4f", self[EFFECTOR]);
        fprintf(fp, "\t%6.4f", lateral[EFFECTOR]);
        fprintf(fp, "\t%6.4f", parental[EFFECTOR]);
        fprintf(fp, "\t%6.4f", environmental[EFFECTOR]);
        fprintf(fp, "\t%6.4f", persistence);
        fprintf(fp, "\t%6.4f", selectionThreshold);
        fprintf(fp, "\t%6.4f", noiseSD[SCHEMA]);
        fprintf(fp, "\t%6.4f", noiseSD[OBJECT]);
        fprintf(fp, "\t%6.4f", noiseSD[EFFECTOR]);
        fprintf(fp, "\t%6.4f", restAct);

        fprintf(fp, "\t%d", la->numActions);
        fprintf(fp, "\t%d", la->numErrors);
        fprintf(fp, "\t%d", la->numInterventions);
        fprintf(fp, "\t%d", la->numAccomplished);
        fprintf(fp, "\t%d", la->numActionAdditions);
        fprintf(fp, "\t%d", la->numActionAnticipations);
        fprintf(fp, "\t%d", la->numStepOmissions);
        fprintf(fp, "\t%d", la->numPerseverations);

        fprintf(fp, "\t%d", la->numToolOmissions);
        fprintf(fp, "\t%d", la->numMisUse1);
        fprintf(fp, "\t%d", la->numMisUse2);
        fprintf(fp, "\t%d", la->numMisLocation1);
        fprintf(fp, "\t%d", la->numMisLocation2);
        fprintf(fp, "\t%d", la->numObjectSubstitution);
        fprintf(fp, "\t%d", la->numToying);
        fprintf(fp, "\t%d", la->numPantomime);
        fprintf(fp, "\t%d", la->numPerplexity);
        fprintf(fp, "\t%d", la->numCycles);
        fprintf(fp, "\t%6.4f", la->propCommission);
        fprintf(fp, "\t%d", la->complete);

        fprintf(fp, "\n");

        fclose(fp);
    }
}

/*----------------------------------------------------------------------------*/

static void initialise_error_counts(DataAnalysis *results)
{
    results->numActions = 0;
    results->numErrors = 0;
    results->numInterventions = 0;
    results->numAccomplished = 0;
    results->numActionAdditions = 0;
    results->numActionAnticipations = 0;
    results->numStepOmissions = 0;
    results->numPerseverations = 0;
    results->numToolOmissions = 0;
    results->numMisUse1 = 0;
    results->numMisUse2 = 0;
    results->numMisLocation1 = 0;
    results->numMisLocation2 = 0;
    results->numObjectSubstitution = 0;
    results->numToying = 0;
    results->numPantomime = 0;
    results->numPerplexity = 0;
    results->numCycles = 0;
}

static void accumulate_results(DataAnalysis *results, DataAnalysis *la)
{
    results->numActions += la->numActions;
    results->numErrors += la->numErrors;
    results->numInterventions += la->numInterventions;
    results->numAccomplished += la->numAccomplished;
    results->numActionAdditions += la->numActionAdditions;
    results->numActionAnticipations += la->numActionAnticipations;
    results->numStepOmissions += la->numStepOmissions;
    results->numPerseverations += la->numPerseverations;
    results->numToolOmissions += la->numToolOmissions;
    results->numMisUse1 += la->numMisUse1;
    results->numMisUse2 += la->numMisUse2;
    results->numMisLocation1 += la->numMisLocation1;
    results->numMisLocation2 += la->numMisLocation2;
    results->numObjectSubstitution += la->numObjectSubstitution;
    results->numToying += la->numToying;
    results->numPantomime += la->numPantomime;
    results->numPerplexity += la->numPerplexity;
    results->numCycles += la->numCycles;
}

static void run_and_analyse_task(ContentionScheduler *cs, char *tmpdir, int uid, int n, TaskTypeId task, DataAnalysis *results)
{
    char tmpfile[32];
    DataAnalysis la;
    FILE *fp;

    la.actionSequence = NULL;
    la.log = NULL;

    g_snprintf(tmpfile, 32, "%s/tmp.%d.%d", tmpdir, uid, n);

    initialise_task(task, &(cs->task));
    reinitialise_contention_scheduler(cs, tmpfile);
    if ((fp = fopen(cs->logfile, "w")) != NULL) { fclose(fp); }
    run(cs, FALSE);
    if (analyse_results(tmpfile, &(cs->task), &la)) {
        write_index_line(tmpdir, uid, &la);
    }
    accumulate_results(results, &la);
}

int main(int argc, char **argv)
{
    extern int getpid();

    ContentionScheduler *cs = create_contention_scheduler();
    DataAnalysis results;
    char tmpdir[16];
    int uid, hit;

    server_set(SP_HOST, "dropbear.psyc.bbk.ac.uk");
    server_set(SP_PAGE, "/expts/cs_mot_nso/express.cgi");
    server_set(SP_VERBOSE, stderr);

/* Specify local design and data files to over-ride the network versions: ------
    server_set(SP_LOCAL_DESIGN, "/home/rick/COGENT/projects/ContentionScheduling/designs/1005416894");
    server_set(SP_LOCAL_DATA, "/tmp/cs_data/");
------------------------------------------------------------------------------*/

    server_init(argc, argv);

    server_declare_variable("trial", VAR_WIV, VAR_INT, &hit);

    server_declare_variable("schema_self", VAR_BIV, VAR_DOUBLE, &self[SCHEMA]);
    server_declare_variable("schema_lateral", VAR_BIV, VAR_DOUBLE, &lateral[SCHEMA]);
    server_declare_variable("schema_from_schema", VAR_BIV, VAR_DOUBLE, &parental[SCHEMA]);
    server_declare_variable("schema_from_object", VAR_BIV, VAR_DOUBLE, &environmental[SCHEMA]);
    server_declare_variable("object_self", VAR_BIV, VAR_DOUBLE, &self[OBJECT]);
    server_declare_variable("object_lateral", VAR_BIV, VAR_DOUBLE, &lateral[OBJECT]);
    server_declare_variable("object_from_schema", VAR_BIV, VAR_DOUBLE, &parental[OBJECT]);
    server_declare_variable("object_from_object", VAR_BIV, VAR_DOUBLE, &environmental[OBJECT]);
    server_declare_variable("effector_self", VAR_BIV, VAR_DOUBLE, &self[EFFECTOR]);
    server_declare_variable("effector_lateral", VAR_BIV, VAR_DOUBLE, &lateral[EFFECTOR]);
    server_declare_variable("effector_from_schema", VAR_BIV, VAR_DOUBLE, &parental[EFFECTOR]);
    server_declare_variable("effector_from_effector", VAR_BIV, VAR_DOUBLE, &environmental[EFFECTOR]);
    server_declare_variable("persistence", VAR_BIV, VAR_DOUBLE, &persistence);
    server_declare_variable("selection_threshold", VAR_BIV, VAR_DOUBLE, &selectionThreshold);
    server_declare_variable("noise_schema", VAR_BIV, VAR_DOUBLE, &noiseSD[SCHEMA]);
    server_declare_variable("noise_object", VAR_BIV, VAR_DOUBLE, &noiseSD[OBJECT]);
    server_declare_variable("noise_effector", VAR_BIV, VAR_DOUBLE, &noiseSD[EFFECTOR]);
    server_declare_variable("rest", VAR_BIV, VAR_DOUBLE, &restAct);
    server_declare_variable("intervene", VAR_BIV, VAR_BOOLEAN, &interventions);

    server_declare_variable("num_actions", VAR_DV, VAR_INT, &results.numActions);
    server_declare_variable("num_cycles", VAR_DV, VAR_INT, &results.numCycles);
    server_declare_variable("num_errors", VAR_DV, VAR_INT, &results.numErrors);
    server_declare_variable("num_interventions", VAR_DV, VAR_INT, &results.numInterventions);
    server_declare_variable("num_step_omissions", VAR_DV, VAR_INT, &results.numStepOmissions);
    server_declare_variable("num_action_anticipations", VAR_DV, VAR_INT, &results.numActionAnticipations);
    server_declare_variable("num_action_additions", VAR_DV, VAR_INT, &results.numActionAdditions);
    server_declare_variable("num_perseverations", VAR_DV, VAR_INT, &results.numPerseverations);

    server_declare_variable("num_tool_omission", VAR_DV, VAR_INT, &results.numToolOmissions);
    server_declare_variable("num_misuse_1", VAR_DV, VAR_INT, &results.numMisUse1);
    server_declare_variable("num_misuse_2", VAR_DV, VAR_INT, &results.numMisUse2);
    server_declare_variable("num_mislocation_1", VAR_DV, VAR_INT, &results.numMisLocation1);
    server_declare_variable("num_mislocation_2", VAR_DV, VAR_INT, &results.numMisLocation2);
    server_declare_variable("num_toying", VAR_DV, VAR_INT, &results.numToying);
    server_declare_variable("num_pantomime", VAR_DV, VAR_INT, &results.numPantomime);
    server_declare_variable("num_perplexity", VAR_DV, VAR_INT, &results.numPerplexity);

    results.actionSequence = NULL;
    results.log = NULL;

    initialise_task(JUICE_BASIC, &(cs->task));
    initialise_contention_scheduler(cs);

    messages[RUN] = NULL;   // stdout;
    messages[WORLD] = NULL; // stdout;

    g_snprintf(tmpdir, 16, "%d", getpid());
    make_directory(tmpdir);
    write_index_headings(tmpdir);

    while ((uid = server_next_participant()) > 0) {
        initialise_error_counts(&results);
        do {
            run_and_analyse_task(cs, tmpdir, uid, 0, LETTER_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 1, JUICE_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 2, ESPRESSO_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 3, CANDLE_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 4, BOTTLE_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 5, LETTER_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 6, JUICE_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 7, ESPRESSO_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 8, CANDLE_BASIC, &results);
            run_and_analyse_task(cs, tmpdir, uid, 9, BOTTLE_BASIC, &results);
        } while (server_next_hit(uid) > 0);
    }
    destroy_contention_scheduler(cs);
    server_free();
#ifdef MALLOC_CHECK
    zmalloc_check();
#endif
    exit(0);
}
#endif
/******************************************************************************/
/******************************************************************************/
