#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "lib_string.h"

#define INPUT_FILE "dvs_so.dat"

#define MAX_LINE_LENGTH 1024

char buffer[MAX_LINE_LENGTH];

typedef enum boolean {FALSE, TRUE} Boolean; 

#include <glib.h>


#define VARS 22
#define ROWS 15
#define COLS 20
 
static int row_max = 0;
static int col_max = 0;

double dr_data[11] = {0.00, 0.00,  4.75, 0.00, 5.00, 1.75, 0.00, 0.75, 0.00, 0.25, 0.00};
double fg_data[11] = {0.00, 0.00, 10.50, 0.75, 1.25, 2.50, 5.75, 1.50, 0.75, 5.75, 2.50};

// Random data
//double dr_data[11] = {0.00, 0.00,  1.75, 1.00, 1.00, 1.75, 1.00, 1.75, 1.00, 1.25, 1.00};
//double fg_data[11] = {0.00, 0.00,  4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00};

char *table_headings[VARS];
double table[VARS][ROWS][COLS];
int current_table;
int current_row;

// A simple program to read the matrices of data generated by the express
// client and find the best fit to the actual data.

/*----------------------------------------------------------------------------*/

static void global_vars_initialise()
{
    int i, j, k;

    current_table = -1;
    for (i = 0; i < VARS; i++) {
        table_headings[i] = NULL;
        for (j = 0; j < ROWS; j++) {
            for (k = 0; k < COLS; k++) {
                table[i][j][k] = 0.0;
            }
        }
    }
}

static void global_vars_destroy()
{
    while (current_table > 0) {
        free(table_headings[current_table]);
        table_headings[current_table] = NULL;
        current_table--;
    }
}

/*----------------------------------------------------------------------------*/

static Boolean express_data_line_is_table_heading(char *buffer, char **variable)
{
    int i = strlen("Tabulating ");

    if (strncmp(buffer, "Tabulating ", i) == 0) {
        int j = 0;
        while ((i + j < MAX_LINE_LENGTH) && (buffer[i+j] != ' ')) {
            j++;
        }
        *variable = string_new(j+1);
        strncpy(*variable, &buffer[i], j);
	return(TRUE);
    }
    else {
        return(FALSE);
    }
}

static Boolean express_data_line_is_table_column_headings(char *buffer)
{
    return((buffer[0] == '\t') && (buffer[1] == '\t'));
}

static int express_data_line_is_table_row_data(char *buffer, double *iv)
{
    // Each row of data starts with either a IV's value (in single quotes)
    // or the word "overall" (without quotes). For the time being we are
    // only concerned with the former type of row.

    if (buffer[0] == '\'') {
        int j = 0;
        // Line begins with a single quote ... find the matching close quote:
        while ((j+1 < MAX_LINE_LENGTH) && (buffer[j+1] != '\'')) {
            j++;
        }
        if (buffer[j+1] == '\'') {
            *iv = atof(&buffer[1]);
            return(j+2);
        }
    }
    return(-1);
}

/*----------------------------------------------------------------------------*/

static void express_data_line_install_column_heading(char *buffer, int cv, int cc)
{
    if (cc < COLS) {
        table[cv][0][cc] = atof(buffer);
    }
    else {
        fprintf(stdout, "Error: COLS is not big enough\n");
        abort();
    }
}

static void express_data_line_install_column_headings(char *buffer)
{
    // We have a set of column headings to be installed in cells
    // [current_table][0][i] for i = 1 to COLS-1

    int i = 1, j = 3;

    do {
        express_data_line_install_column_heading(&buffer[j], current_table, i++);
        while ((buffer[j] != '\0') && (isdigit(buffer[j]) || (buffer[j] == '.'))) {
            j++;
        }
        if (buffer[j] == '\'') { j++; }
        while (buffer[j] == '\t') { j++; }
        if (buffer[j] == '\'') { j++; }
        if (!isdigit(buffer[j])) { return; }
    } while (TRUE);
}

/*----------------------------------------------------------------------------*/

static Boolean express_data_line_install_cell(char *buffer, int cv, int cr, int cc)
{
    if (isdigit(buffer[0])) {
        table[cv][cr][cc] = atof(buffer);
        return(TRUE);
    }
    else {
        return(FALSE);
    }
}

static void express_data_line_install_row_data(char *buffer, double iv)
{
    // We have a row of data in cells
    // [current_table][current_row][i] for i = 1 to COLS-1
    // Note that the first element should be within

    int col = 1, j = 0;

    table[current_table][current_row][0] = iv;

    while ((buffer[j] == '\t') || (buffer[j] == ' ')) { j++; }
    while ((col < COLS) && express_data_line_install_cell(&buffer[j], current_table, current_row, col)) {
        while (isdigit(buffer[j])) { j++; }
        if (buffer[j] == '.') { j++; }
        while (isdigit(buffer[j])) { j++; }
        while ((buffer[j] == '\t') || (buffer[j] == ' ')) { j++; }
        col++;
    }
    // The last column is an overall mean -- ignore it:
    col_max = col-1;
}

/*----------------------------------------------------------------------------*/

static void express_data_line_add_to_table(char *buffer)
{
    char *variable;
    double iv;
    int j;

    if (express_data_line_is_table_heading(buffer, &variable)) {
        if (++current_table < VARS) {
            table_headings[current_table] = variable;
            current_row = 0;
        }
        else {
            fprintf(stdout, "Error: VARS is not big enough\n");
            abort();
        }
    }
    else if (express_data_line_is_table_column_headings(buffer)) {
        // We have a set of column headings .. install in cells
        // [current_table][0][i] for i = 1 to COLS-1
        // Note: current_row should be zero when we get here
        express_data_line_install_column_headings(buffer);
        current_row = 0;
    }
    else if ((j = express_data_line_is_table_row_data(buffer, &iv)) > 0) {
        if (current_row++ >= ROWS) {
            fprintf(stdout, "Error: ROWS is not big enough\n");
            abort();
        }
        else {
            express_data_line_install_row_data(&buffer[j], iv);
        }
    }
    else if (strncmp(buffer, "overall", 7) == 0) {
        row_max = current_row + 1;
    }
}

static void express_data_file_read(char *filename)
{
    FILE *fp = fopen(filename, "r");
    if (fp == NULL) {
        fprintf(stderr, "Cannot read input from %s\n", filename);
        abort();
    }

    while (fgets(buffer, MAX_LINE_LENGTH, fp) != NULL) {
        if (strlen(buffer) == (MAX_LINE_LENGTH-1)) {
            fprintf(stderr, "Error: Line is longer than MAX_LINE_LENGTH\n");
            abort();
        }
        else {
            express_data_line_add_to_table(buffer);
        }
    }
    fclose(fp);
}

/******************************************************************************/

static int create_new_table(char *label)
{
    if (current_table < VARS) {
        int i, j;

        current_table++;
        table_headings[current_table] = string_copy(label);
        for (i = 0; i < ROWS; i++) {
            for (j = 0; j < COLS; j++) {
                if ((i == 0) || (j == 0)) {
                    table[current_table][i][j] = table[0][i][j];
                }
                else {
                    table[current_table][i][j] = 0.0;
                }
            }
        }
        return(current_table);
    }
    else {
        return(-1);
    }
}
    
/******************************************************************************/

static void discard_table(int table)
{
    /* Discards the table data if it is the most recent one */
    if (current_table == table) {
        free(table_headings[current_table]);
        current_table--;
    }
}

/******************************************************************************/

static double ssd(double d1, double d2)
{
    return((d1 - d2) * (d1 - d2));
}

static double wssd(double d1, double d2)
{
    if (d1 != 0.0) {
        return(((d1 - d2) * (d1 - d2)) / d1);
    }
    else {
        return(((d1 - d2) * (d1 - d2)) / 0.01);
    }
}

static void populate_fit_table(int i, double *data, double func())
{
    int j, k, var;

    for (j = 1; j < ROWS; j++) {
        for (k = 1; k < COLS; k++) {
            for (var = 2; var < 11; var++) {
                table[i][j][k] += func(data[var], table[var][j][k]);
            }
            table[i][j][k] = sqrt(table[i][j][k] / 9.0); 
        }
    }
}

static double correlate(double *x, double *y, int n)
{
    double sx = 0.0, sy = 0.0, sxx = 0.0, syy = 0.0, sxy = 0.0;
    double numerator, denominator;
    int i;

    for (i = 0; i < n; i++) {
        sx  += x[i];
        sy  += y[i];
        sxx += x[i] * x[i];
        syy += y[i] * y[i];
        sxy += x[i] * y[i];
    }
    numerator = (n * sxy - sx * sy);
    denominator = sqrt((n * sxx - sx * sx) * (n * syy - sy * sy));
    if (denominator > 0) {
        return(numerator / denominator);
    }
    else {
        return(0.0);
    }
}

static void populate_correlation_table(int i, double *data)
{
    int j, k;
    double model[9];

    for (j = 1; j < ROWS; j++) {
        for (k = 1; k < COLS; k++) {
            model[0]  = table[2][j][k];
            model[1]  = table[3][j][k];
            model[2]  = table[4][j][k];
            model[3]  = table[5][j][k];
            model[4]  = table[6][j][k];
            model[5]  = table[7][j][k];
            model[6]  = table[8][j][k];
            model[7]  = table[9][j][k];
            model[8] = table[10][j][k];

            table[i][j][k] = correlate(model, &data[2], 9);
        }
    }
}

static int calculate_correlation(char *label, double *data)
{
    // Create a new table reflecting the sum square differences for the
    // patient data 
    int j;

    if ((j = create_new_table(label)) > 0) {
        populate_correlation_table(j, data);
    }
    return(j);
}

static int calculate_fit(char *label, double *data, double func())
{
    // Create a new table reflecting the sum square differences for the
    // patient data 
    int j;

    if ((j = create_new_table(label)) > 0) {
        populate_fit_table(j, data, func);
    }
    return(j);
}

/******************************************************************************/

static void ff_print_table(FILE *fp, int i)
{
    int j, k;

    if (table_headings[i] != NULL) {
        fprintf(fp, "\n%s:\n", table_headings[i]);
        for (j = 0; j < row_max; j++) {
            for (k = 0; k < col_max; k++) {
                if ((j == 0) && (k == 0)) {
                    fprintf(fp, "\t  -----");
                }
                else {
                    fprintf(fp, "\t%7.3f", (table[i][j][k] > 999.999) ? 999.999 : table[i][j][k]);
                }
            }
            fprintf(fp, "\n");
        }
    }
}

/******************************************************************************/
/* Utility function for printing the tables (mainly for debugging): ***********/

void ff_print_tables(FILE *fp)
{
    int i;

    for (i = 0; i < VARS; i++) {
        if (table_headings[i] != NULL) {
            ff_print_table(fp, i);
	}
    }
}

/******************************************************************************/

static void scan_table_for_correlation(FILE *fp, char *label, int i, double *data)
{
    double best;
    int j, k, x, y;

    x = 1; y = 1;
    best = table[i][x][y];

    for (j = 1; j < row_max; j++) {
        for (k = 1; k < col_max; k++) {
            if (table[i][j][k] > best) {
                best = table[i][j][k];
                x = j; y = k;
            }
        }
    }
    fprintf(fp, "Best %s fit for %s is %5.3f at row = %5.3f, column = %5.3f\n", label, table_headings[i], best, table[i][x][0], table[i][0][y]);
    fprintf(fp, "Patient: %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f\n", data[2], data[3], data[4], data[5], data[6], data[7], data[8], data[9], data[10]);
    fprintf(fp, "Model:   %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f\n", table[2][x][y], table[3][x][y], table[4][x][y], table[5][x][y], table[6][x][y], table[7][x][y], table[8][x][y], table[9][x][y], table[10][x][y]);
    fprintf(fp, "\n");
}

static void scan_table_for_fit(FILE *fp, char *label, int i, double *data)
{
    double best;
    int j, k, x, y;

    x = 1; y = 1;
    best = table[i][x][y];

    for (j = 1; j < row_max; j++) {
        for (k = 1; k < col_max; k++) {
            if (table[i][j][k] < best) {
                best = table[i][j][k];
                x = j; y = k;
            }
        }
    }
    fprintf(fp, "Best %s fit for %s is %5.3f at row = %5.3f, column = %5.3f\n", label, table_headings[i], best, table[i][x][0], table[i][0][y]);
    fprintf(fp, "Patient: %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f\n", data[2], data[3], data[4], data[5], data[6], data[7], data[8], data[9], data[10]);
    fprintf(fp, "Model:   %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f\n", table[2][x][y], table[3][x][y], table[4][x][y], table[5][x][y], table[6][x][y], table[7][x][y], table[8][x][y], table[9][x][y], table[10][x][y]);
    fprintf(fp, "\n");
}

/******************************************************************************/

static void find_correlation(char *label, double *data)
{
    int j;

    j = calculate_correlation(label, data);
    ff_print_table(stdout, j);
    scan_table_for_correlation(stdout, "correlation", j, data);
    discard_table(j);
}


static void find_fit_with_func(char *label, double *data, char *type, double func())
{
    int j;

    j = calculate_fit(label, data, func);
//    ff_print_table(stdout, j);
    scan_table_for_fit(stdout, type, j, data);
    discard_table(j);
}

static void find_fit(char *label, double *data)
{
//    find_correlation(label, data);
    find_fit_with_func(label, data, "ssd", ssd);
    find_fit_with_func(label, data, "wssd", wssd);
}

/******************************************************************************/

int main(int argc, char **argv)
{
    if (argc != 2) {
        fprintf(stderr, "Usage: %s input_file\n", argv[0]);
        exit(-1);
    }
    else {
        global_vars_initialise();
        express_data_file_read(argv[1]);
        find_fit("Patient DR", dr_data);
        find_fit("Patient FG", fg_data);
        global_vars_destroy();
        exit(0);
    }
}

/******************************************************************************/
