/*******************************************************************************

    File:       lib_string.c
    Contents:   Miscellaneous string manipulation functions.
    Author:     Rick Cooper
    Copyright (c) 2004 Richard P. Cooper

    This file is part of COGENT.

    COGENT is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COGENT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COGENT; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Public procedures:


*******************************************************************************/

/******** Include files: ******************************************************/

#include "lib_string.h"
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

/******************************************************************************/
/* String manipulation functions: *********************************************/

char *string_new(int l)
{
    if (l > 0) {
        return((char *)malloc(l * sizeof(char)));
    }
    else {
        return(NULL);
    }
}

/*----------------------------------------------------------------------------*/

char *string_copy(const char *s)
{
    char *copy;

    if (s == NULL) {
        return(NULL);
    }
    else if ((copy = string_new(strlen(s)+1)) == NULL) {
        return(NULL);
    }
    else {
        strcpy(copy, s);
        return(copy);
    }
}

/*----------------------------------------------------------------------------*/

char *string_copy_substring(const char *s, int l)
{
    char *copy;

    if (s == NULL) {
        return(NULL);
    }
    else if ((copy = string_new(l+1)) == NULL) {
        return(NULL);
    }
    else {
        strncpy(copy, s, l);
        copy[l] = '\0';
        return(copy);
    }
}

/*----------------------------------------------------------------------------*/

void string_trim_trailing_blanks(char *string)
{
    int l = 0;
    while (string[l] != '\0') {
        l++;
    }
    while ((--l >= 0) && (string[l] == ' ')) {
        string[l] = '\0';
    }
}

/*----------------------------------------------------------------------------*/

int string_split(char *fullstring)
{
    int l = strlen(fullstring);
    int i = l / 2, j = l / 2;

    /* Find the first space character in fullstring before fullstring[l/2]:   */

    while ((i > 0) && (fullstring[i] != ' ')) {
        i--;
    }

    /* Find the first space character in fullstring after fullstring[l/2]:    */

    while ((j < l) && (fullstring[j] != ' ')) {
        j++;
    }

    /* Choose i or j, whichever is closest to l/2                             */

    if ((i > 0) || (j < l)) {
        return(((l/2 - i) > (j - l/2)) ? j : i);
    }
    else {
        return(0);
    }
}

/*----------------------------------------------------------------------------*/

long string_to_long(const char *string)
{
    long result = 0;
    int i = 0, n = strlen(string);

    if (n > 0) {
        while ((i < n) && (isdigit((int) string[i]))) {
            result = result * 10 + (string[i++] - '0');
        }
        if (i < n) {
            result = -1;
        }
    }
    else {
        result = -1;
    }
    return(result);
}

/******************************************************************************/
