#include "../cs.h"
#include "task.h"

GoalType goalEspressoBaseFilled(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isEspressoBase(os->head), isFullObject(os->head)));
    }
    return(found);
}

GoalType goalEspressoFilterPositioned(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
        if (os->head->carrier != NULL) {
            found = FZOR(found, FZAND(isEspressoFilter(os->head), isEspressoBase(os->head->carrier)));
        }
    }
    return(found);
}

GoalType goalEspressoFilterAssembled(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(FZAND(isEspressoFilter(os->head), isCarried(os->head)), isFullObject(os->head)));
    }
    return(found);
}

GoalType goalEspressoFilterFull(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isEspressoFilter(os->head), isFullObject(os->head)));
    }
    return(found);
}

GoalType goalEspressoPotAssembled(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isAssembledEspressoPot(os->head));
    }
    return(found);
}

GoalType goalAssembledEspressoPotOnTable(ContentionScheduler *cs)
{
    return(goalOnTable(cs, isAssembledEspressoPot));
}

GoalType goalHoldFullJug(ContentionScheduler *cs)
{
    return(goalHold(cs, isFullJug));
}

GoalType goalHoldEspressoFilter(ContentionScheduler *cs)
{
    return(goalHold(cs, isEspressoFilter));
}

GoalType goalHoldEspressoTop(ContentionScheduler *cs)
{
    return(goalHold(cs, isEspressoTop));
}

GoalType goalHoldEspressoBase(ContentionScheduler *cs)
{
    return(goalHold(cs, isEspressoBase));
}

GoalType goalHoldEspressoBaseAndTop(ContentionScheduler *cs)
{
    return(FZAND(goalHold(cs, isEspressoBase), goalHold(cs, isEspressoTop)));
}

GoalType goalHoldEspressoImplement(ContentionScheduler *cs)
{
    return(goalHold(cs, isTool));
}

GoalType goalHoldFullImplement(ContentionScheduler *cs)
{
    return(goalHold(cs, isFullImplement));
}

GoalType goalHoldEmptyImplement(ContentionScheduler *cs)
{
    return(goalHold(cs, isEmptyImplement));
}

GoalType goalImplementNotHeld(ContentionScheduler *cs)
{
    return(FZNOT(goalHold(cs, isTool)));
}

GoalType goalJugOnTableAndEspressoBaseFull(ContentionScheduler *cs)
{
    return(FZAND(goalOnTable(cs, isJug), goalEspressoBaseFilled(cs)));
}

GoalType goalEspressoTopFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isEspressoTop));
}

GoalType goalEspressoBaseFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isEspressoBase));
}

GoalType goalHoldFullJugAndTargetFixated(ContentionScheduler *cs)
{
    return(FZAND(goalHoldFullJug(cs), goalTargetFixated(cs)));
}

GoalType goalHoldEspressoFilterAndTargetFixated(ContentionScheduler *cs)
{
    return(FZAND(goalHoldEspressoFilter(cs), goalTargetFixated(cs)));
}

GoalType goalHoldEspressoImplementAndSourceFixated(ContentionScheduler *cs)
{
    return(FZAND(goalHoldEspressoImplement(cs), goalSourceFixated(cs)));
}

GoalType goalHoldFullImplementAndTargetFixated(ContentionScheduler *cs)
{
    return(FZAND(goalHoldFullImplement(cs), goalTargetFixated(cs)));
}

GoalType goalEmptyScoopHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isEmptyImplement));
}

/******************************************************************************/

static GoalType goalCheckFixatedObjectForFunction(ContentionScheduler *cs, ObjectRole role)
{
    extern Boolean fixatedObjectIsBestForFunction(ContentionScheduler *cs, ObjectRole role);

    return(fixatedObjectIsBestForFunction(cs, role) ? GOAL_TRUE : GOAL_FALSE);
}

GoalType goalSourceFixated(ContentionScheduler *cs)
{
    return(goalCheckFixatedObjectForFunction(cs, SOURCE));
}

GoalType goalTargetFixated(ContentionScheduler *cs)
{
    return(goalCheckFixatedObjectForFunction(cs, TARGET));
}

GoalType goalThemeFixated(ContentionScheduler *cs)
{
    return(goalCheckFixatedObjectForFunction(cs, THEME));
}

GoalType goalImplementFixated(ContentionScheduler *cs)
{
    return(goalCheckFixatedObjectForFunction(cs, IMPLEMENT));
}

/******************************************************************************/
