/*******************************************************************************

    FILE: espresso/object.c
    AUTHOR: R. Cooper
    CREATION DATE: Fri Apr 16 07:43:59 2004
    CONTENTS: 

*******************************************************************************/

#include "../cs.h"
#include "task.h"

extern GoalType feature_overlap(int af, int bf);

/******************************************************************************/

GoalType isEspressoTop(Object *this)
{
    return(feature_overlap(this->features, IS_MEDIUM | IS_CONTAINER | IS_METAL | IS_CYLINDRICAL | IS_LID | HOLESIZE_2));
}

GoalType isEspressoBase(Object *this)
{
    return(feature_overlap(this->features, IS_MEDIUM | IS_CONTAINER | IS_METAL | IS_CYLINDRICAL | HAS_SCREW_THREAD | HOLESIZE_2));
}

GoalType isEspressoFilter(Object *this)
{
    return(feature_overlap(this->features, IS_SMALL | IS_CONTAINER | IS_METAL | IS_TOOL | IS_CYLINDRICAL | HOLESIZE_2));
}

GoalType isAssembledEspressoPot(Object *this)
{
    GoalType found = GOAL_FALSE;
    ObjectList *os;

    for (os = this->itemsCarried; os != NULL; os = os->tail) {
        found = FZOR(found, isEspressoTop(os->head));
    }
    return(FZAND(found, isEspressoBase(this)));
}

GoalType isJug(Object *this)
{
    return(feature_overlap(this->features, IS_CONTAINER | IS_MEDIUM | IS_GLASS | IS_CYLINDRICAL));
}

GoalType isSpoon(Object *this)
{
    return(feature_overlap(this->features, IS_SMALL | IS_TOOL | IS_EXTENDED));
}

GoalType isCoffeePacket(Object *this)
{
    return(feature_overlap(this->features, IS_MEDIUM | IS_CONTAINER | IS_PLASTIC));
}
 
GoalType isFullJug(Object *this)
{
    return(FZAND(isJug(this), isFullObject(this)));
}

GoalType isFullContainer(Object *this)
{
    return(FZAND(isContainer(this), isFullObject(this)));
}

GoalType isOpenContainer(Object *this)
{
    return(FZAND(isContainer(this), isOpen(this)));
}

GoalType isFullImplement(Object *this)
{
    return(FZAND(isTool(this), isFullObject(this)));
}

GoalType isEmptyImplement(Object *this)
{
    return(FZAND(isTool(this), isEmptyObject(this)));
}

/******************************************************************************/

void write_espresso_object_attributes(FILE *fp, Object *this)
{
    if (this->state & STATE_OPEN) { 
        fprintf(fp, "open, ");
    }
    else if (this->state & STATE_CLOSED) { 
        fprintf(fp, "closed, ");
    }
}

/******************************************************************************/

void install_espresso_basic_objects(ContentionScheduler *cs)
{
    Object *object;

    object = objectInstall(cs, "espresso_pot_base", IS_MEDIUM | IS_CONTAINER | IS_METAL | IS_CYLINDRICAL | HAS_SCREW_THREAD | HOLESIZE_2, STATE_OPEN);
     objectSetPosition(object, 1, 1);

    object = objectInstall(cs, "espresso_pot_filter", IS_SMALL | IS_CONTAINER | IS_METAL | IS_TOOL | IS_CYLINDRICAL | HOLESIZE_2, STATE_OPEN);
    objectSetPosition(object, 0, 1);

    object = objectInstall(cs, "espresso_pot_top", IS_MEDIUM | IS_CONTAINER | IS_METAL | IS_CYLINDRICAL | IS_LID | HOLESIZE_2, STATE_OPEN);
    objectSetPosition(object, -1, 1);

    object = objectInstall(cs, "water_jug", IS_CONTAINER | IS_MEDIUM | IS_GLASS | IS_CYLINDRICAL, STATE_OPEN);
    objectSetPosition(object, -2, 2);
    objectInstallContents(object, objectInstallItem(cs, "water", IS_FLUID));

    object = objectInstall(cs, "spoon", IS_SMALL | IS_TOOL | IS_EXTENDED, 0);
    objectSetPosition(object, 1, 2);

    object = objectInstall(cs, "coffee_tin", IS_MEDIUM | IS_CONTAINER | IS_PLASTIC, STATE_OPEN);
    objectSetPosition(object, 2, 2);
    objectInstallContents(object, objectInstallItem(cs, "coffee_grounds", IS_FLUID));

    object = objectInstall(cs, "table", 0, 0);
}

/******************************************************************************/
