/******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

void install_espresso_schemas(ContentionScheduler *cs)
{
    Schema *s;

    s = install_schema(cs, "schemaPrepareEspresso", "goalPrepareEspresso", ACTION_NONE, HIGH);
    install_schema_object_trigger(s, SOURCE, schemaPrepareEspressoTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPrepareEspressoTriggersTarget);
    install_schema_object_trigger(s, IMPLEMENT, schemaPrepareEspressoTriggersImplement);
    install_schema_effector_trigger(s, schemaPrepareEspressoTriggersEffector);
    install_schema_subgoal(s, "goalFillEspressoBase", goalTrue, goalJugOnTableAndEspressoBaseFull);
    install_schema_subgoal(s, "goalAssembleEspressoFilter", goalEspressoBaseFilled, goalEspressoFilterAssembled);
    install_schema_subgoal(s, "goalAssembleEspressoTop", goalEspressoFilterAssembled, goalAssembledEspressoPotOnTable);

    cs->intended_schema = s;

    /* Intermediate level schemas: */

    s = install_schema(cs, "schemaFillEspressoBase", "goalFillEspressoBase", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaFillEspressoBaseTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaFillEspressoBaseTriggersTarget);
    install_schema_effector_trigger(s, schemaFillEspressoBaseTriggersEffector);
    install_schema_subgoal(s, "goalPickUpSource", goalTrue, goalHoldFullJug);
    install_schema_subgoal(s, "goalPourSomeIntoTarget", goalHoldFullJug, goalEspressoBaseFilled);
    install_schema_subgoal(s, "goalPutDownSource", goalEspressoBaseFilled, goalJugOnTableAndEspressoBaseFull);
    install_schema_scope(s, SOURCE);
    install_schema_scope(s, TARGET);

    s = install_schema(cs, "schemaAssembleEspressoFilter", "goalAssembleEspressoFilter", ACTION_NONE, MEDIUM);
    install_schema_effector_trigger(s, schemaAssembleEspressoFilterTriggersEffector);
    install_schema_subgoal(s, "goalPositionEspressoFilter", goalTrue, goalEspressoFilterPositioned);
    install_schema_subgoal(s, "goalAddGroundsToFilter", goalEspressoFilterPositioned, goalEspressoFilterAssembled);

    s = install_schema(cs, "schemaPositionEspressoFilter", "goalPositionEspressoFilter", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaPositionEspressoFilterTriggersTheme);
    install_schema_object_trigger(s, TARGET, schemaPositionEspressoFilterTriggersTarget);
    install_schema_effector_trigger(s, schemaPositionEspressoFilterTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalHoldEspressoFilter);
    install_schema_subgoal(s, "goalCoverTarget", goalHoldEspressoFilter, goalEspressoFilterPositioned);
    install_schema_scope(s, THEME);
    install_schema_scope(s, TARGET);

    s = install_schema(cs, "schemaAddGroundsToFilter", "goalAddGroundsToFilter", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaAddGroundsToFilterTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaAddGroundsToFilterTriggersTarget);
    install_schema_object_trigger(s, IMPLEMENT, schemaAddGroundsToFilterTriggersImplement);
    install_schema_effector_trigger(s, schemaAddGroundsToFilterTriggersEffector);
    install_schema_subgoal(s, "goalPickUpImplement", goalTrue, goalHoldEspressoImplement);
    install_schema_subgoal(s, "goalDipScoopIntoSource", goalHoldEspressoImplement, goalHoldFullImplement);
    install_schema_subgoal(s, "goalEmptyScoopIntoTarget", goalHoldFullImplement, goalEspressoFilterFull);
    install_schema_subgoal(s, "goalPutDown", goalEspressoFilterAssembled, goalImplementNotHeld);
    install_schema_scope(s, IMPLEMENT);
    install_schema_scope(s, SOURCE);
    install_schema_scope(s, TARGET);

    s = install_schema(cs, "schemaAssembleEspressoTop", "goalAssembleEspressoTop", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaAssembleEspressoTopTriggersTheme);
    install_schema_object_trigger(s, TARGET, schemaAssembleEspressoTopTriggersTarget);
    install_schema_effector_trigger(s, schemaAssembleEspressoTopTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTarget", goalTrue, goalHoldEspressoBase);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalHoldEspressoTop);
    install_schema_subgoal(s, "goalClose", goalHoldEspressoBaseAndTop, goalEspressoPotAssembled);
    install_schema_subgoal(s, "goalPutDownTarget", goalEspressoPotAssembled, goalAssembledEspressoPotOnTable);
    install_schema_scope(s, THEME);
    install_schema_scope(s, TARGET);

    /* Intermediate2 level schemas: */

    s = install_schema(cs, "schemaPickUpSource", "goalPickUpSource", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaPickUpSourceTriggersSource);
    install_schema_effector_trigger(s, schemaPickUpSourceTriggersEffector);
    install_schema_subgoal(s, "goalFixateSource", goalTrue, goalCurrentSourceFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentSourceFixated, goalCurrentSourceHeld);

    s = install_schema(cs, "schemaPickUpTarget", "goalPickUpTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaPickUpTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaPickUpTargetTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentTargetFixated, goalCurrentTargetHeld);

    s = install_schema(cs, "schemaPickUpTheme", "goalPickUpTheme", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaPickUpThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaPickUpThemeTriggersEffector);
    install_schema_subgoal(s, "goalFixateTheme", goalTrue, goalCurrentThemeFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentThemeFixated, goalCurrentThemeHeld);

    s = install_schema(cs, "schemaPickUpImplement", "goalPickUpImplement", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpImplementTriggersEffector);
    install_schema_subgoal(s, "goalFixateImplement", goalTrue, goalCurrentImplementFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentImplementFixated, goalCurrentImplementHeld);

    s = install_schema(cs, "schemaPourSomeIntoTarget", "goalPourSomeIntoTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaPourSomeIntoTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaPourSomeIntoTargetTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalPourSome", goalCurrentTargetFixated, goalCurrentTargetFull);

    s = install_schema(cs, "schemaPutDownSource", "goalPutDownSource", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaPutDownSourceTriggersSource);
    install_schema_effector_trigger(s, schemaPutDownSourceTriggersEffector);
    install_schema_subgoal(s, "goalPutDown", goalTrue, goalCurrentSourceOnTable);

    s = install_schema(cs, "schemaPutDownTarget", "goalPutDownTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaPutDownTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaPutDownTargetTriggersEffector);
    install_schema_subgoal(s, "goalPutDown", goalTrue, goalCurrentTargetOnTable);

    s = install_schema(cs, "schemaCoverTarget", "goalCoverTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaCoverTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaCoverTargetTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalCover", goalCurrentTargetFixated, goalCurrentTargetCovered);

    s = install_schema(cs, "schemaDipScoopIntoSource", "goalDipScoopIntoSource", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaDipScoopIntoSourceTriggersSource);
    install_schema_effector_trigger(s, schemaDipScoopIntoSourceTriggersEffector);
    install_schema_subgoal(s, "goalFixateSource", goalTrue, goalCurrentSourceFixated);
    install_schema_subgoal(s, "goalScoop", goalCurrentSourceFixated, goalHoldFullImplement);

    s = install_schema(cs, "schemaEmptyScoopIntoTarget", "goalEmptyScoopIntoTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaEmptyScoopIntoTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaEmptyScoopIntoTargetTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalEmptyScoop", goalCurrentTargetFixated, goalEmptyScoopHeld);

    /* Low level schemas: */

    s = install_schema(cs, "schemaFixateSource", "goalFixateSource", ACTION_FIXATE_SOURCE, LOW);
    install_schema_object_trigger(s, SOURCE, schemaFixateSourceTriggersSource);
    install_schema_effector_trigger(s, schemaFixateSourceTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTarget", "goalFixateTarget", ACTION_FIXATE_TARGET, LOW);
    install_schema_object_trigger(s, TARGET, schemaFixateTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaFixateTargetTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateImplement", "goalFixateImplement", ACTION_FIXATE_IMPLEMENT, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaFixateImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaFixateImplementTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTheme", "goalFixateTheme", ACTION_FIXATE_THEME, LOW);
    install_schema_object_trigger(s, THEME, schemaFixateThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaFixateThemeTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPickUp", "goalPickUp", ACTION_PICK_UP, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPickUpTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPickUpTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPickUpTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPutDown", "goalPutDown", ACTION_PUT_DOWN, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPutDownTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPutDownTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPutDownTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPutDownTriggersImplement);
    install_schema_effector_trigger(s, schemaPutDownTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaCover", "goalCover", ACTION_COVER, LOW);
    install_schema_object_trigger(s, THEME, schemaCoverTriggersTheme);
    install_schema_object_trigger(s, TARGET, schemaCoverTriggersTarget);
    install_schema_effector_trigger(s, schemaCoverTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaScoop", "goalScoop", ACTION_SCOOP, LOW);
    install_schema_object_trigger(s, SOURCE, schemaScoopTriggersSource);
    install_schema_object_trigger(s, IMPLEMENT, schemaScoopTriggersImplement);
    install_schema_effector_trigger(s, schemaScoopTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPourSome", "goalPourSome", ACTION_POUR_SOME, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPourSomeTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPourSomeTriggersTarget);
    install_schema_effector_trigger(s, schemaPourSomeTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaEmptyScoop", "goalEmptyScoop", ACTION_EMPTY_SCOOP, LOW);
    install_schema_object_trigger(s, TARGET, schemaEmptyScoopTriggersTarget);
    install_schema_object_trigger(s, IMPLEMENT, schemaEmptyScoopTriggersImplement);
    install_schema_effector_trigger(s, schemaEmptyScoopTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaScrewClosed", "goalClose", ACTION_SCREW_POT_CLOSED, LOW);
    install_schema_object_trigger(s, THEME, schemaScrewPotClosedTriggersTheme);
    install_schema_object_trigger(s, TARGET, schemaScrewPotClosedTriggersTarget);
    install_schema_effector_trigger(s, schemaScrewPotClosedTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaScrewOpen", "goalOpen", ACTION_SCREW_POT_OPEN, LOW);
    install_schema_object_trigger(s, THEME, schemaScrewPotOpenTriggersTheme);
    install_schema_object_trigger(s, TARGET, schemaScrewPotOpenTriggersTarget);
    install_schema_effector_trigger(s, schemaScrewPotOpenTriggersEffector);
    install_schema_resource(s, HAND);
}

/******************************************************************************/
