#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "lib_string.h"

#define INPUT_FILE "dvs_noise.dat"

#define MAX_LINE_LENGTH 1024

char buffer[MAX_LINE_LENGTH];

typedef enum boolean {FALSE, TRUE} Boolean; 

#include <glib.h>


#define VARS 22
#define ROWS 10
#define COLS 20
 
double dr_data[11] = {0.00, 12.50,  4.75, 0.00, 5.00, 1.75, 0.00, 0.75, 0.00, 0.25, 0.00};
double fg_data[11] = {0.00, 31.25, 10.50, 0.75, 1.25, 2.50, 5.75, 1.50, 0.75, 5.75, 2.50};

char *table_headings[VARS];
double table[VARS][ROWS][COLS];
int current_table;
int current_row;

// A simple program to read the matrices of data generated by the express
// client and find the best fit to the actual data.

/*----------------------------------------------------------------------------*/

static void global_vars_initialise()
{
    int i, j, k;

    current_table = -1;
    for (i = 0; i < VARS; i++) {
        table_headings[i] = NULL;
        for (j = 0; j < ROWS; j++) {
            for (k = 0; k < COLS; k++) {
                table[i][j][k] = 0.0;
            }
        }
    }
}

static void global_vars_destroy()
{
    while (current_table > 0) {
        free(table_headings[current_table]);
        table_headings[current_table] = NULL;
        current_table--;
    }
}

/*----------------------------------------------------------------------------*/

static Boolean express_data_line_is_table_heading(char *buffer, char **variable)
{
    int i = strlen("Tabulating ");

    if (strncmp(buffer, "Tabulating ", i) == 0) {
        int j = 0;
        while ((i + j < MAX_LINE_LENGTH) && (buffer[i+j] != ' ')) {
            j++;
        }
        *variable = string_new(j+1);
        strncpy(*variable, &buffer[i], j);
	return(TRUE);
    }
    else {
        return(FALSE);
    }
}

static Boolean express_data_line_is_table_column_headings(char *buffer)
{
    return((buffer[0] == '\t') && (buffer[1] == '\t'));
}

static int express_data_line_is_table_row_data(char *buffer, double *iv)
{
    // Each row of data starts with either a IV's value (in single quotes)
    // or the word "overall" (without quotes). For the time being we are
    // only concerned with the former type of row.

    if (buffer[0] == '\'') {
        int j = 0;
        // Line begins with a single quote ... find the matching close quote:
        while ((j+1 < MAX_LINE_LENGTH) && (buffer[j+1] != '\'')) {
            j++;
        }
        if (buffer[j+1] == '\'') {
            *iv = atof(&buffer[1]);
            return(j+2);
        }
    }
    return(-1);
}

/*----------------------------------------------------------------------------*/

static void express_data_line_install_column_heading(char *buffer, int cv, int cc)
{
    if (cc < COLS) {
        table[cv][0][cc] = atof(buffer);
    }
    else {
        fprintf(stdout, "Error: COLS is not big enough\n");
        abort();
    }
}

static void express_data_line_install_column_headings(char *buffer)
{
    // We have a set of column headings to be installed in cells
    // [current_table][0][i] for i = 1 to COLS-1

    int i = 1, j = 3;

    do {
        express_data_line_install_column_heading(&buffer[j], current_table, i++);
        while ((buffer[j] != '\0') && (isdigit(buffer[j]) || (buffer[j] == '.'))) {
            j++;
        }
        if (buffer[j] == '\'') { j++; }
        while (buffer[j] == '\t') { j++; }
        if (buffer[j] == '\'') { j++; }
        if (!isdigit(buffer[j])) { return; }
    } while (TRUE);
}

/*----------------------------------------------------------------------------*/

static Boolean express_data_line_install_cell(char *buffer, int cv, int cr, int cc)
{
    if (isdigit(buffer[0])) {
        table[cv][cr][cc] = atof(buffer);
        return(TRUE);
    }
    else {
        return(FALSE);
    }
}

static void express_data_line_install_row_data(char *buffer, double iv)
{
    // We have a row of data in cells
    // [current_table][current_row][i] for i = 1 to COLS-1
    // Note that the first element should be within

    int col = 1, j = 0;

    table[current_table][current_row][0] = iv;

    while ((buffer[j] == '\t') || (buffer[j] == ' ')) { j++; }
    while ((col < COLS) && express_data_line_install_cell(&buffer[j], current_table, current_row, col)) {
        while (isdigit(buffer[j])) { j++; }
        if (buffer[j] == '.') { j++; }
        while (isdigit(buffer[j])) { j++; }
        while ((buffer[j] == '\t') || (buffer[j] == ' ')) { j++; }
        col++;
    }
}

/*----------------------------------------------------------------------------*/

static void express_data_line_add_to_table(char *buffer)
{
    char *variable;
    double iv;
    int j;

    if (express_data_line_is_table_heading(buffer, &variable)) {
        if (++current_table < VARS) {
            table_headings[current_table] = variable;
            current_row = 0;
        }
        else {
            fprintf(stdout, "Error: VARS is not big enough\n");
            abort();
        }
    }
    else if (express_data_line_is_table_column_headings(buffer)) {
        // We have a set of column headings .. install in cells
        // [current_table][0][i] for i = 1 to COLS-1
        // Note: current_row should be zero when we get here
        express_data_line_install_column_headings(buffer);
        current_row = 0;
    }
    else if ((j = express_data_line_is_table_row_data(buffer, &iv)) > 0) {
        if (current_row++ >= ROWS) {
            fprintf(stdout, "Error: ROWS is not big enough\n");
            abort();
        }
        else {
            express_data_line_install_row_data(&buffer[j], iv);
        }
    }
}

static void express_data_file_read(char *filename)
{
    FILE *fp = fopen(filename, "r");
    if (fp == NULL) {
        fprintf(stderr, "Cannot read input from %s\n", filename);
        abort();
    }

    while (fgets(buffer, MAX_LINE_LENGTH, fp) != NULL) {
        if (strlen(buffer) == (MAX_LINE_LENGTH-1)) {
            fprintf(stderr, "Error: Line is longer than MAX_LINE_LENGTH\n");
            abort();
        }
        else {
            express_data_line_add_to_table(buffer);
        }
    }
    fclose(fp);
}

/******************************************************************************/

static int create_new_table(char *label)
{
    if (current_table < VARS) {
        int i, j;

        current_table++;
        table_headings[current_table] = string_copy(label);
        for (i = 0; i < ROWS; i++) {
            for (j = 0; j < COLS; j++) {
                if ((i == 0) || (j == 0)) {
                    table[current_table][i][j] = table[0][i][j];
                }
                else {
                    table[current_table][i][j] = 0.0;
                }
            }
        }
        return(current_table);
    }
    else {
        return(-1);
    }
}
    
/******************************************************************************/

static void discard_table(int table)
{
    /* Discards the table data if it is the most recent one */
    if (current_table == table) {
        free(table_headings[current_table]);
        current_table--;
    }
}

/******************************************************************************/

static double ssd(double d1, double d2)
{
    return((d1 - d2) * (d1 - d2));
}

static double wssd(double d1, double d2)
{
    if (d1 != 0.0) {
        return(((d1 - d2) * (d1 - d2)) / d1);
    }
    else {
        return(((d1 - d2) * (d1 - d2)) / 0.01);
    }
}

static void populate_fit_table(int i, double *data, double func())
{
    int j, k, var;

    for (j = 1; j < ROWS; j++) {
        for (k = 1; k < COLS; k++) {
            for (var = 1; var < 11; var++) {
                table[i][j][k] += func(data[var], table[var][j][k]);
            }
        }
    }
}

static int calculate_fit(char *label, double *data, double func())
{
    // Create a new table reflecting the sum square differences for the
    // patient data 
    int j;

    if ((j = create_new_table(label)) > 0) {
        populate_fit_table(j, data, func);
    }
    return(j);
}

/******************************************************************************/

static void ff_print_table(FILE *fp, int i)
{
    int j, k;

    if (table_headings[i] != NULL) {
        fprintf(fp, "\n%s:\n", table_headings[i]);
        for (j = 0; j < ROWS; j++) {
            for (k = 0; k < COLS; k++) {
                fprintf(fp, "\t%6.3f", (table[i][j][k] > 999.999) ? 999.999 : table[i][j][k]);
            }
            fprintf(fp, "\n");
        }
    }
}

/******************************************************************************/
/* Utility function for printing the tables (mainly for debugging): ***********/

void ff_print_tables(FILE *fp)
{
    int i;

    for (i = 0; i < VARS; i++) {
        if (table_headings[i] != NULL) {
            ff_print_table(fp, i);
	}
    }
}

/******************************************************************************/

static void scan_table_for_fit(FILE *fp, char *label, int i)
{
    double best;
    int j, k, x, y;

    x = 1; y = 1;
    best = table[i][x][y];

    for (j = 1; j < ROWS; j++) {
        for (k = 1; k < COLS; k++) {
            if (table[i][j][k] < best) {
                best = table[i][j][k];
                x = j; y = k;
            }
        }
    }
    fprintf(fp, "Best %s fit is %f at row = %f, column = %f\n", label, best, table[i][x][0], table[i][0][y]);
}

/******************************************************************************/

static void find_fit_with_func(char *label, double *data, char *type, double func())
{
    int j;

    j = calculate_fit(label, data, func);
    ff_print_table(stdout, j);
    scan_table_for_fit(stdout, type, j);
    discard_table(j);
}

static void find_fit(char *label, double *data)
{
    find_fit_with_func(label, data, "ssd", ssd);
    find_fit_with_func(label, data, "wssd", wssd);
}

/******************************************************************************/

int main(int argc, char **argv)
{
    global_vars_initialise();
    express_data_file_read(INPUT_FILE);
    find_fit("Patient DR", dr_data);
    find_fit("Patient FG", fg_data);
    global_vars_destroy();
    exit(0);
}

/******************************************************************************/
