/*******************************************************************************

    FILE: juice/goal.c
    AUTHOR: R. Cooper
    CREATION DATE: Fri Jul 30 14:03:24 2004
    CONTENTS: Goal functions for the juice task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

GoalType goalCurrentThemeCut(ContentionScheduler *cs)
{
    return((cs->current_object[THEME] != NULL) ? isOrangePiece(cs->current_object[THEME]) : GOAL_FALSE);
}

GoalType goalCurrentThemeOnTable(ContentionScheduler *cs)
{
    return((cs->current_object[THEME] != NULL) ? FZNOT(isHeld(cs->current_object[THEME])) : GOAL_FALSE);
}

GoalType goalCurrentImplementOnTable(ContentionScheduler *cs)
{
    return((cs->current_object[IMPLEMENT] != NULL) ? FZNOT(isHeld(cs->current_object[IMPLEMENT])) : GOAL_FALSE);
}

GoalType goalOrangeCut(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isOrangePiece(os->head));
    }
    return(found);
}

GoalType goalOrangeJuiced(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isJuicedOrangePiece(os->head));
    }
    return(found);
}

GoalType goalJuicerFull(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isFullJuicer(os->head));
    }
    return(found);
}

GoalType goalJuicerEmpty(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isEmptyJuicer(os->head));
    }
    return(found);
}

GoalType goalJuiceGlassFull(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isFullJuiceGlass(os->head));
    }
    return(found);
}

GoalType goalOrangePieceJuicedAndNotHeld(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isJuicedOrangePiece(os->head), FZNOT(isHeld(os->head))));
    }
    return(found);
}

/*----------------------------------------------------------------------------*/

GoalType goalKnifeOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalOnTable(cs, isKnifeBlade), goalOnTable(cs, isKnifeHandle)));
}

GoalType goalFullJuiceGlassOnTable(ContentionScheduler *cs)
{
    return(goalOnTable(cs, isFullJuiceGlass));
}


GoalType goalEmptyJuicerOnTable(ContentionScheduler *cs)
{
    return(goalOnTable(cs, isEmptyJuicer));
}

/*----------------------------------------------------------------------------*/

GoalType goalKnifeHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isKnifeHandle));
}

GoalType goalOrangePieceHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isOrangePiece));
}

GoalType goalJuicedOrangePieceHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isJuicedOrangePiece));
}

GoalType goalJuicerHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isJuicer));
}

GoalType goalEmptyJuicerHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isEmptyJuicer));
}

GoalType goalFullJuicerHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isFullJuicer));
}

GoalType goalJuiceGlassHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isJuiceGlass));
}

GoalType goalFullJuiceGlassHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isFullJuiceGlass));
}

GoalType goalEmptyContainerHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isEmptyContainer));
}

/*----------------------------------------------------------------------------*/

GoalType goalKnifeFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isKnife));
}

GoalType goalJuicerFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isJuicer));
}

GoalType goalJuiceGlassFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isJuiceGlass));
}

GoalType goalOrangeFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isOrange));
}

GoalType goalOrangePieceFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isOrangePiece));
}

/*----------------------------------------------------------------------------*/

GoalType goalFullJuicerHeldAndJuiceGlassFixated(ContentionScheduler *cs)
{
    return(FZAND(goalFullJuicerHeld(cs), goalJuiceGlassFixated(cs)));
}

GoalType goalKnifeHeldAndOrangeFixated(ContentionScheduler *cs)
{
    return(FZAND(goalKnifeHeld(cs), goalOrangeFixated(cs)));
}

GoalType goalJuicerEmptyAndJuiceGlassFull(ContentionScheduler *cs)
{
    return(FZAND(goalJuicerEmpty(cs), goalJuiceGlassFull(cs)));
}

GoalType goalOrangeCutAndKnifeOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalOrangeCut(cs), goalKnifeOnTable(cs)));
}

GoalType goalOrangePieceHeldAndJuicerFixated(ContentionScheduler *cs)
{
    return(FZAND(goalOrangePieceHeld(cs), goalJuicerFixated(cs)));
}

GoalType goalEmptyJuicerOnTableAndJuiceGlassFull(ContentionScheduler *cs)
{
    return(FZAND(goalEmptyJuicerOnTable(cs), goalJuiceGlassFull(cs)));
}

/*----------------------------------------------------------------------------*/

GoalType goalBallFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isSphere));
}

GoalType goalBallHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isSphere));
}

GoalType goalBallHeldAndTargetFixated(ContentionScheduler *cs)
{
    return(FZAND(goalBallHeld(cs), goalCurrentTargetFixated(cs)));
}

GoalType goalBallBounced(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isSphere(os->head), wasBounced(os->head)));
    }
    return(found);
}

GoalType goalBouncedBallOnTable(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(FZNOT(isHeld(os->head)), FZAND(isSphere(os->head), wasBounced(os->head))));
    }
    return(found);
}

/*----------------------------------------------------------------------------*/

GoalType goalJuiceConsumed(ContentionScheduler *cs)
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isContainer(os->head), isUsed(os->head)));
    }
    return(found);
}

GoalType goalJuiceConsumedAndSourceOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalJuiceConsumed(cs), goalCurrentSourceOnTable(cs)));
}

GoalType goalOrangeCutAndHandsEmpty(ContentionScheduler *cs)
{
    return(FZAND(goalOrangeCut(cs), goalHandsEmpty(cs)));
}

GoalType goalOrangeJuicedAndHandsEmpty(ContentionScheduler *cs)
{
    return(FZAND(goalOrangeJuiced(cs), goalHandsEmpty(cs)));
}

/******************************************************************************/
