/*******************************************************************************

    FILE: juice/object.c
    AUTHOR: R. Cooper
    CREATION DATE: Fri Jul 30 14:03:13 2004
    CONTENTS: Object-related routines for the juice task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

extern double feature_overlap(int af, int bf);

/******************************************************************************/

double isKnifeHandle(Object *o)
{
    return(feature_overlap(o->features, IS_TOOL | IS_EXTENDED | IS_HANDLE));
}

double isKnifeBlade(Object *o)
{
    return(feature_overlap(o->features, IS_TOOL | IS_SHARP | IS_EXTENDED | IS_METAL));
}

double isKnife(Object *o)
{
    return(FZOR(isKnifeHandle(o), isKnifeBlade(o)));
}

double isOrange(Object *o)
{
    return(feature_overlap(o->features, IS_SPHERE | IS_MEDIUM | IS_ORANGE_COLOURED));
}

double isOrangePiece(Object *o)
{
    return(feature_overlap(o->features, IS_HEMISPHERE | IS_MEDIUM | IS_ORANGE_COLOURED));
}

double isJuicedOrangePiece(Object *o)
{
    return(FZAND(isOrangePiece(o), isSqueezed(o)));
}

double isJuiceGlass(Object *o)
{
    return(feature_overlap(o->features, IS_CONTAINER | IS_CYLINDRICAL | IS_GLASS));
}

double isEmptyJuiceGlass(Object *o)
{
    return(FZAND(isJuiceGlass(o), isEmptyObject(o)));
}

double isFullJuiceGlass(Object *o)
{
    return(FZAND(isJuiceGlass(o), isFullObject(o)));
}

double isJuicer(Object *o)
{
    return(feature_overlap(o->features, IS_CONTAINER | IS_MEDIUM | IS_TOOL));
}

double isEmptyJuicer(Object *o)
{
    return(FZAND(isJuicer(o), isEmptyObject(o)));
}

double isFullJuicer(Object *o)
{
    return(FZAND(isJuicer(o), isFullObject(o)));
}

/******************************************************************************/

double isSqueezed(Object *o)
{
    return((o->state & STATE_USED) ? 1.0 : 0.0);
}

double isCut(Object *o)
{
    return((o->state & STATE_CUT) ? 1.0 : 0.0);
}

double isSoft(Object *o)
{
    return(GOAL_FALSE);
}

double isFirm(Object *o)
{
    return(isOrange(o));
}

/******************************************************************************/

void write_juice_object_attributes(FILE *fp, Object *o)
{
    if (o->state & STATE_OPEN) { 
        fprintf(fp, "open, ");
    }
    else if (o->state & STATE_CLOSED) { 
        fprintf(fp, "closed, ");
    }

    if (o->state & STATE_CUT) { 
        fprintf(fp, "cut, ");
    }

    if (o->state & STATE_USED) { 
        fprintf(fp, "squeezed, ");
    }

    if (o->itemsAttached) {
        fprintf(fp, "has attachments, ");
    }
}

/******************************************************************************/

void install_juice_basic_objects(ContentionScheduler *cs)
{
    Object *object;

    object = objectInstall(cs, "knife_handle", IS_TOOL | IS_EXTENDED | IS_HANDLE, 0);
    objectSetPosition(object, -1, 2);
    objectInstallAttachment(object, objectInstall(cs, "knife_blade", IS_TOOL | IS_SHARP | IS_EXTENDED | IS_METAL, 0));
    object = objectInstall(cs, "orange", IS_SPHERE | IS_MEDIUM | IS_ORANGE_COLOURED, 0);
    objectSetPosition(object, 1, 2);

    object = objectInstall(cs, "juicer", IS_CONTAINER | IS_MEDIUM | IS_TOOL, STATE_OPEN);
    objectSetPosition(object, -2, 2);
    objectInstallAttachment(object, objectInstall(cs, "filter", IS_FLAT | IS_TOOL | IS_DISK | IS_MEDIUM, 0));

    object = objectInstall(cs, "glass", IS_CONTAINER | IS_CYLINDRICAL | IS_GLASS, STATE_OPEN);
    objectSetPosition(object, 2, 2);
    object = objectInstall(cs, "table", 0, 0);
}

/******************************************************************************/
