/*******************************************************************************

    FILE: juice/schema.c
    AUTHOR: R. Cooper
    CREATION DATE: Fri Jul 30 14:03:39 2004
    CONTENTS: Schemas for the juice task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

void install_juice_schemas(ContentionScheduler *cs)
{
    Schema *s;

    s = install_schema(cs, "schemaPrepareJuice", "goalPrepareJuice", ACTION_NONE, HIGH);
    install_schema_object_trigger(s, SOURCE, schemaPrepareJuiceTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPrepareJuiceTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPrepareJuiceTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPrepareJuiceTriggersImplement);
    install_schema_effector_trigger(s, schemaPrepareJuiceTriggersEffector);
    install_schema_subgoal(s, "goalCutOrange", goalTrue, goalOrangeCutAndHandsEmpty);
    install_schema_subgoal(s, "goalJuiceOrange", goalOrangeCut, goalOrangeJuicedAndHandsEmpty);
    install_schema_subgoal(s, "goalFillJuiceGlass", goalOrangeJuiced, goalEmptyJuicerOnTableAndJuiceGlassFull);
    install_schema_scope(s, SOURCE);

    cs->intended_schema = s;

    /* Intermediate level schemas: */

    s = install_schema(cs, "schemaCutOrange", "goalCutOrange", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaCutOrangeTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaCutOrangeTriggersImplement);
    install_schema_effector_trigger(s, schemaCutOrangeTriggersEffector);
    install_schema_subgoal(s, "goalPickUpImplement", goalTrue, goalKnifeHeld);
    install_schema_subgoal(s, "goalCutTheme", goalKnifeHeld, goalOrangeCut);
    install_schema_subgoal(s, "goalPutDownImplement", goalOrangeCut, goalOrangeCutAndKnifeOnTable);
    install_schema_scope(s, IMPLEMENT);

    s = install_schema(cs, "schemaJuiceOrangeWithJuicer", "goalJuiceOrange", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaJuiceOrangeWithJuicerTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaJuiceOrangeWithJuicerTriggersImplement);
    install_schema_effector_trigger(s, schemaJuiceOrangeWithJuicerTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalOrangePieceHeld);
    install_schema_subgoal(s, "goalSqueezeWithImplement", goalOrangePieceHeld, goalJuicedOrangePieceHeld);
    install_schema_subgoal(s, "goalPutDownTheme", goalJuicedOrangePieceHeld, goalOrangePieceJuicedAndNotHeld);
    install_schema_scope(s, THEME);

    s = install_schema(cs, "schemaJuiceOrangeByHand", "goalJuiceOrange", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaJuiceOrangeByHandTriggersTheme);
    install_schema_object_trigger(s, TARGET, schemaJuiceOrangeByHandTriggersTarget);
    install_schema_effector_trigger(s, schemaJuiceOrangeByHandTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalOrangePieceHeld);
    install_schema_subgoal(s, "goalSqueezeOverGlass", goalOrangePieceHeld, goalJuicedOrangePieceHeld);
    install_schema_subgoal(s, "goalPutDownTheme", goalJuicedOrangePieceHeld, goalOrangePieceJuicedAndNotHeld);
    install_schema_scope(s, THEME);

    s = install_schema(cs, "schemaFillJuiceGlass", "goalFillJuiceGlass", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaFillGlassTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaFillGlassTriggersTarget);
    install_schema_effector_trigger(s, schemaFillGlassTriggersEffector);
    install_schema_subgoal(s, "goalPickUpSource", goalTrue, goalJuicerHeld);
    install_schema_subgoal(s, "goalPourAllIntoTarget", goalFullJuicerHeld, goalEmptyJuicerHeld);
    install_schema_subgoal(s, "goalPutDownSource", goalEmptyJuicerHeld, goalEmptyJuicerOnTable);
    install_schema_scope(s, THEME);

    s = install_schema(cs, "schemaBounceBall", "goalBounceBall", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaBounceBallTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaBounceBallTriggersTheme);
    install_schema_effector_trigger(s, schemaBounceBallTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalBallHeld);
    install_schema_subgoal(s, "goalBounceOnTarget", goalBallHeld, goalBallBounced);
    install_schema_subgoal(s, "goalPutDownTheme", goalBallBounced,goalBouncedBallOnTable);
    install_schema_scope(s, TARGET);
    install_schema_scope(s, THEME);

    s = install_schema(cs, "schemaDrinkJuice", "goalDrinkFromGlass", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaDrinkJuiceTriggersSource);
    install_schema_effector_trigger(s, schemaDrinkJuiceTriggersEffector);
    install_schema_subgoal(s, "goalPickUpSource", goalTrue, goalCurrentSourceHeld);
    install_schema_subgoal(s, "goalDrink", goalCurrentSourceHeld, goalJuiceConsumed);
    install_schema_subgoal(s, "goalPutDownSource", goalJuiceConsumed, goalJuiceConsumedAndSourceOnTable);
    install_schema_scope(s, SOURCE);

    /* Intermediate2 level schemas: */

    s = install_schema(cs, "schemaPickUpSource", "goalPickUpSource", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaPickUpSourceTriggersSource);
    install_schema_effector_trigger(s, schemaPickUpSourceTriggersEffector);
    install_schema_subgoal(s, "goalFixateSource", goalTrue, goalCurrentSourceFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentSourceFixated, goalCurrentSourceHeld);

    s = install_schema(cs, "schemaPickUpImplement", "goalPickUpImplement", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpImplementTriggersEffector);
    install_schema_subgoal(s, "goalFixateImplement", goalTrue, goalCurrentImplementFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentImplementFixated, goalCurrentImplementHeld);

    s = install_schema(cs, "schemaPickUpTheme", "goalPickUpTheme", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaPickUpThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaPickUpThemeTriggersEffector);
    install_schema_subgoal(s, "goalFixateTheme", goalTrue, goalCurrentThemeFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentThemeFixated, goalCurrentThemeHeld);

    s = install_schema(cs, "schemaPutDownSource", "goalPutDownSource", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaPutDownSourceTriggersSource);
    install_schema_effector_trigger(s, schemaPutDownSourceTriggersEffector);
    install_schema_subgoal(s, "goalPutDown", goalTrue, goalCurrentSourceOnTable);

    s = install_schema(cs, "schemaPutDownImplement", "goalPutDownImplement", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, IMPLEMENT, schemaPutDownImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaPutDownImplementTriggersEffector);
    install_schema_subgoal(s, "goalPutDown", goalTrue, goalCurrentImplementOnTable);

    s = install_schema(cs, "schemaPutDownTheme", "goalPutDownTheme", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaPutDownThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaPutDownThemeTriggersEffector);
    install_schema_subgoal(s, "goalPutDown", goalTrue, goalCurrentThemeOnTable);

    s = install_schema(cs, "schemaCutTheme", "goalCutTheme", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaCutThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaCutThemeTriggersEffector);
    install_schema_subgoal(s, "goalFixateTheme", goalTrue, goalCurrentThemeFixated);
    install_schema_subgoal(s, "goalCut", goalCurrentThemeFixated, goalCurrentThemeCut);

    s = install_schema(cs, "schemaSqueezeWithImplement", "goalSqueezeWithImplement", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, IMPLEMENT, schemaSqueezeWithImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaSqueezeWithImplementTriggersEffector);
    install_schema_subgoal(s, "goalFixateImplement", goalTrue, goalCurrentImplementFixated);
    install_schema_subgoal(s, "goalSqueezeWithSqueezer", goalCurrentImplementFixated, goalJuicedOrangePieceHeld);

    s = install_schema(cs, "schemaPourAllIntoTarget", "goalPourAllIntoTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaPourAllIntoTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaPourAllIntoTargetTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalPourAll", goalCurrentTargetFixated, goalEmptyContainerHeld);

    s = install_schema(cs, "schemaBounceOnTarget", "goalBounceOnTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaBounceOnTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaBounceOnTargetTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalBounce", goalCurrentTargetFixated, goalBallBounced);

    s = install_schema(cs, "schemaSqueezeOverGlass", "goalSqueezeOverGlass", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaSqueezeOverGlassTriggersTarget);
    install_schema_effector_trigger(s, schemaSqueezeOverGlassTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalSqueezeByHand", goalCurrentTargetFixated, goalJuicedOrangePieceHeld);

    /* Low level schemas: */

    s = install_schema(cs, "schemaFixateSource", "goalFixateSource", ACTION_FIXATE_SOURCE, LOW);
    install_schema_object_trigger(s, SOURCE, schemaFixateSourceTriggersSource);
    install_schema_effector_trigger(s, schemaFixateSourceTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTarget", "goalFixateTarget", ACTION_FIXATE_TARGET, LOW);
    install_schema_object_trigger(s, TARGET, schemaFixateTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaFixateTargetTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateImplement", "goalFixateImplement", ACTION_FIXATE_IMPLEMENT, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaFixateImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaFixateImplementTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTheme", "goalFixateTheme", ACTION_FIXATE_THEME, LOW);
    install_schema_object_trigger(s, THEME, schemaFixateThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaFixateThemeTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPickUp", "goalPickUp", ACTION_PICK_UP, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPickUpTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPickUpTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPickUpTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPutDown", "goalPutDown", ACTION_PUT_DOWN, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPutDownTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPutDownTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPutDownTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPutDownTriggersImplement);
    install_schema_effector_trigger(s, schemaPutDownTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaBounce", "goalBounce", ACTION_BOUNCE_BALL, LOW);
    install_schema_object_trigger(s, THEME, schemaBounceTriggersTheme);
    install_schema_effector_trigger(s, schemaBounceTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaCutBySawing", "goalCut", ACTION_CUT_BY_SAWING, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaCutBySawingTriggersImplement);
    install_schema_object_trigger(s, THEME, schemaCutBySawingTriggersTheme);
    install_schema_effector_trigger(s, schemaCutBySawingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaCutByPushing", "goalCut", ACTION_CUT_BY_PUSHING, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaCutByPushingTriggersImplement);
    install_schema_object_trigger(s, THEME, schemaCutByPushingTriggersTheme);
    install_schema_effector_trigger(s, schemaCutByPushingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPourAll", "goalPourAll", ACTION_POUR_ALL, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPourAllTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPourAllTriggersTarget);
    install_schema_effector_trigger(s, schemaPourAllTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaSqueezeByTwisting", "goalSqueezeWithSqueezer", ACTION_SQUEEZE_BY_TWISTING, LOW);
    install_schema_object_trigger(s, THEME, schemaSqueezeByTwistingTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaSqueezeByTwistingTriggersImplement);
    install_schema_effector_trigger(s, schemaSqueezeByTwistingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaSqueezeByPushing", "goalSqueezeWithSqueezer", ACTION_SQUEEZE_BY_TWISTING, LOW);
    install_schema_object_trigger(s, THEME, schemaSqueezeByPushingTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaSqueezeByPushingTriggersImplement);
    install_schema_effector_trigger(s, schemaSqueezeByPushingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaSqueezeByHand", "goalSqueezeByHand", ACTION_SQUEEZE_BY_HAND, LOW);
    install_schema_object_trigger(s, THEME, schemaSqueezeByHandTriggersTheme);
    install_schema_object_trigger(s, TARGET, schemaSqueezeByHandTriggersTarget);
    install_schema_effector_trigger(s, schemaSqueezeByHandTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaDrink", "goalDrink", ACTION_DRINK, LOW);
    install_schema_object_trigger(s, SOURCE, schemaDrinkTriggersSource);
    install_schema_effector_trigger(s, schemaDrinkTriggersEffector);
    install_schema_resource(s, HAND);

    /* Further distractor schema: fixate on the table? */

}

/******************************************************************************/
