/*******************************************************************************

    FILE: juice/schema.c
    AUTHOR: R. Cooper
    CREATION DATE: Fri Jul 30 14:03:39 2004
    CONTENTS: Schemas for the juice task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

void install_juice_schemas(ContentionScheduler *cs)
{
    Schema *s;

    s = install_schema(cs, "schemaPrepareJuice", "goalPrepareJuice", ACTION_NONE, HIGH);
    install_schema_object_trigger(s, SOURCE, schemaPrepareJuiceTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPrepareJuiceTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPrepareJuiceTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPrepareJuiceTriggersImplement);
    install_schema_effector_trigger(s, schemaPrepareJuiceTriggersEffector);
    install_schema_subgoal(s, "goalCutOrange", goalTrue, goalOrangeCut);
    install_schema_subgoal(s, "goalJuiceOrange", goalOrangeCut, goalJuicerFull);
    install_schema_subgoal(s, "goalFillJuiceGlass", goalOrangeJuiced, goalEmptyJuicerOnTableAndJuiceGlassFull);

    install_schema_scope(s, SOURCE);

    cs->intended_schema = s;

    /* Intermediate level schemas: */

    s = install_schema(cs, "schemaCutOrange", "goalCutOrange", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaCutOrangeTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaCutOrangeTriggersImplement);
    install_schema_effector_trigger(s, schemaCutOrangeTriggersEffector);
    install_schema_subgoal(s, "goalFixateImplement", goalTrue, goalKnifeFixated);
    install_schema_subgoal(s, "goalPickUp", goalKnifeFixated, goalKnifeHeld);
    install_schema_subgoal(s, "goalFixateTheme", goalKnifeHeld, goalKnifeHeldAndOrangeFixated);
    install_schema_subgoal(s, "goalCut", goalKnifeHeldAndOrangeFixated, goalOrangeCut);
    install_schema_subgoal(s, "goalPutDown", goalOrangeCut, goalOrangeCutAndKnifeOnTable);
    install_schema_scope(s, IMPLEMENT);

    s = install_schema(cs, "schemaJuiceOrange", "goalJuiceOrange", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaJuiceOrangeTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaJuiceOrangeTriggersImplement);
    install_schema_effector_trigger(s, schemaJuiceOrangeTriggersEffector);
    install_schema_subgoal(s, "goalFixateTheme", goalTrue, goalOrangePieceFixated);
    install_schema_subgoal(s, "goalPickUp", goalOrangePieceFixated, goalOrangePieceHeld);
    install_schema_subgoal(s, "goalFixateImplement", goalOrangePieceHeld, goalOrangePieceHeldAndJuicerFixated);
    install_schema_subgoal(s, "goalSqueezeWithSqueezer", goalOrangePieceHeldAndJuicerFixated, goalJuicedOrangePieceHeld);
    install_schema_subgoal(s, "goalPutDown", goalJuicedOrangePieceHeld, goalOrangePieceJuicedAndNotHeld);
    install_schema_scope(s, THEME);

    s = install_schema(cs, "schemaFillJuiceGlass", "goalFillJuiceGlass", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, SOURCE, schemaFillGlassTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaFillGlassTriggersTarget);
    install_schema_effector_trigger(s, schemaFillGlassTriggersEffector);
    install_schema_subgoal(s, "goalFixateSource", goalTrue, goalJuicerFixated);
    install_schema_subgoal(s, "goalPickUp", goalJuicerFixated, goalJuicerHeld);
    install_schema_subgoal(s, "goalFixateTarget", goalFullJuicerHeld, goalFullJuicerHeldAndJuiceGlassFixated);
    install_schema_subgoal(s, "goalPourAll", goalFullJuicerHeldAndJuiceGlassFixated, goalEmptyJuicerHeld);
    install_schema_subgoal(s, "goalPutDown", goalEmptyJuicerHeld, goalEmptyJuicerOnTable);
//    install_schema_subgoal(s, "goalPutDown", goalFullJuiceGlassHeld, goalFullJuiceGlassOnTable);
    install_schema_scope(s, THEME);

    s = install_schema(cs, "schemaBounceBall", "goalBounceBall", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaBounceBallTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaBounceBallTriggersTheme);
    install_schema_effector_trigger(s, schemaBounceBallTriggersEffector);
    install_schema_subgoal(s, "goalFixateTheme", goalTrue, goalBallFixated);
    install_schema_subgoal(s, "goalPickUp", goalBallFixated, goalBallHeld);
    install_schema_subgoal(s, "goalFixateTarget", goalBallHeld, goalBallHeldAndTargetFixated);
    install_schema_subgoal(s, "goalBounce", goalBallHeldAndTargetFixated, goalBallBounced);
    install_schema_scope(s, TARGET);
    install_schema_scope(s, THEME);

    /* Low level schemas: */

    s = install_schema(cs, "schemaFixateSource", "goalFixateSource", ACTION_FIXATE_SOURCE, LOW);
    install_schema_object_trigger(s, SOURCE, schemaFixateSourceTriggersSource);
    install_schema_effector_trigger(s, schemaFixateSourceTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTarget", "goalFixateTarget", ACTION_FIXATE_TARGET, LOW);
    install_schema_object_trigger(s, TARGET, schemaFixateTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaFixateTargetTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateImplement", "goalFixateImplement", ACTION_FIXATE_IMPLEMENT, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaFixateImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaFixateImplementTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTheme", "goalFixateTheme", ACTION_FIXATE_THEME, LOW);
    install_schema_object_trigger(s, THEME, schemaFixateThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaFixateThemeTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPickUp", "goalPickUp", ACTION_PICK_UP, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPickUpTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPickUpTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPickUpTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPutDown", "goalPutDown", ACTION_PUT_DOWN, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPutDownTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPutDownTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPutDownTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPutDownTriggersImplement);
    install_schema_effector_trigger(s, schemaPutDownTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaBounce", "goalBounce", ACTION_BOUNCE_BALL, LOW);
    install_schema_object_trigger(s, THEME, schemaBounceTriggersTheme);
    install_schema_effector_trigger(s, schemaBounceTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaCutBySawing", "goalCut", ACTION_CUT_BY_SAWING, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaCutBySawingTriggersImplement);
    install_schema_object_trigger(s, THEME, schemaCutBySawingTriggersTheme);
    install_schema_effector_trigger(s, schemaCutBySawingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaCutByPushing", "goalCut", ACTION_CUT_BY_PUSHING, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaCutByPushingTriggersImplement);
    install_schema_object_trigger(s, THEME, schemaCutByPushingTriggersTheme);
    install_schema_effector_trigger(s, schemaCutByPushingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPourAll", "goalPourAll", ACTION_POUR_ALL, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPourAllTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPourAllTriggersTarget);
    install_schema_effector_trigger(s, schemaPourAllTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaSqueezeByTwisting", "goalSqueezeWithSqueezer", ACTION_SQUEEZE_BY_TWISTING, LOW);
    install_schema_object_trigger(s, THEME, schemaSqueezeByTwistingTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaSqueezeByTwistingTriggersImplement);
    install_schema_effector_trigger(s, schemaSqueezeByTwistingTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaSqueezeByPushing", "goalSqueezeWithSqueezer", ACTION_SQUEEZE_BY_TWISTING, LOW);
    install_schema_object_trigger(s, THEME, schemaSqueezeByPushingTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaSqueezeByPushingTriggersImplement);
    install_schema_effector_trigger(s, schemaSqueezeByPushingTriggersEffector);
    install_schema_resource(s, HAND);

    /* Further distractor schema: fixate on the table */

}

/******************************************************************************/
