/*******************************************************************************

    FILE: juice/triggers.c
    AUTHOR: R. Cooper
    CREATION DATE: Fri Jul 30 14:03:24 2004
    CONTENTS: Schema triggering functions for the juice task

*******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

extern double holds(Effector *e, double f(Object *object));

/******************************************************************************/

double schemaPrepareJuiceTriggersSource(ContentionScheduler *cs, Object *object)
{
    return(isFullJuicer(object));
}

double schemaPrepareJuiceTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(restAct);
}

double schemaPrepareJuiceTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isOrange(object));
}

double schemaPrepareJuiceTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return(restAct);
}

double schemaPrepareJuiceTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

/*----------------------------------------------------------------------------*/

double schemaCutOrangeTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isOrange(object));
}

double schemaCutOrangeTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return(isKnifeHandle(object));
}

double schemaCutOrangeTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isKnifeHandle));
}

/*----------------------------------------------------------------------------*/

double schemaJuiceOrangeWithJuicerTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isOrangePiece(object));
}

double schemaJuiceOrangeWithJuicerTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return(isJuicer(object));
}

double schemaJuiceOrangeWithJuicerTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isOrangePiece));
}

/*----------------------------------------------------------------------------*/

double schemaJuiceOrangeByHandTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isOrangePiece(object) / 2.0);
}

double schemaJuiceOrangeByHandTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(isJuiceGlass(object) / 2.0);
}

double schemaJuiceOrangeByHandTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isOrangePiece));
}

/*----------------------------------------------------------------------------*/

double schemaFillGlassTriggersSource(ContentionScheduler *cs, Object *object)
{
    return(isFullJuicer(object));
}

double schemaFillGlassTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(isJuiceGlass(object));
}

double schemaFillGlassTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

/*----------------------------------------------------------------------------*/

double schemaCutBySawingTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return(isKnifeHandle(object));
}

double schemaCutBySawingTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isFirm(object));
}

double schemaCutBySawingTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isKnifeHandle));
}

/*----------------------------------------------------------------------------*/

double schemaCutByPushingTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return(isKnifeHandle(object));
}

double schemaCutByPushingTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isSoft(object));
}

double schemaCutByPushingTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isKnifeHandle));
}

/*----------------------------------------------------------------------------*/

double schemaPourAllTriggersSource(ContentionScheduler *cs, Object *object)
{
    return(isFullJuicer(object));
}

double schemaPourAllTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(FZOR(isJuiceGlass(object), isTable(object)));
}

double schemaPourAllTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(FZOR(holds(e, isFullJuicer), holds(e, isJuiceGlass)));
}

/*----------------------------------------------------------------------------*/

double schemaSqueezeByTwistingTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isOrangePiece(object));
}

double schemaSqueezeByTwistingTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return(isJuicer(object));
}

double schemaSqueezeByTwistingTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isOrangePiece));
}

/*----------------------------------------------------------------------------*/

double schemaSqueezeByPushingTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isOrangePiece(object));
}

double schemaSqueezeByPushingTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return(isJuicer(object) / 2.0);
}

double schemaSqueezeByPushingTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isOrangePiece));
}

/*----------------------------------------------------------------------------*/

double schemaSqueezeByHandTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isOrangePiece(object));
}

double schemaSqueezeByHandTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(isJuiceGlass(object));
}

double schemaSqueezeByHandTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isOrangePiece));
}

/*----------------------------------------------------------------------------*/

double schemaBounceBallTriggersTarget(ContentionScheduler *cs, Object *o)
{
    return(isTable(o));
}

double schemaBounceBallTriggersTheme(ContentionScheduler *cs, Object *o)
{
    return(isSphere(o));
}

double schemaBounceBallTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isSphere));
}

/*----------------------------------------------------------------------------*/

double schemaBounceTriggersTheme(ContentionScheduler *cs, Object *o)
{
    return(FZAND(isSphere(o), isHeld(o)));
}

double schemaBounceTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isSphere));
}

/******************************************************************************/

double schemaCutThemeTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isOrange(object));
}

double schemaCutThemeTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

double schemaSqueezeWithImplementTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return(isJuicer(object));
}

double schemaSqueezeWithImplementTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

double schemaPourAllIntoTargetTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(FZOR(isJuiceGlass(object), isTable(object)));
}

double schemaPourAllIntoTargetTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(restAct);
}

double schemaBounceOnTargetTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(isTable(object));
}

double schemaBounceOnTargetTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isSphere));
}

double schemaSqueezeOverGlassTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(isJuiceGlass(object));
}

double schemaSqueezeOverGlassTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isOrangePiece));
}

double schemaDrinkJuiceTriggersSource(ContentionScheduler *cs, Object *o)
{
    return(isFullObject(o));
}

double schemaDrinkJuiceTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(holds(e, isFullObject));
}

/******************************************************************************/
