#include "../cs.h"
#include "task.h"

GoalType goalTargetAddressed(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isAddressed(os->head));
    }
    return(found);
}

GoalType goalCurrentTargetAddressed(ContentionScheduler *cs)
{
    if (cs->current_object[TARGET] == NULL) {
        return(GOAL_FALSE);
    }
    else {
        return(isAddressed(cs->current_object[TARGET]));
    }
}

GoalType goalEnvelopeStuffed(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isEnvelope(os->head), isStuffed(os->head)));
    }
    return(found);
}

GoalType goalEnvelopeLicked(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isEnvelope(os->head), isMoist(os->head)));
    }
    return(found);
}

GoalType goalEnvelopeOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalOnTable(cs, isEnvelopeBack), goalOnTable(cs, isEnvelopeFront)));
}

GoalType goalFoldedLetterOnTable(ContentionScheduler *cs)
{
    return(goalOnTable(cs, isFoldedLetter));
}

GoalType goalEnvelopeSealed(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isEnvelope(os->head), isSealed(os->head)));
    }
    return(found);
}

GoalType goalStampApplied(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, isStamped(os->head));
    }
    return(found);
}

GoalType goalSealedEnvelopeOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalEnvelopeSealed(cs), FZAND(goalOnTable(cs, isEnvelopeBack), goalOnTable(cs, isEnvelopeFront))));
}

GoalType goalStuffedEnvelopeOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalEnvelopeStuffed(cs), FZAND(goalOnTable(cs, isEnvelopeBack), goalOnTable(cs, isEnvelopeFront))));
}

GoalType goalEnvelopeHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isEnvelope));
}

GoalType goalLetterHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isLetter));
}

GoalType goalPenHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isPen));
}

GoalType goalStampHeld(ContentionScheduler *cs)
{
    return(goalHold(cs, isStamp));
}

GoalType goalLetterHeldAndEnvelopeHeld(ContentionScheduler *cs)
{
    return(FZAND(goalHold(cs, isLetter), goalHold(cs, isEnvelope)));
}

GoalType goalLetterFolded(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isLetter(os->head), isFolded(os->head)));
    }
    return(found);
}

GoalType goalLetterFoldedAndEnvelopeHeld(ContentionScheduler *cs)
{
    ObjectList *os;
    double found1 = 0.0, found2 = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found1 = FZOR(found1, FZAND(isLetter(os->head), isFolded(os->head)));
	found2 = FZOR(found2, FZAND(isEnvelope(os->head), isHeld(os->head)));
    }
    return(FZAND(found1, found2));
}

GoalType goalPenOnTable(ContentionScheduler *cs)
{
    return(goalOnTable(cs, isPen));
}

GoalType goalStampLicked(ContentionScheduler *cs)
{
    ObjectList *os;
    double found = 0.0;

    for (os = cs->objects; os != NULL; os = os->tail) {
	found = FZOR(found, FZAND(isStamp(os->head), isMoist(os->head)));
    }
    return(found);
}

GoalType goalTargetAddressedAndPenOnTable(ContentionScheduler *cs)
{
    return(FZAND(goalTargetAddressed(cs), goalPenOnTable(cs)));
}

GoalType goalPenFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isPen));
}

GoalType goalLetterFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isLetter));
}

GoalType goalPaperFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isPaper));
}

GoalType goalEnvelopeFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isEnvelope));
}

GoalType goalStampFixated(ContentionScheduler *cs)
{
    return(goalFixated(cs, isStamp));
}

GoalType goalPenHeldAndAddressTargetFixated(ContentionScheduler *cs)
{
    return(FZAND(goalPenHeld(cs), goalPaperFixated(cs)));
}

GoalType goalStampLickedAndStampTargetFixated(ContentionScheduler *cs)
{
    return(FZAND(goalStampLicked(cs), goalPaperFixated(cs)));
}

GoalType goalLetterFoldedAndEnvelopeFixated(ContentionScheduler *cs)
{
    return(FZAND(goalLetterFolded(cs), goalEnvelopeFixated(cs)));
}

/******************************************************************************/
