/*******************************************************************************

    FILE: letter_object.c
    AUTHOR: R. Cooper
    CREATION DATE: 
    CONTENTS: 

*******************************************************************************/

#include "../cs.h"
#include "task.h"

extern double feature_overlap(int af, int bf);

/******************************************************************************/

double isStampFront(Object *this)
{
    return(feature_overlap(this->features, IS_FLAT | IS_SQUARE | IS_SMALL | IS_FRONT));
}

double isEnvelopeFront(Object *this)
{
    return(feature_overlap(this->features, IS_CONTAINER | IS_FLAT | IS_SQUARE | IS_MEDIUM | IS_FRONT));
}

double isStampBack(Object *this)
{
    return(feature_overlap(this->features, IS_FLAT | IS_SQUARE | IS_SMALL | IS_BACK));
}

double isEnvelopeBack(Object *this)
{
    return(feature_overlap(this->features, IS_CONTAINER | IS_FLAT | IS_SQUARE | IS_MEDIUM | IS_BACK));
}

double isStamp(Object *this)
{
    return(FZOR(isStampBack(this), isStampFront(this)));
}

double isEnvelope(Object *this)
{
    return(FZOR(isEnvelopeBack(this), isEnvelopeFront(this)));
}

double isLetter(Object *this)
{
    return(feature_overlap(this->features, IS_FLAT | IS_SQUARE | IS_MEDIUM));
}

double isPen(Object *this)
{
    return(FZOR(isPenNib(this), isPenStalk(this)));
}

double isPenNib(Object *this)
{
    return(feature_overlap(this->features, IS_TOOL | IS_MEDIUM | IS_EXTENDED | IS_SHARP));
}

double isPenStalk(Object *this)
{
    return(feature_overlap(this->features, IS_TOOL | IS_MEDIUM | IS_EXTENDED | IS_BLUNT));
}

double isPaper(Object *o)
{
    return(FZOR(isLetter(o), isEnvelope(o)));
}

double isEnvelopeBackOrStampBack(Object *o)
{
    return(FZOR(isEnvelopeBack(o), isStampBack(o)));
}

double isStamped(Object *this)
{
    return((this->state & STATE_STAMPED) ? 1.0 : 0.0);
}

double isFolded(Object *this)
{
    return((this->state & STATE_FOLDED) ? 1.0 : 0.0);
}

double isMoist(Object *this)
{
    return((this->state & STATE_MOIST) ? 1.0 : 0.0);
}

double isAddressed(Object *this)
{
    return((this->state & STATE_ADDRESSED) ? 1.0 : 0.0);
}

double isStuffed(Object *this)
{
    return(isFullObject(this));
}

double isSealed(Object *this)
{
    return(isClosed(this));
}

double isFoldedLetter(Object *this)
{
    return(FZAND(isFolded(this), isLetter(this)));
}

double isUnsealedEnvelope(Object *this)
{
    return(FZAND(isEnvelope(this), FZNOT(isSealed(this))));
}

/******************************************************************************/

void write_letter_object_attributes(FILE *fp, Object *this)
{
    if (this->state & STATE_OPEN) { 
        fprintf(fp, "open, ");
    }
    else if (this->state & STATE_CLOSED) { 
        fprintf(fp, "closed, ");
    }

    if (this->state & STATE_ADDRESSED) { 
        fprintf(fp, "addressed, ");
    }

    if (this->state & STATE_FOLDED) { 
        fprintf(fp, "folded, ");
    }

    if (this->state & STATE_MOIST) { 
        fprintf(fp, "moist, ");
    }

    if (this->state & STATE_STAMPED) { 
        fprintf(fp, "stamped, ");
    }

    if (this->itemsAttached) {
        fprintf(fp, "has attachments, ");
    }
}

/******************************************************************************/

void install_letter_basic_objects(ContentionScheduler *cs)
{
    Object *object;

    objectInstall(cs, "notePaper", IS_FLAT | IS_SQUARE | IS_MEDIUM, 0);
    object = objectInstall(cs, "pen_nib", IS_TOOL | IS_MEDIUM | IS_EXTENDED | IS_SHARP, 0);
    objectInstallAttachment(object, objectInstall(cs, "pen_stalk", IS_TOOL | IS_MEDIUM | IS_EXTENDED | IS_BLUNT, 0));
    object = objectInstall(cs, "stamp_front", IS_FLAT | IS_SQUARE | IS_SMALL | IS_FRONT, 0);
    objectInstallAttachment(object, objectInstall(cs, "stamp_back",  IS_FLAT | IS_SQUARE | IS_SMALL | IS_BACK, 0));
    object = objectInstall(cs, "envelope_front", IS_CONTAINER | IS_FLAT | IS_SQUARE | IS_MEDIUM | IS_FRONT, STATE_OPEN);
    objectInstallAttachment(object, objectInstall(cs, "envelope_back",  IS_CONTAINER | IS_FLAT | IS_SQUARE | IS_MEDIUM | IS_BACK, STATE_OPEN));
}

/******************************************************************************/
