/******************************************************************************/

#include "../cs.h"
#include "task.h"

/******************************************************************************/

void install_letter_schemas(ContentionScheduler *cs)
{
    Schema *s;

    s = install_schema(cs, "schemaPrepareLetter", "goalPrepareLetter", ACTION_NONE, HIGH);
    install_schema_object_trigger(s, TARGET, schemaPrepareLetterTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPrepareLetterTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPrepareLetterTriggersImplement);
    install_schema_effector_trigger(s, schemaPrepareLetterTriggersEffector);
    install_schema_subgoal(s, "goalAddressEnvelope", goalTrue, goalTargetAddressedAndPenOnTable);
    install_schema_subgoal(s, "goalApplyStamp", goalTrue, goalStampApplied);
    install_schema_subgoal(s, "goalFoldLetter", goalTrue, goalLetterFolded);
    install_schema_subgoal(s, "goalInsertLetter", goalLetterFolded, goalEnvelopeStuffed);
    install_schema_subgoal(s, "goalSealEnvelope", goalEnvelopeStuffed, goalSealedEnvelopeOnTable);

    cs->intended_schema = s;

    /* Intermediate level schemas: */

    s = install_schema(cs, "schemaAddressEnvelope", "goalAddressEnvelope", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaAddressEnvelopeTriggersTarget);
    install_schema_object_trigger(s, IMPLEMENT, schemaAddressEnvelopeTriggersImplement);
    install_schema_effector_trigger(s, schemaAddressEnvelopeTriggersEffector);
    install_schema_subgoal(s, "goalPickUpImplement", goalTrue, goalPenHeld);
    install_schema_subgoal(s, "goalAddressTarget", goalPenHeld, goalTargetAddressed);
    install_schema_subgoal(s, "goalPutDownImplement", goalTargetAddressed, goalTargetAddressedAndPenOnTable);
    install_schema_scope(s, TARGET);
    install_schema_scope(s, IMPLEMENT);

    s = install_schema(cs, "schemaApplyStamp", "goalApplyStamp", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaApplyStampTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaApplyStampTriggersTheme);
    install_schema_effector_trigger(s, schemaApplyStampTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalStampHeld);
    install_schema_subgoal(s, "goalLick", goalStampHeld, goalStampLicked);
    install_schema_subgoal(s, "goalStampTarget", goalStampLicked, goalStampApplied);
    install_schema_scope(s, TARGET);
    install_schema_scope(s, THEME);

    s = install_schema(cs, "schemaFoldLetter", "goalFoldLetter", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaFoldLetterTriggersTheme);
    install_schema_effector_trigger(s, schemaFoldLetterTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalLetterHeld);
    install_schema_subgoal(s, "goalFold", goalLetterHeld, goalLetterFolded);
    install_schema_subgoal(s, "goalPutDownTheme", goalLetterFolded, goalFoldedLetterOnTable);
    install_schema_scope(s, THEME);

    s = install_schema(cs, "schemaInsertLetter", "goalInsertLetter", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaInsertLetterTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaInsertLetterTriggersTheme);
    install_schema_effector_trigger(s, schemaInsertLetterTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalLetterHeld);
    install_schema_subgoal(s, "goalPickUpTarget", goalTrue, goalEnvelopeHeld);
    install_schema_subgoal(s, "goalPackEnvelope", goalLetterHeldAndEnvelopeHeld, goalEnvelopeStuffed);
    install_schema_subgoal(s, "goalPutDownTarget", goalEnvelopeStuffed, goalStuffedEnvelopeOnTable);
    install_schema_scope(s, TARGET);
    install_schema_scope(s, THEME);

    s = install_schema(cs, "schemaSealEnvelope", "goalSealEnvelope", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaSealEnvelopeTriggersTheme);
    install_schema_effector_trigger(s, schemaSealEnvelopeTriggersEffector);
    install_schema_subgoal(s, "goalPickUpTheme", goalTrue, goalEnvelopeHeld);
    install_schema_subgoal(s, "goalSealHeldEnvelope", goalEnvelopeHeld, goalEnvelopeSealed);
    install_schema_subgoal(s, "goalPutDownTheme", goalEnvelopeSealed, goalSealedEnvelopeOnTable);
    install_schema_scope(s, THEME);

    /* Intermediate2 level schemas: */

    s = install_schema(cs, "schemaPickUpTarget", "goalPickUpTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaPickUpTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaPickUpTargetTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentTargetFixated, goalCurrentTargetHeld);

    s = install_schema(cs, "schemaPickUpTheme", "goalPickUpTheme", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaPickUpThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaPickUpThemeTriggersEffector);
    install_schema_subgoal(s, "goalFixateTheme", goalTrue, goalCurrentThemeFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentThemeFixated, goalCurrentThemeHeld);

    s = install_schema(cs, "schemaPickUpImplement", "goalPickUpImplement", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpImplementTriggersEffector);
    install_schema_subgoal(s, "goalFixateImplement", goalTrue, goalCurrentImplementFixated);
    install_schema_subgoal(s, "goalPickUp", goalCurrentImplementFixated, goalCurrentImplementHeld);

    s = install_schema(cs, "schemaPutDownImplement", "goalPutDownImplement", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, IMPLEMENT, schemaPutDownImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaPutDownImplementTriggersEffector);
    install_schema_subgoal(s, "goalPutDown", goalTrue, goalCurrentImplementOnTable);

    s = install_schema(cs, "schemaPutDownTheme", "goalPutDownTheme", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaPutDownThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaPutDownThemeTriggersEffector);
    install_schema_subgoal(s, "goalPutDown", goalTrue, goalCurrentThemeOnTable);

    s = install_schema(cs, "schemaAddressTarget", "goalAddressTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaAddressTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaAddressTargetTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalTrue, goalCurrentTargetFixated);
    install_schema_subgoal(s, "goalWriteAddress", goalCurrentTargetFixated, goalCurrentTargetAddressed);

    s = install_schema(cs, "schemaStampTarget", "goalStampTarget", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, TARGET, schemaStampTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaStampTargetTriggersEffector);
    install_schema_subgoal(s, "goalFixateTarget", goalStampLicked, goalStampLickedAndStampTargetFixated);
    install_schema_subgoal(s, "goalPlaceStamp", goalStampLickedAndStampTargetFixated, goalStampApplied);

    s = install_schema(cs, "schemaLickAndSeal", "goalSealHeldEnvelope", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaLickAndSealTriggersTheme);
    install_schema_effector_trigger(s, schemaLickAndSealTriggersEffector);
    install_schema_subgoal(s, "goalLick", goalEnvelopeHeld, goalEnvelopeLicked);
    install_schema_subgoal(s, "goalSealIt", goalEnvelopeLicked, goalEnvelopeSealed);

    s = install_schema(cs, "schemaTuckFlap", "goalCloseHeldEnvelope", ACTION_NONE, MEDIUM);
    install_schema_object_trigger(s, THEME, schemaTuckFlapTriggersTheme);
    install_schema_effector_trigger(s, schemaTuckFlapTriggersEffector);
    install_schema_subgoal(s, "goalTuck", goalEnvelopeHeld, goalEnvelopeSealed);

    /* Low level schemas: */

    s = install_schema(cs, "schemaFixateSource", "goalFixateSource", ACTION_FIXATE_SOURCE, LOW);
    install_schema_object_trigger(s, SOURCE, schemaFixateSourceTriggersSource);
    install_schema_effector_trigger(s, schemaFixateSourceTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTarget", "goalFixateTarget", ACTION_FIXATE_TARGET, LOW);
    install_schema_object_trigger(s, TARGET, schemaFixateTargetTriggersTarget);
    install_schema_effector_trigger(s, schemaFixateTargetTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateImplement", "goalFixateImplement", ACTION_FIXATE_IMPLEMENT, LOW);
    install_schema_object_trigger(s, IMPLEMENT, schemaFixateImplementTriggersImplement);
    install_schema_effector_trigger(s, schemaFixateImplementTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFixateTheme", "goalFixateTheme", ACTION_FIXATE_THEME, LOW);
    install_schema_object_trigger(s, THEME, schemaFixateThemeTriggersTheme);
    install_schema_effector_trigger(s, schemaFixateThemeTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPickUp", "goalPickUp", ACTION_PICK_UP, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPickUpTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPickUpTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPickUpTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPickUpTriggersImplement);
    install_schema_effector_trigger(s, schemaPickUpTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPutDown", "goalPutDown", ACTION_PUT_DOWN, LOW);
    install_schema_object_trigger(s, SOURCE, schemaPutDownTriggersSource);
    install_schema_object_trigger(s, TARGET, schemaPutDownTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPutDownTriggersTheme);
    install_schema_object_trigger(s, IMPLEMENT, schemaPutDownTriggersImplement);
    install_schema_effector_trigger(s, schemaPutDownTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPlaceStamp", "goalPlaceStamp", ACTION_PLACE_STAMP, LOW);
    install_schema_object_trigger(s, TARGET, schemaPlaceStampTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPlaceStampTriggersTheme);
    install_schema_effector_trigger(s, schemaPlaceStampTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaWriteAddress", "goalWriteAddress", ACTION_WRITE_ADDRESS, LOW);
    install_schema_object_trigger(s, TARGET, schemaWriteAddressTriggersTarget);
    install_schema_object_trigger(s, IMPLEMENT, schemaWriteAddressTriggersImplement);
    install_schema_effector_trigger(s, schemaWriteAddressTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaFold", "goalFold", ACTION_FOLD_LETTER, LOW);
    install_schema_object_trigger(s, THEME, schemaFoldTriggersTheme);
    install_schema_effector_trigger(s, schemaFoldTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaPackEnvelope", "goalPackEnvelope", ACTION_PACK_ENVELOPE, LOW);
    install_schema_object_trigger(s, TARGET, schemaPackEnvelopeTriggersTarget);
    install_schema_object_trigger(s, THEME, schemaPackEnvelopeTriggersTheme);
    install_schema_effector_trigger(s, schemaPackEnvelopeTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaSealIt", "goalSealIt", ACTION_SEAL_ENVELOPE, LOW);
    install_schema_object_trigger(s, TARGET, schemaSealItTriggersTarget);
    install_schema_effector_trigger(s, schemaSealItTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaLick", "goalLick", ACTION_MOISTEN_WITH_TOUNGE, LOW);
    install_schema_object_trigger(s, THEME, schemaLickTriggersTheme);
    install_schema_effector_trigger(s, schemaLickTriggersEffector);
    install_schema_resource(s, HAND);

    s = install_schema(cs, "schemaTuck", "goalTuck", ACTION_TUCK, LOW);
    install_schema_object_trigger(s, THEME, schemaTuckTriggersTheme);
    install_schema_effector_trigger(s, schemaTuckTriggersEffector);
    install_schema_resource(s, HAND);
}

/******************************************************************************/
