
#include "cs.h"
#include "analyse.h"

extern void install_null_task_schemas(ContentionScheduler *cs);
extern void install_null_task_objects(ContentionScheduler *cs);
extern void install_null_task_effectors(ContentionScheduler *cs);
extern void write_null_task_object_attributes(FILE *fp, Object *this);
extern void write_null_task_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_null_task(DataAnalysis *la);

extern void install_letter_schemas(ContentionScheduler *cs);
extern void install_letter_basic_objects(ContentionScheduler *cs);
extern void install_letter_effectors(ContentionScheduler *cs);
extern void write_letter_object_attributes(FILE *fp, Object *this);
extern void write_letter_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_letter_task(DataAnalysis *la);

extern void install_juice_schemas(ContentionScheduler *cs);
extern void install_juice_basic_objects(ContentionScheduler *cs);
extern void install_juice_effectors(ContentionScheduler *cs);
extern void write_juice_object_attributes(FILE *fp, Object *this);
extern void write_juice_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_juice_task(DataAnalysis *la);

extern void install_espresso_schemas(ContentionScheduler *cs);
extern void install_espresso_basic_objects(ContentionScheduler *cs);
extern void install_espresso_effectors(ContentionScheduler *cs);
extern void write_espresso_object_attributes(FILE *fp, Object *this);
extern void write_espresso_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_espresso_task(DataAnalysis *la);

extern void install_candle_schemas(ContentionScheduler *cs);
extern void install_candle_basic_objects(ContentionScheduler *cs);
extern void install_candle_effectors(ContentionScheduler *cs);
extern void write_candle_object_attributes(FILE *fp, Object *this);
extern void write_candle_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_candle_task(DataAnalysis *la);

extern void install_bottle_schemas(ContentionScheduler *cs);
extern void install_bottle_basic_objects(ContentionScheduler *cs);
extern void install_bottle_effectors(ContentionScheduler *cs);
extern void write_bottle_object_attributes(FILE *fp, Object *this);
extern void write_bottle_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_bottle_task(DataAnalysis *la);

extern void install_lunch_schemas(ContentionScheduler *cs);
extern void install_lunch_basic_objects(ContentionScheduler *cs);
extern void install_lunch_distractor_objects(ContentionScheduler *cs);
extern void install_lunch_effectors(ContentionScheduler *cs);
extern void write_lunch_object_attributes(FILE *fp, Object *this);
extern void write_lunch_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_lunch_task(DataAnalysis *la);

char *task_names[TT_MAX] = {
    "None",
    "PrepareLetter",
    "PrepareJuice",
    "PrepareEspresso",
    "LightCandle",
    "DrinkFromCorkedBottle",
    "MakeLunch (no distractors)",
    "MakeLunch (with distractors)",
};

void initialise_task(TaskTypeId id, TaskType *task)
{
    task->id = id;
    switch (task->id) {
        case NULL_TASK: {
            task->install_schemas = install_null_task_schemas;
            task->install_objects = install_null_task_objects;
            task->install_effectors = install_null_task_effectors;
            task->write_object_attributes = write_null_task_object_attributes;
            task->write_effector_attributes = write_null_task_effector_attributes;
            task->analyse = analyse_null_task;
            break;
        }
        case LETTER_BASIC: {
            task->install_schemas = install_letter_schemas;
            task->install_objects = install_letter_basic_objects;
            task->install_effectors = install_letter_effectors;
            task->write_object_attributes = write_letter_object_attributes;
            task->write_effector_attributes = write_letter_effector_attributes;
            task->analyse = analyse_letter_task;
            break;
        }
        case JUICE_BASIC: {
            task->install_schemas = install_juice_schemas;
            task->install_objects = install_juice_basic_objects;
            task->install_effectors = install_juice_effectors;
            task->write_object_attributes = write_juice_object_attributes;
            task->write_effector_attributes = write_juice_effector_attributes;
            task->analyse = analyse_juice_task;
            break;
        }
        case ESPRESSO_BASIC: {
            task->install_schemas = install_espresso_schemas;
            task->install_objects = install_espresso_basic_objects;
            task->install_effectors = install_espresso_effectors;
            task->write_object_attributes = write_espresso_object_attributes;
            task->write_effector_attributes = write_espresso_effector_attributes;
            task->analyse = analyse_espresso_task;
            break;
        }
        case CANDLE_BASIC: {
            task->install_schemas = install_candle_schemas;
            task->install_objects = install_candle_basic_objects;
            task->install_effectors = install_candle_effectors;
            task->write_object_attributes = write_candle_object_attributes;
            task->write_effector_attributes = write_candle_effector_attributes;
            task->analyse = analyse_candle_task;
            break;
        }
        case BOTTLE_BASIC: {
            task->install_schemas = install_bottle_schemas;
            task->install_objects = install_bottle_basic_objects;
            task->install_effectors = install_bottle_effectors;
            task->write_object_attributes = write_bottle_object_attributes;
            task->write_effector_attributes = write_bottle_effector_attributes;
            task->analyse = analyse_bottle_task;
            break;
        }
/*
        case LUNCH_BASIC: {
            task->install_schemas = install_lunch_schemas;
            task->install_objects = install_lunch_basic_objects;
            task->install_effectors = install_lunch_effectors;
            task->write_object_attributes = write_lunch_object_attributes;
            task->write_effector_attributes = write_lunch_effector_attributes;            
            task->analyse = analyse_lunch_task;
            break;
        }
        case LUNCH_DISTRACTOR: {
            task->install_schemas = install_lunch_schemas;
            task->install_objects = install_lunch_distractor_objects;
            task->install_effectors = install_lunch_effectors;
            task->write_object_attributes = write_lunch_object_attributes;
            task->write_effector_attributes = write_lunch_effector_attributes;
            task->analyse = analyse_lunch_task;
            break;
        }
*/
        default: {
            fprintf(stdout, "ERROR: Task definition not found\n");
        }
    }
}
